#ifndef _ECS_CHAMP_DEF_H_
#define _ECS_CHAMP_DEF_H_

/*============================================================================
 *  Prototypes des fonctions
 *   associees a la structure `ecs_champ_t' decrivant un champ
 *   et propres aux champs principaux de type "definition"
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilite
 *============================================================================*/


/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C
 *----------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_param_rc_glob.h"
#include "ecs_def.h"
#include "ecs_tab_glob.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  des paquetages visibles
 *----------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_publ.h"


/*============================================================================
 *                       Prototypes de fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui construit
 *   les dfinitions des faces par dcomposition des champs des cellules
 *----------------------------------------------------------------------------*/

void ecs_champ_def__decompose_cel
(
 ecs_champ_t *vect_champ_fac[],
 ecs_champ_t *champ_def_cel,
 ecs_champ_t *champ_typ_geo_cel
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui construit
 *   les dfinitions des artes par dcomposition des champs des faces
 *----------------------------------------------------------------------------*/

void ecs_champ_def__decompose_fac
(
 ecs_champ_t *vect_champ_are[],
 ecs_champ_t *champ_def_elt
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui realise la fusion des definitions des elements
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_champ_def__int_fusionne
(
 ecs_champ_t   * this_champ_def, /* <-> Reference sur champ des definitions   */
 size_t        * nbr_elt_cpct,
 ecs_tab_int_t * signe_elt       /* <-- Signe des elts apres tri de leur def. */
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui realise la fusion des definitions des elements
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_champ_def__real_fusionne
(
 ecs_champ_t * this_champ_def, /* <-> Reference sur le champ des definitions */
 size_t      * nbr_elt_cpct
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui compte les faces priodiques.
 *----------------------------------------------------------------------------*/

void ecs_champ_def__compte_fac_perio
(
 ecs_champ_t    *const champ_att_fac,
 ecs_tab_int_t  *const nbr_fac_perio
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui construit les listes de faces internes et de bord, ainsi que
 *  les listes de faces avec erreur de connectivit (i.e. qui appartiennent
 *   2 cellules ou plus vu d'un mme ct) et de faces isoles.
 *
 *  Les listes dont les pointeurs fournis en argument sont  NULL ne
 *  sont pas construites. Celles dont les pointeurs sont fournis
 *  sont initialises et alloues si elles ne sont pas vides.
 *----------------------------------------------------------------------------*/

void ecs_champ_def__cree_listes_fac
(
 ecs_champ_t      *const champ_def_fac_cel,
 size_t                  nbr_fac,
 ecs_tab_int_t    *const liste_fac_erreur,
 ecs_tab_int_t    *const liste_fac_interne,
 ecs_tab_int_t    *const liste_fac_de_bord,
 ecs_tab_int_t    *const liste_fac_isolee
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui renvoie la liste des cellules partageant au moins un
 *  mme ct d'une de leurs faces avec une autre cellule.
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_champ_def__liste_cel_err_c
(
       size_t                nbr_cel_tot,
       ecs_champ_t    *const champ_def_fac_cel,
 const ecs_tab_int_t         liste_fac_erreur
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui construit la liste des cellules attaches  une liste
 *  de faces fournie en argument.
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_champ_def__liste_cel_fac
(
 const ecs_int_t            nbr_fac,
       ecs_champ_t   *const champ_def_cel,
 const ecs_tab_int_t        liste_fac
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui construit la table de connectivite "faces -> sommets"
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_def__cree_fac_som
(
 ecs_champ_t       *const champ_def_fac,
 ecs_champ_t       *const champ_def_are
) ;


/*----------------------------------------------------------------------------
 *  Fonction realisant le decoupage des faces en triangles
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_def__dec_fac_tria
(
 ecs_champ_t        *      vect_champ_fac[],
 ecs_champ_t        *      vect_champ_are_new[],
 ecs_champ_t        *const champ_def_are,
 ecs_champ_t        *const champ_def_som,
 ecs_tab_int_t      *const liste_filtre,
 ecs_tab_int_t      *const liste_fac_dec
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui construit un tableau de booleens conforme a une liste
 *   de sous-elements
 *  Un sous-element est a ECS_TRUE
 *   s'il intervient dans la definition des elements
 *----------------------------------------------------------------------------*/

void ecs_champ_def__cree_masque
(
 ecs_tab_bool_t        bool_sselt_select,
 ecs_champ_t    *const champ_def_elt
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui renvoie la liste des faces de type "polygone"
 *  (base sur la dfinition de faces par plus de 4 artes)
 *----------------------------------------------------------------------------*/

ecs_tab_int_t  ecs_champ_def__liste_fac_poly
(
 ecs_champ_t  *const champ_def_fac
) ;


/*----------------------------------------------------------------------------
 *  Suppression des sommets ne participant pas  la connectivit
 * (renvoie le champ de renumrotation des sommets, ou NULL si aucune
 * renumrotation n'a t ncessaire).
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_def__nettoie_nodal
(
 ecs_champ_t       *const champ_def_som,
 ecs_champ_t       *const champ_def_are,
 ecs_champ_t       *const champ_def_fac,
 ecs_champ_t       *const champ_def_cel
) ;


/*----------------------------------------------------------------------------
 *  Correction si ncessaire de l'orientation des lments en
 *   connectivit nodale.
 *----------------------------------------------------------------------------*/

void ecs_champ_def__orient_nodal
(
       ecs_champ_t    *const champ_def_som,   /*  -> Dfinition des sommets   */
       ecs_champ_t    *const champ_def_fac,   /*  -> Dfinition des faces     */
       ecs_champ_t    *const champ_def_cel,   /*  -> Dfinition des cellules  */
       ecs_champ_t    *const champ_typ_cel,   /*  -> Types des cellules       */
       ecs_tab_int_t  *const liste_cel_err,   /* <-  Cels. erreur (option)    */
       ecs_tab_int_t  *const liste_cel_cor,   /* <-  Cels. correct. (option)  */
 const ecs_bool_t            correc_orient    /*  -> Correction ou non        */
) ;


/*----------------------------------------------------------------------------
 *  Fusion des sommets confondus d'aprs la longueur des artes.
 * La connectivit des artes est mise  jour.
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_def__nettoie_som_are
(
 ecs_champ_t       *const champ_def_som,
 ecs_champ_t       *const champ_def_are
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui supprime les ventuelles artes dgnres
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_def__nettoie_are
(
 ecs_champ_t       *const champ_def_are
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui supprime les ventuelles faces dgnres
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_def__nettoie_fac
(
 ecs_champ_t       *const champ_def_fac
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui recolle les faces non conformes
 *
 *  Les listes des faces nouvelles ou modifies sont construites (et alloues)
 *  ici ; les structures liste_fac_new et liste_fac_mod correspondantes sont
 *  donc vides en entre ; idem pour liste_fac_err qui indiquera les indices
 *  des faces pour lesquelles le dcoupage en sous-faces a chou
 *
 *  On prend en entre soit une dfinition de type "visibilit" entre faces
 *   recoller (par exemple une filiation), ou alors une simple liste de
 *  faces slectionnes. L'un de vec_fac_vis et tab_fac_select doit donc
 *  tre  NULL, et l'autre non.
 *----------------------------------------------------------------------------*/

void ecs_champ_def__recolle
(
       ecs_champ_t       *const champ_def_fac,
       ecs_champ_t       *const champ_def_are,
       ecs_champ_t       *const champ_def_som,
       ecs_champ_t     * *const champ_fac_old_new,
       ecs_champ_t     * *const champ_are_old_new,
       ecs_champ_t     * *const champ_som_old_new,
       ecs_champ_t     * *const champ_fac_perio,
       ecs_champ_t       *const champ_fac_vis,
       ecs_tab_int_t     *const tab_fac_de_bord_select,
       ecs_tab_int_t     *const liste_fac_new,
       ecs_tab_int_t     *const liste_fac_mod,
       ecs_tab_int_t     *const liste_fac_err,
 const ecs_param_rc_t           param_rc
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui retourne le vecteur des coordonnes des sommets
 *   pour le champ `ecs_champ_t' associ  des sommets.
 *
 *  La fonction retourne un pointeur sur le dbut du vecteur des coordonnes
 *   en cas de succs, NULL en cas d'chec (si le champ ne correspond
 *   pas  un champ rel de dimension 3).
 *----------------------------------------------------------------------------*/

ecs_real_t * ecs_champ_def__ret_coord_som
(
 const ecs_champ_t  *const champ_coord
) ;


#endif /* _ECS_CHAMP_DEF_H_ */
