#ifndef _ECS_RENUM_PRIV_H_
#define _ECS_RENUM_PRIV_H_

/*============================================================================
 *  Dfinition prive de la structure `ecs_renum_t' dcrivant une
 *  connectivit por le noyau du Code_Saturne avec
 *  rnumerotation vectorielle ou parallle
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilit
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C
 *----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  des paquetages visibles
 *----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_renum_publ.h"


/*============================================================================
 *                       Dfinition de macros
 *============================================================================*/


/*============================================================================
 *                       Dfinition de la structure
 *============================================================================*/

struct _ecs_renum_t {

  int             nbr_dom             ; /* Nombre de domaines de dcoupage    */

  int             nbr_per             ; /* Nombre de priodicits             */
  int             nbr_per_rot         ; /* Nombre de priodicits de rotation */

  ecs_size_t      nbr_cel             ; /* Nombre total de cellules           */
  ecs_size_t    * nbr_cel_loc         ; /* Nombre de cellules par domaine     */

  size_t          nbr_fac_interne     ; /* Nombre total de faces internes     */
  size_t          nbr_fac_de_bord     ; /* Nombre total de faces de bord      */

  size_t          nbr_couples_perio   ; /* Nbr. couples de faces priodiques  */
  ecs_size_t    * nbr_fac_interne_loc ; /* Nbr. de faces internes par domaine */
  ecs_size_t    * nbr_fac_de_bord_loc ; /* Nbr. de faces de bord par domaine  */
  ecs_size_t    * nbr_fac_par_loc     ; /* Nombre de faces sur frontire      */
                                        /* parallle par domaine              */

  size_t          nbr_som             ; /* Nombre total de sommets            */
  ecs_size_t    * nbr_som_loc         ; /* Nombre de sommets par domaine      */

  ecs_int_t     * val_connect_fac_interne_cel ;
  ecs_int_t     * val_connect_fac_de_bord_cel ;
  ecs_int_t   * * val_connect_fac_interne_cel_loc ;
  ecs_int_t   * * val_connect_fac_de_bord_cel_loc ;

  ecs_int_t     * liste_fac_interne       ;
  ecs_int_t     * liste_fac_de_bord       ;

  ecs_int_t     * connect_fac_perio       ; /* Pour chaque couple de faces    */
                                            /* priodiques : [i, j, num_per]  */
                                            /* avec signe num_per selon       */
                                            /* priodicit normale ou inverse */

  ecs_int_t   * * idx_fac_perio_loc       ; /* Index des couples de faces     */
                                            /* priodiques par priodicit    */
                                            /* pour chaque domaine            */
  ecs_int_t   * * liste_fac_perio_loc     ; /* Couples de faces priodiques ; */
                                            /* signe 1re face indique        */
                                            /* priodicit normale ou inverse */
  ecs_int_t   * * num_dom_fac_perio       ; /* Numro du domaine voisin via   */
                                            /* une face frontire priodique  */

  ecs_size_t    * pos_connect_fac_de_bord_som ;
  ecs_size_t    * pos_connect_fac_interne_som ;
  ecs_int_t     * val_connect_fac_de_bord_som ;
  ecs_int_t     * val_connect_fac_interne_som ;
  ecs_size_t  * * pos_connect_fac_de_bord_som_loc ;
  ecs_size_t  * * pos_connect_fac_interne_som_loc ;
  ecs_int_t   * * val_connect_fac_de_bord_som_loc ;
  ecs_int_t   * * val_connect_fac_interne_som_loc ;

  ecs_int_t   * * num_glob_cel        ; /* Tableaux de numros globaux des    */
                                        /* cellules (par domaine)             */
  ecs_int_t   * * num_glob_fac        ; /* Tableaux de numros globaux des    */
                                        /* faces internes (par domaine)       */
  ecs_int_t   * * num_glob_fbr        ; /* Tableaux de numros globaux des    */
                                        /* faces de bord (par domaine)        */

  ecs_int_t   * * num_glob_som        ; /* Tableaux de numros globaux des    */
                                        /* sommets (par domaine)              */

  ecs_int_t     * num_dom_cel         ; /* Numros de domaines des cellules   */

} ;


#endif /* _ECS_RENUM_PRIV_H_ */
