#ifndef _ECS_VEC_GRD_H_
#define _ECS_VEC_GRD_H_

/*============================================================================
 *  Prototypes des fonctions
 *   associes aux structures `ecs_vec_int_t' et `ecs_vec_real_t' dcrivant
 *   les vecteurs indexs entier et rel
 *   et propres aux vecteurs indexs
 *      lis aux champs auxiliaires de type "grandeur"
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilit
 *============================================================================*/


/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C
 *----------------------------------------------------------------------------*/

#include <limits.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"
#include "ecs_tab_glob.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  des paquetages visibles
 *----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_vec_int_publ.h"
#include "ecs_vec_real_publ.h"


/*============================================================================
 *                       Prototypes de fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui calcule les coordonnes minimales et maximales
 *----------------------------------------------------------------------------*/

void ecs_vec_grd__calc_coo_ext
(
 ecs_vec_real_t * vec_som
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui modifie les coordonnes du maillage
 *----------------------------------------------------------------------------*/

void ecs_vec_grd__transf_coo
(
 ecs_vec_real_t  * vec_som,
 const double      matrice[3][4]
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui calcule le maximum associ  une cellule d'un tableau de
 *   valeurs associes aux faces internes.
 *----------------------------------------------------------------------------*/

ecs_tab_real_t ecs_vec_grd__max_fac_int
(
       size_t                   nbr_cel           ,
 const ecs_vec_int_t     *const vec_def_fac_cel   ,
 const ecs_tab_int_t     *const liste_fac_interne ,
 const ecs_tab_real_t    *const val_fac_interne   ,
       ecs_real_t               val_min
) ;


/*----------------------------------------------------------------------------
 *   Calcul de la matrice d'une rotation dfinie par un angle theta (en
 * radians) et un vecteur unitaire direction.
 *   La matrice est calcule via la formule suivante :
 *
 * R = (1-cos(theta))direction.transp(direction) + cos(theta)I + sin(theta)V
 *
 *          [ 0     -direction(3)  direction(2)]
 * avec V = [ direction(3)  0     -direction(1)]
 *          [-direction(2)  direction(1)  0    ]
 *
 * Remarque : le calcul de la matrice ncessite 10 additions et 24 produits.
 *            on pourrait rduire le nombre de multiplications (tout en
 *            augmentant le nombre d'additions) en passant par les
 *            quaternions. On pourrait aussi rduire le nombre de produits
 *             18 en stockant 6 produits dans des variables intermdiares.
 *----------------------------------------------------------------------------*/

void ecs_vec_grd__mat_rot_3d
(
 ecs_real_t  angle,              /* --> Angle de rotation en degres           */
 ecs_real_t  direction[3],       /* --> Vecteur directeur (unitaire) de l'axe */
 ecs_real_t  matrice[3][3]       /* <-- Matrice definissant la rotation       */
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui calcule les nouvelles coordonnees d'un vecteur ou d'un point
 *  dans le cas d'une periodicite.
 *----------------------------------------------------------------------------*/

ecs_vec_real_t * ecs_vec_grd__vec_new_coor_perio
(
 const ecs_vec_real_t    * vec_def             ,
 const ecs_real_t          translation[3]      ,
 const ecs_real_t          point_inv[3]        ,
 const ecs_real_t          matrice[3][3]       ,
 const ecs_bool_t          periodicite_inverse
) ;


#endif /* _ECS_VEC_GRD_H_ */
