#ifndef _ECS_POST_ENS_PRIV_H_
#define _ECS_POST_ENS_PRIV_H_

/*============================================================================
 *  Dfinition de la structure `ecs_post_ens_t' pour post-traitement EnSight
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilit
 *============================================================================*/


/*----------------------------------------------------------------------------*
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <bft_file.h>


/*----------------------------------------------------------------------------*
 *  Fichiers `include' publics  du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"


/*----------------------------------------------------------------------------*
 *  Fichiers `include' publics  des paquetages visibles
 *----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*
 *  Fichiers `include' publics  du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_post_ens_publ.h"


/*============================================================================
 *                         Dfinitions d'numerations
 *============================================================================*/


/*============================================================================
 *                          Dfinitions de types
 *============================================================================*/

/* Structure conservant des informations sur les parts EnSight                */

typedef struct {

  char       *   nom_maillage  ; /* Nom du maillage associ                   */
  char       *   nom_part      ; /* Nom du "part" EnSight                     */
  int            num_part      ; /* Numro du "part"                          */
  ecs_int_t      nbr_som       ; /* Nombre de sommets associs                */
  int            nbr_typ_ele   ; /* Nombre de type d'lments considrs      */
  ecs_int_t  *   nbr_ele_typ   ; /* Nombre d'lments par type                */
  char       * * nom_typ_ele   ; /* Noms des types d'lments                 */
  ecs_int_t    * lst_parents   ; /* Liste des lments parents                */

} ecs_post_ens_part_t ;


/* Structure dfinissant un cas EnSight                                       */

struct _ecs_post_ens_t {

  char                    * nom_cas       ; /* Nom du cas                     */
  char                    * prefixe_rep   ; /* Prfixe du rpertoire EnSight  */
  char                    * prefixe_fic   ; /* Prfixe des fichiers EnSight   */
  char                    * nom_fic_case  ; /* Nom du fichier "case"          */
  int                       nbr_var       ; /* Nombre de variables            */
  char                  * * nom_var       ; /* Noms des variables             */
  char                  * * ligne_var     ; /* Ligne pour une variable        */

  int                       nbr_part      ; /* Nombre de ``parts'' gomtrie  */
  ecs_post_ens_part_t   * * tab_part      ; /* Descripteurs des ``parts''     */

  bft_file_t              * fic_geo       ; /* Pointeur sur fichier gomtrie */

  ecs_bool_t                single        ; /* Ne pas subdiviser un maillage
                                               en plusieurs parts si vrai     */
  ecs_bool_t                modifie       ; /* Modification depuis dernire   */
                                            /* criture du fichier case       */

  ecs_bool_t                text          ; /* Fichiers texte et non binaires */
  ecs_bool_t                big_endian    ; /* Fichiers binaires "big-endian" */

} ;


/*============================================================================
 *                       Dfinition de macros
 *============================================================================*/


#endif /* _ECS_POST_ENS_PRIV_H_ */
