#ifndef _ECS_PRE_H_
#define _ECS_PRE_H_

/*============================================================================
 *  Definition des fonctions de base
 *   de remplissage de la structure de maillage a partir des donnees lues
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilite
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <stdlib.h>
#include <string.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles des paquetages visibles
 *----------------------------------------------------------------------------*/

#include "ecs_maillage_publ.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage courant
 *----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associe au fichier courant
 *----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
 *  Fichiers `include' prives   du  paquetage courant
 *----------------------------------------------------------------------------*/


/*============================================================================
 *                       Dfinition de type numr
 *============================================================================*/

typedef enum {

  ECS_PRE_FORMAT_NUL = -1,
  ECS_PRE_FORMAT_CGNS,
  ECS_PRE_FORMAT_COMET,
  ECS_PRE_FORMAT_ENS,
  ECS_PRE_FORMAT_GAMBIT,
  ECS_PRE_FORMAT_GMSH,
  ECS_PRE_FORMAT_HEX,
  ECS_PRE_FORMAT_IDEAS,
  ECS_PRE_FORMAT_MED,
  ECS_PRE_FORMAT_NOPO,

  ECS_PRE_FORMAT_META

} ecs_pre_format_t ;


/*============================================================================
 *                       Prototypes de fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui renvoie le nombre de formats de lecture supports
 *----------------------------------------------------------------------------*/

int ecs_pre__n_formats
(
 void
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui affiche la liste les formats supports
 *----------------------------------------------------------------------------*/

void ecs_pre__aff_formats
(
 void
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui renvoie le type de format de fichier associ  un fichier
 *   et  une cl optionnelle donns
 *----------------------------------------------------------------------------*/

ecs_pre_format_t  ecs_pre__type_format
(
 const char  *nom_fic,
 const char  *mot_cle
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui lit les maillages sur fichiers
 *
 *  La fonction renvoie le maillage concatn
 *----------------------------------------------------------------------------*/

ecs_maillage_t * ecs_pre__lit_maillage
(
 const char        *nom_fic,
 ecs_pre_format_t   format,
 int                num_maillage,
 ecs_bool_t         cree_grp_cel_section,
 ecs_bool_t         cree_grp_cel_zone,
 ecs_bool_t         cree_grp_fac_section,
 ecs_bool_t         cree_grp_fac_zone
) ;


#endif /* _ECS_PRE_H_ */
