#ifndef _ECS_DEF_H_
#define _ECS_DEF_H_

/*============================================================================
 * Dfinitions, variables globales, et fonctions de base
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*----------------------------------------------------------------------------
 *  Fichier  `include' associ  la configuration du cas
 *  (gnr par le Makefile)
 *----------------------------------------------------------------------------*/

#include "ecs_config.h"


/*============================================================================
 * Dfinition de l'architecture
 *============================================================================*/

#if defined(__sgi__) || defined(__sgi) || defined(sgi)
#define ECS_ARCH_IRIX_64

#elif defined(__hpux__) || defined(__hpux) || defined(hpux)
#define ECS_ARCH_HP_UX

#elif defined(__linux__) || defined(__linux) || defined(linux)
#define ECS_ARCH_Linux

#elif defined(__sun__) || defined(__sun) || defined(sun)
#define ECS_ARCH_SunOS

#elif defined(__uxpv__) || defined(__uxpv) || defined(uxpv)
#define ECS_ARCH_UNIX_System_V

#elif defined(__osf__)
#define ECS_ARCH_OSF1

#endif


/*============================================================================
 * Dfinitions de types
 *============================================================================*/

/*  Types de dimension fixe. */

#if (defined ECS_ARCH_Linux)
#include <stdint.h>

typedef int32_t         ecs_int_32_t;   /* Entier sur 4 octets */
typedef float           ecs_real_32_t;  /* Rel sur 4 octets */

#else

typedef int             ecs_int_32_t;   /* Entier sur 4 octets */
typedef float           ecs_real_32_t;  /* Rel sur 4 octets */

#endif


/* Types usuels */

typedef int             ecs_int_t;      /* Entier */

typedef unsigned        ecs_size_t;     /* Taille pour les index */

typedef double          ecs_real_t;     /* Rel (virgule flottante) */

typedef char            ecs_byte_t;     /* Octet (unit de mmoire non type) */

typedef ecs_real_t      ecs_point_t[3];

typedef enum {                          /* Boolen */
  ECS_FALSE ,
  ECS_TRUE
} ecs_bool_t;


/* numration de type ("type de type") pour transmettre le type d'une donne */

typedef enum {
  ECS_TYPE_char,
  ECS_TYPE_ecs_bool_t,
  ECS_TYPE_ecs_int_t,
  ECS_TYPE_ecs_int_32_t,
  ECS_TYPE_ecs_point_t,
  ECS_TYPE_ecs_real_t,
  ECS_TYPE_ecs_real_32_t,
  ECS_TYPE_ecs_size_t,
  ECS_TYPE_size_t,
  ECS_TYPE_void
} ecs_type_t;


/* numration lie  la dimension spatiale */

typedef enum {

  ECS_DIM_1 = 1 ,
  ECS_DIM_2     , /* Obligatoirement 2 */
  ECS_DIM_3       /* Obligatoirement 3 */

} ecs_dim_t ;


/* Qualificateurs de types restrict (existe en standard en C99) */

#if defined(__GNUC__)
#define restrict __restrict
#else
#define restrict
#endif


/*=============================================================================
 * Dfinitions de macros
 *============================================================================*/

/* Macros "classiques" */

#define ECS_ABS(a)     ((a) <  0  ? -(a) : (a))  /* Valeur absolue de a */
#define ECS_MIN(a,b)   ((a) > (b) ?  (b) : (a))  /* Minimum de a et b */
#define ECS_MAX(a,b)   ((a) < (b) ?  (b) : (a))  /* Maximum de a et b */


/* Dfinition du caractre de sparation de rpertoires */

#define ECS_PATH_SEP             '/'


/* Dfinition de macro pour ecs_int_t defini comme `int' */

#define ECS_FORMAT_ecs_int_t     "d"
#define ECS_FORMAT_ecs_real_t    "e"
#define ECS_FORMAT_char          "c"


/* Pour compilation IRIX64 avec adressage 64 bits (option -64) */

#if defined(ECS_ARCH_IRIX64) && (_MIPS_SZLONG == 64)
#define ECS_FORMAT_size_t "lu"
#else
#define ECS_FORMAT_size_t "u"
#endif


#define ECS_REAL_PRECISION    1.e-13


#define ECS_STR_SIZE       80

#define ECS_PAS_NUL         0
#define ECS_PAS_UNITE       1

#define ECS_LNG_AFF_STR           43
#define ECS_LNG_AFF_ENT            8
#define ECS_LNG_AFF_REE_MANTIS    11
#define ECS_LNG_AFF_REE_PRECIS     2

#define ECS_FMT_AFF_REE_PARAM     "%.15E"


/*
 * Macros pour internationalisation ventuelle via gettext() ou une fonction
 * semblable (pour encadrer les chanes de caractres imprimables)
 */

#if defined(ENABLE_NLS)

#include <libintl.h>
#define _(String) gettext(String)
#define gettext_noop(String) String
#define N_(String) gettext_noop(String)

#else

#define _(String) String
#define N_(String) String
#define textdomain(Domain)
#define bindtextdomain(Package, Directory)

#endif


/*=============================================================================
 * Dfinitions de variables globales
 *============================================================================*/

extern char      ecs_glob_build_date[]; /* Date de compilation */

extern int       ecs_glob_have_cgns;    /* Support du format CGNS */
extern int       ecs_glob_cgns_ver_maj;
extern int       ecs_glob_cgns_ver_min;
extern int       ecs_glob_cgns_ver_rel;

extern int       ecs_glob_have_med;     /* Support de la librairie d'change de
                                           maillages et champs MED */

extern int       ecs_glob_have_metis;   /* Support du dcoupeur de graphes de
                                           connectivit METIS */


/*=============================================================================
 * Prototypes de fonctions
 *============================================================================*/

/*----------------------------------------------------------------------------
 * Fonction d'initialisation de la gestion des erreurs
 *----------------------------------------------------------------------------*/

void ecs_init_gestion_erreur
(
 void
) ;


/*----------------------------------------------------------------------------
 * Fonction d'arret
 *----------------------------------------------------------------------------*/

void ecs_exit
(
 const ecs_int_t  statut
);


/*----------------------------------------------------------------------------
 * Fonction d'impression d'un avertissement
 *----------------------------------------------------------------------------*/

void ecs_warn
(
 void
);


/*----------------------------------------------------------------------------
 *  Fonction qui imprime une chane de caractres avec une largeur
 *  de colonne donne.
 *----------------------------------------------------------------------------*/

void ecs_print_padded_str
(
 const char  *str,
       int    width
);


#endif /* _ECS_DEF_H_ */
