#ifndef _ECS_ELT_TYP_GLOB_H_
#define _ECS_ELT_TYP_GLOB_H_

/*============================================================================
 *  Dfinitions d'un type numr et de macros
 *   dfinissant les types des lments reconnus par le code
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilit
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C
 *----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
 *  Fichiers `include' publics  du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"
#include "ecs_elt_typ_glob.h"


/*============================================================================
 *                       Dfinition du type enumr
 *============================================================================*/


typedef enum {

  ECS_ELT_TYP_NUL       ,  /*  Pas de type         */

  ECS_ELT_TYP_SOM       ,  /*  Sommet              */

  ECS_ELT_TYP_ARE       ,  /*  Arete               */

  ECS_ELT_TYP_FAC_TRIA  ,  /*  Triangle            */
  ECS_ELT_TYP_FAC_QUAD  ,  /*  Quadrangle          */

  ECS_ELT_TYP_CEL_TETRA ,  /*  Tetraedre           */
  ECS_ELT_TYP_CEL_PYRAM ,  /*  Pyramide            */
  ECS_ELT_TYP_CEL_PRISM ,  /*  Prisme              */
  ECS_ELT_TYP_CEL_HEXA  ,  /*  Hexaedre            */

  ECS_ELT_TYP_FAC_POLY  ,  /*  Polygone quelconque */
  ECS_ELT_TYP_CEL_POLY  ,  /*  Polyedre quelconque */

  ECS_ELT_TYP_FIN

} ecs_elt_typ_t ;


/*============================================================================
 *                            Dfinition de macros
 *============================================================================*/

/*=========================================================================
 *  Dcomposition des lments en sous-lments
 *   avec dfinition des sous-lments par rapport aux sommets de l'lment
 *
 *  - arte   en sommets
 *  - face    en artes
 *  - cellule en faces
 *
 *  Les faces sont dfinies par rapport  leur normale extrieure
 *   (les sommets se suivent dans le sens trigonomtrique)
 *=========================================================================*/

/* Nombre maximal de sommets       definissant un element */
#define ECS_ELT_TYP_NBR_MAX_SOM         8

/* Nombre maximal de sous-elements definissant un element */
#define ECS_ELT_TYP_NBR_MAX_SSELT       6

/* Nombre maximal de sommets       definissant un sous-element    */
/* (c'est aussi, le nombre maximal d'aretes definissant une face) */
#define ECS_ELT_TYP_SSELT_NBR_MAX_SOM   4

/* Longueur maximale du nom d'un type d'lment (+1 pour le `\0' !) */
#define ECS_ELT_TYP_LNG_MAX_NOM        11


#endif /* _ECS_ELT_TYP_GLOB_H_ */
