/*============================================================================
 *  Dfinitions des fonctions de base associes  une liste chane de
 *  structures `ecs_champ_t' dcrivant un champ
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilit
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <string.h> /* strcmp() */

#include <bft_file.h>
#include <bft_mem.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"
#include "ecs_fic.h"
#include "ecs_tab.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles des paquetages visibles
 *----------------------------------------------------------------------------*/

#include "ecs_descr.h"
#include "ecs_param_perio_glob.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_vec_int.h"
#include "ecs_champ_att.h"
#include "ecs_champ_grd.h"
#include "ecs_champ.h"
#include "ecs_vec_def_perio.h"


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associ au fichier courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_chaine.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' privs   du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_priv.h"


/*============================================================================
 *                       Prototypes de fonctions prives
 *============================================================================*/


/*============================================================================
 *                             Fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction librant la portion d'une liste chane de structures `ecs_champ_t'
 *    partir d'un noeud dont le pointeur est donn en argument.
 *  Le noeud est  NULL au retour de la fonction
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__detruit
(
 ecs_champ_t * * this_champ_noeud
)
{

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (*this_champ_noeud != NULL) {

    ecs_champ_chaine__detruit(&(*this_champ_noeud)->l_champ_sui) ;

    *this_champ_noeud = ecs_champ__detruit(*this_champ_noeud) ;

  }


}


/*----------------------------------------------------------------------------
 *  Fonction imprimant  partir d'un noeud `ecs_champ_t' donn
 *   une liste chane de champs
 *   sur le flux dcrit par la structure `bft_file_t'
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__imprime
(
 const ecs_champ_t *const this_champ_noeud ,
       ecs_int_t          imp_col          ,
 const ecs_int_t          nbr_imp          ,
       bft_file_t  *const fic_imp
)
{

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


#define ECS_FCT_IMP_CHAMP_NOEUD  "champ"


  if (this_champ_noeud != NULL) {

    ecs_champ_chaine__imprime(this_champ_noeud->l_champ_sui,
                              imp_col,
                              nbr_imp,
                              fic_imp) ;

    ecs_fic__imprime_ptr(fic_imp, imp_col, ECS_FCT_IMP_CHAMP_NOEUD,
                         (const void *)this_champ_noeud) ;

    ecs_champ__imprime(this_champ_noeud,
                       imp_col,
                       nbr_imp,
                       fic_imp) ;

  }


}


/*----------------------------------------------------------------------------
 *  Fonction qui renvoie la taille en octets
 *   d'une chane de structures `ecs_champ_t' dont la tte est donne
 *----------------------------------------------------------------------------*/

float ecs_champ_chaine__ret_taille
(
 const ecs_champ_t *const this_champ_noeud
)
{

  float  taille       ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  taille       = 0. ;


  if (this_champ_noeud != NULL) {

    taille += ecs_champ_chaine__ret_taille(this_champ_noeud->l_champ_sui) ;

    taille += ecs_champ__ret_taille(this_champ_noeud) ;

  }


  return taille ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui retourne dans une liste chane de champs
 *   dont un noeud est donn la rfrence au champ suivant.
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_chaine__ret_suivant
(
 ecs_champ_t *const this_champ
)
{

  return this_champ->l_champ_sui ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui recherche dans une liste chane de champs
 *   dont la tte de la liste est donne,
 *   le champ correspondant au nom fourni en argument
 *
 *  La fonction retourne :
 *   -    la rfrence du champ trouv,
 *   - ou NULL si aucun champ n'a t trouv
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_chaine__trouve_nom
(
       ecs_champ_t *const this_champ_tete , /* --> Tte de liste des champs   */
                                            /*     ou est recherche le champ  */
 const char        *const nom_champ         /* --> Nom  rechercher           */
)
{


  ecs_champ_t *ptr_champ ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (this_champ_tete == NULL || nom_champ == NULL)
    return NULL ;


  ptr_champ = this_champ_tete ;


  while ((ptr_champ != NULL)          &&
         (strcmp(ptr_champ->nom,
                 nom_champ))     != 0    ) {

    ptr_champ = ptr_champ->l_champ_sui ;

  }


  return ptr_champ ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui recherche dans une liste chane de champs
 *   dont la tte de la liste est donne,
 *   le premier champ correspondant au statut fourni en argument
 *
 *  La fonction retourne :
 *   -    la rfrence du champ trouv,
 *   - ou NULL si aucun champ n'a t trouv
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_chaine__trouve_statut
(
 ecs_champ_t         *const this_champ_tete , /* --> Tte de liste des champs */
                                              /*     o on recherche le champ */
 ECS_CHAMP_STATUT_E         statut            /* --> Nom  rechercher         */
)
{


  ecs_champ_t *ptr_champ ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(this_champ_tete != NULL) ;


  ptr_champ = this_champ_tete ;


  while (   (ptr_champ != NULL)
         && (ptr_champ->statut_e != statut)) {

    ptr_champ = ptr_champ->l_champ_sui ;

  }

  return ptr_champ ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui supprime un noeud (c-a-d la rfrence au champ donn)
 *   dans une liste chane de champs dont la tte est donne
 *
 *  La tte de la liste n'est pas modifie
 *   sauf si le noeud  supprimer est aussi la tte de la liste !
 *
 *  Attention! Le champ dont la rfrence est supprime de la liste,
 *              n'est pas dtruit et
 *              son lien sur un champ suivant est r-initialis  `NULL'
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__supprime
(
 ecs_champ_t * *const this_champ_noeud ,  /* --> Adresse du champ  supprimer */
 ecs_champ_t * *const champ_tete          /* --> tte de la liste contenant   */
                                          /*      le noeud  supprimer        */
)
{

  ecs_champ_t *loc_champ_sui  ;
  ecs_champ_t *ptr_champ      ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (*this_champ_noeud == NULL)
    return ;


  assert(*champ_tete != NULL) ;


  if ((*this_champ_noeud) == *champ_tete) {

    /* Le champ noeud  supprimer est la tte de liste des champs */
    /* Le champ suivant est la nouvelle tte                      */

    *champ_tete = (*champ_tete)->l_champ_sui ;

  }
  else {

    /* On recherche le noeud qui prcde le noeud  supprimer */

    loc_champ_sui = (*this_champ_noeud)->l_champ_sui ;


    for (ptr_champ = *champ_tete ;
         ptr_champ != NULL && ptr_champ->l_champ_sui != (*this_champ_noeud) ;
         ptr_champ = ptr_champ->l_champ_sui)
      ;


    /* Le noeud  supprimer doit tre contenu dans la liste */
    /* dont `*champ_tete' est la tte                       */
    assert(ptr_champ != NULL);

    ptr_champ->l_champ_sui = loc_champ_sui ;


  } /* Fin else : le champ  supprimer n'est pas le champ de tte */


  /* Re-initialisation  `NULL' du lien sur un champ suivant */
  /*  pour le champ dconnect de la chane                  */

  (*this_champ_noeud)->l_champ_sui = NULL ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui ajoute  la fin d'une liste chane de champs rceptrice
 *   une liste chane de champs  concatner.
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__ajoute
(
 ecs_champ_t * *const this_champ_tete   ,
 ecs_champ_t   *const champ_concat_tete
)
{

  ecs_champ_t *loc_champ_prec ;
  ecs_champ_t *ptr_champ      ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(this_champ_tete != NULL) ;


  if (*this_champ_tete != NULL) {

    /* On va  la fin de la chane rceptrice */

    for (ptr_champ = *this_champ_tete       ;
         ptr_champ != NULL                  ;
         ptr_champ = ptr_champ->l_champ_sui  )
      loc_champ_prec = ptr_champ ;


    /* On ajoute le lien avec le dbut de la chane  concatner */

    loc_champ_prec->l_champ_sui = champ_concat_tete ;


  }
  else {


    *this_champ_tete = champ_concat_tete ;

  }


}


/*----------------------------------------------------------------------------
 *  Fonction qui ajoute  la fin d'une liste chane de champs rceptrice
 *   une liste chane de champs  concatner.
 *  Si des champs de mme nom existent dans la liste chane, ils sont
 *   supprims avant l'ajout des nouveaux champs.
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__remplace
(
 ecs_champ_t * *const this_champ_tete   ,
 ecs_champ_t   *const champ_concat_tete
)
{

  ecs_champ_t *champ_supprime ;
  ecs_champ_t *ptr_champ      ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(this_champ_tete != NULL) ;


  if (*this_champ_tete != NULL) {


    /* On supprime les champs de mme nom (prcaution) */

    for (ptr_champ = champ_concat_tete      ;
         ptr_champ != NULL                  ;
         ptr_champ = ptr_champ->l_champ_sui  ) {

      champ_supprime = ecs_champ_chaine__trouve_nom(*this_champ_tete,
                                                    ptr_champ->nom) ;

      if (champ_supprime != NULL) {

        ecs_champ_chaine__supprime(&champ_supprime,
                                   this_champ_tete) ;

        ecs_champ__detruit(champ_supprime) ;

      }

    }

  }


  ecs_champ_chaine__ajoute(this_champ_tete,
                           champ_concat_tete) ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui concatne les champs identiques
 *   de deux listes chanes de champs
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__concatene
(
 ecs_champ_t * * this_champ_tete    ,  /* <-> Tte de la chane rceptrice    */
 ecs_champ_t * * concat_champ_tete  ,  /* <-> Tte de la chane contenant     */
                                       /*         les champs  concatner     */
 size_t          nbr_elt_init_recep ,  /* --> Nbr d'elts de l'entit recept.  */
 size_t          nbr_elt_ent_concat    /* --> Nbr d'elts ent.  concatner.   */
)
{

  ecs_champ_t *this_champ_ptr    ;
  ecs_champ_t *concat_champ_ptr  ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(this_champ_tete != NULL) ;
  assert(concat_champ_tete != NULL) ;


  /*---------------------------------------------------------*/
  /* Boucle sur les champs de la chane de champs rceptrice */
  /*---------------------------------------------------------*/


  for (this_champ_ptr  = *this_champ_tete ;
       this_champ_ptr != NULL  ;
       this_champ_ptr  = this_champ_ptr->l_champ_sui) {


    /* Le champ existe-t-il dans l'entit de maillage  concatner ? */
    /*---------------------------------------------------------------*/

    if (concat_champ_tete != NULL)
      concat_champ_ptr = *concat_champ_tete ;
    else
      concat_champ_ptr = NULL ;


    while ((concat_champ_ptr != NULL)           &&
           (strcmp(this_champ_ptr->nom,
                   concat_champ_ptr->nom)) != 0    ) {

      concat_champ_ptr = concat_champ_ptr->l_champ_sui ;

    }


    /* ------------------------------------------------------------------ */
    /* PARTIE REALISEE PENDANT LA FUSION                                  */
    /* ------------------------------------------------------------------ */
    /*    Si le champ n'existe pas dans l'entit de maillage  concatner */
    /* et si c'est un champ de type "attribut"                            */
    /* (pour lequel tous les lments doivent tre renseignes)            */
    /* on initialise les valeurs de l'attribut pour le champ  concatner */
    /* ------------------------------------------------------------------ */

    if (concat_champ_ptr != NULL) {

      /* Le champ existe dans l'entit de maillage  concatner */

      if ((*concat_champ_tete)->statut_e == ECS_CHAMP_STATUT_REF_ELT)

        /* Pour certains champs (de type filiation, ou l'on a une
           'auto-rfrence'), il faut d'abord dcaler les rfrences
           aux lments du champ  concatner */

        ecs_champ__incremente_val(*concat_champ_tete,
                                  nbr_elt_init_recep) ;

      /* On le concatne */

      ecs_champ__concatene(this_champ_ptr,
                           concat_champ_ptr) ;

      /* On le dtruit de la liste des champs */

      ecs_champ_chaine__supprime(&concat_champ_ptr,
                                 concat_champ_tete) ;

      concat_champ_ptr = ecs_champ__detruit(concat_champ_ptr) ;

    }

    else {

      /* Le champ n'existe pas dans l'entit de maillage  concatner */

      ecs_champ__prolonge(this_champ_ptr,
                          0,
                          nbr_elt_ent_concat) ;

    }

  } /* Fin : boucle sur les champs de l'entit de maillage receptrice */


  /*----------------------------------------------------------------------*/
  /* La liste chane des champs  concatner RESTANTS                    */
  /*  (qui n'taient pas initialement prsents dans la chane rceptrice) */
  /*  est rajoute  la liste chane des champs rceptrice                */
  /*  (si elle n'est pas vide !)                                           */
  /*----------------------------------------------------------------------*/

  if (concat_champ_tete != NULL) {

    while (*concat_champ_tete != NULL) {

      concat_champ_ptr = (*concat_champ_tete) ;

      if (concat_champ_ptr->statut_e == ECS_CHAMP_STATUT_REF_ELT)

        /* Pour certains champs (de type filiation, ou l'on a une
           'auto-rfrence'), il faut d'abord dcaler les rfrences
           aux lments du champ  concatner */

        ecs_champ__incremente_val(concat_champ_ptr,
                                  nbr_elt_init_recep) ;

      /* Le champ n'existe pas dans l'entit de maillage rceptrice */

      ecs_champ__prolonge(concat_champ_ptr,
                          nbr_elt_init_recep,
                          0) ;


      /* On le dtruit de la liste des champs du maillage  concatner */

      ecs_champ_chaine__supprime(&concat_champ_ptr,
                                 concat_champ_tete) ;

      /* On le transfre  l'entit de maillage rceptrice */

      ecs_champ_chaine__ajoute(this_champ_tete,
                               concat_champ_ptr) ;


    } /* Fin : boucle sur les champs de l'entit de maillage  concatner */

  }

}


/*----------------------------------------------------------------------------
 *  Fonction appliquant un vecteur de transformation
 *    une liste chane de champs donne
 *----------------------------------------------------------------------------*/

void ecs_champ_chaine__transforme
(
       ecs_champ_t         * this_champ_chaine_tete ,
       size_t                nbr_elt_new            ,
 const ecs_tab_int_t         vect_transf            ,
 ECS_PTR_FCT_TRANSF_CHAMP    fct_transformation
)
{

  ecs_champ_t *ptr_champ ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/



  for (ptr_champ = this_champ_chaine_tete ;
       ptr_champ != NULL ;
       ptr_champ = ptr_champ->l_champ_sui) {


    (* fct_transformation)(ptr_champ  ,
                           nbr_elt_new,
                           vect_transf ) ;

  }


}


/*----------------------------------------------------------------------------
 *  Fonction qui construit les champs de la chane de champs des sous-lments
 *                 partir des champs de la chane de champs des lments
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_chaine__herite
(
 ecs_champ_t *champ_elt_tete,
 ecs_champ_t *champ_elt_def_sselt,
 ecs_champ_t *champ_def_sselt
)
{

  ecs_champ_t  *ptr_champ_elt ;
  ecs_champ_t  *champ_sselt   ;
  ecs_champ_t  *this_champ_sselt_tete ;

  const ecs_int_t nbr_sselt = ecs_champ__ret_pos_nbr(champ_def_sselt) - 1 ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  this_champ_sselt_tete = NULL ;


  for (ptr_champ_elt = champ_elt_tete ;
       ptr_champ_elt != NULL ;
       ptr_champ_elt = ptr_champ_elt->l_champ_sui) {


    if (ptr_champ_elt->statut_e == ECS_CHAMP_STATUT_HERITABLE) {


      champ_sselt = ecs_champ_att__herite(ptr_champ_elt,
                                          champ_elt_def_sselt,
                                          nbr_sselt) ;


      if (champ_sselt != NULL) {


        /* Ajout du nouveau champ dans la liste chane */
        /*  des champs des sous-lments                */
        /*----------------------------------------------*/

        ecs_champ_chaine__ajoute(&this_champ_sselt_tete,
                                 champ_sselt) ;
      }


    }


  }  /* Fin de la boucle sur les champs des lments */


  return this_champ_sselt_tete ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui renumrote les labels et les rfrences
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_chaine__renum
(
 ecs_champ_t    *champ_elt_tete,
 ecs_champ_t    *champ_elt_old_new,
 size_t          nbr_elt_new
)
{

  ecs_champ_t  *ptr_champ_elt ;
  ecs_champ_t  *champ_elt_aux ;
  ecs_champ_t  *this_champ_elt_tete ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  this_champ_elt_tete = NULL ;


  for (ptr_champ_elt = champ_elt_tete ;
       ptr_champ_elt != NULL ;
       ptr_champ_elt = ptr_champ_elt->l_champ_sui) {


    if (ptr_champ_elt->statut_e == ECS_CHAMP_STATUT_REF_ELT) {


      champ_elt_aux = ecs_champ__copie(ptr_champ_elt) ;

      ecs_champ_att__renum_perio(champ_elt_aux,
                                 champ_elt_old_new,
                                 nbr_elt_new) ;


      /* Ajout du nouveau champ dans la liste chane */
      /* des champs des sous-lments                 */
      /*----------------------------------------------*/

      ecs_champ_chaine__ajoute(&this_champ_elt_tete,
                               champ_elt_aux) ;


    }


  } /* Fin de la boucle sur les champs des lments */


  return this_champ_elt_tete ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui modifie le champ de connectivit inverse "faces -> cellules"
 *  dans le cas d'une ou plusieurs priodicit.
 *  On retourne un champ marquant les cellules fantmes pour chaque
 *  priodicit.
 *----------------------------------------------------------------------------*/

ecs_int_t ecs_champ_chaine__connect_perio
(
 ecs_champ_t  * champ_att_tete,
 ecs_champ_t  * champ_connect_inverse_fac_cel
)
{

  ecs_int_t     nbr_per ;

  ecs_champ_t  *ptr_champ_att ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  nbr_per = 0 ;


  for (ptr_champ_att = champ_att_tete ;
       ptr_champ_att != NULL ;
       ptr_champ_att = ptr_champ_att->l_champ_sui) {


    if (   strncmp(ptr_champ_att->nom, ECS_CHAMP_NOM_FAC_PERIO,
                   strlen(ECS_CHAMP_NOM_FAC_PERIO))== 0
        && ptr_champ_att->statut_e == ECS_CHAMP_STATUT_REF_ELT) {

      ecs_champ_att__connect_inv_perio(ptr_champ_att,
                                       champ_connect_inverse_fac_cel) ;

      nbr_per++ ;

    }


  } /* Fin de la boucle sur les champs attributs des faces */


  return nbr_per ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui construit une liste de toutes les faces priodiques
 *  de la manire suivante :
 *  =>  [i,j,n; ...] avec i->j ou j->i (i>j) et n le numro de la priodicit
 *      prcd d'un signe correspondant au sens  appliquer
 *  On ne compte pas les faces se voyant elles-mmes.
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_champ_chaine__liste_fac_perio_glob
(
 ecs_champ_t    * champ_att_tete
)
{

  ecs_int_t        nbr_per ;

  ecs_tab_int_t    liste_fac_perio ;

  ecs_vec_int_t ** liste_vec_fac_perio ;

  ecs_champ_t    * ptr_champ_att ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  /* Rcupration des vecteurs de correspondances priodiques */

  nbr_per =  0 ;
  liste_vec_fac_perio = NULL ;

  for (ptr_champ_att = champ_att_tete ;
       ptr_champ_att != NULL ;
       ptr_champ_att = ptr_champ_att->l_champ_sui)

    if (   strncmp(ptr_champ_att->nom, ECS_CHAMP_NOM_FAC_PERIO,
                   strlen(ECS_CHAMP_NOM_FAC_PERIO))== 0
        && ptr_champ_att->statut_e == ECS_CHAMP_STATUT_REF_ELT) {

      BFT_REALLOC(liste_vec_fac_perio, nbr_per + 1, ecs_vec_int_t *) ;

      liste_vec_fac_perio[nbr_per++] =
        ecs_champ__initialise_vec_int(ptr_champ_att) ;

    }


  /* Construction d'une liste globale des faces priodiques */

  liste_fac_perio = ecs_vec_def__liste_fac_perio_glob(liste_vec_fac_perio,
                                                      nbr_per) ;

  /* Libration des vecteurs */

  nbr_per = 0 ;

  for (ptr_champ_att = champ_att_tete ;
       ptr_champ_att != NULL ;
       ptr_champ_att = ptr_champ_att->l_champ_sui)

    if (   strncmp(ptr_champ_att->nom, ECS_CHAMP_NOM_FAC_PERIO,
                   strlen(ECS_CHAMP_NOM_FAC_PERIO))== 0
        && ptr_champ_att->statut_e == ECS_CHAMP_STATUT_REF_ELT)

      ecs_champ__libere_vec_int(ptr_champ_att,
                                liste_vec_fac_perio[nbr_per++]) ;


  BFT_FREE(liste_vec_fac_perio) ;


  return liste_fac_perio ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui construit une liste de toutes les faces internes priodiques
 *  de la manire suivante (avec numros i, j en numrotation 1  n):
 *  =>  [i,j,n; ...] avec i->j ou j->i (i>j) et n le numro de la priodicit
 *      prcd d'un signe correspondant au sens  appliquer
 *  On ne compte pas les faces se voyant elles-mmes.
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_champ_chaine__liste_fac_int_perio
(
 ecs_champ_t    *      champ_att_tete    ,
 size_t                nbr_fac_interne   ,
 ecs_int_t      *const liste_fac_interne
)
{

  size_t        ind ;
  size_t        ifac_1 ;
  size_t        ifac_2 ;
  ecs_int_t     ind_fac_max ;
  size_t        nbr_couples ;

  ecs_int_t    *ind_interne ;

  ecs_tab_int_t liste_fac_perio ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  liste_fac_perio
    = ecs_champ_chaine__liste_fac_perio_glob(champ_att_tete) ;

  nbr_couples = liste_fac_perio.nbr / 3 ;

  /* Cration du tableau de projection */

  ind_fac_max = 0 ;
  for (ind = 0 ; ind < nbr_fac_interne ; ind++) {
    if (liste_fac_interne[ind] > ind_fac_max)
      ind_fac_max = liste_fac_interne[ind] ;
  }
  ind_fac_max += 1 ;

  BFT_MALLOC(ind_interne, ind_fac_max, ecs_int_t) ;

  for (ind = 0 ; ind < (size_t)ind_fac_max ; ind++)
    ind_interne[ind] = -1 ;

  for (ind = 0 ; ind < nbr_fac_interne ; ind++)
    ind_interne[liste_fac_interne[ind]] = ind ;

  assert(nbr_couples*3 == liste_fac_perio.nbr) ;

  /* Projection : numrotation 1  n sur faces globales
     vers numrotation 1  n sur faces internes */

  for (ind = 0 ; ind < nbr_couples ; ind++) {

    ifac_1 = liste_fac_perio.val[ind*3    ] - 1 ;
    ifac_2 = liste_fac_perio.val[ind*3 + 1] - 1 ;

    assert(ind_interne[ifac_1] != -1 && ind_interne[ifac_2] != -1) ;

    liste_fac_perio.val[ind*3    ] = ind_interne[ifac_1] + 1 ;
    liste_fac_perio.val[ind*3 + 1] = ind_interne[ifac_2] + 1 ;

  }

  BFT_FREE(ind_interne) ;

  return liste_fac_perio ;

}


/*============================================================================
 *                              Fonctions prives
 *============================================================================*/

