/*============================================================================
 *  Definitions des fonctions
 *   associees a la structure `ecs_champ_t' decrivant un champ
 *   et propres aux champs principaux de type "definition"
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilite
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <string.h> /* strlen() */

#include <bft_mem.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_chaine_glob.h"
#include "ecs_chrono.h"
#include "ecs_param_rc_glob.h"
#include "ecs_param_perio_glob.h"
#include "ecs_def.h"
#include "ecs_elt_typ_glob.h"
#include "ecs_tab.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles des paquetages visibles
 *----------------------------------------------------------------------------*/

#include "ecs_vec_int.h"
#include "ecs_vec_int_tri.h"
#include "ecs_vec_real.h"
#include "ecs_vec_real_tri.h"
#include "ecs_vec_def.h"
#include "ecs_vec_def_rc.h"
#include "ecs_vec_def_perio.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_vec_int.h"
#include "ecs_champ_vec_real.h"
#include "ecs_champ.h"


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associe au fichier courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_def.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' prives   du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_priv.h"


/*============================================================================
 *                       Prototypes de fonctions prives
 *============================================================================*/


/*============================================================================
 *                             Fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui construit
 *   les dfinitions des faces par dcomposition des champs des cellules
 *----------------------------------------------------------------------------*/

void ecs_champ_def__decompose_cel
(
 ecs_champ_t *vect_champ_fac[],
 ecs_champ_t *champ_def_cel,
 ecs_champ_t *champ_typ_geo_cel
)
{

  ecs_vec_int_t *vec_def_cel ;
  ecs_vec_int_t *vec_typ_geo_cel ;
  ecs_vec_int_t *vec_def_fac ;
  ecs_vec_int_t *vec_cel_def_fac ;

  size_t        nbr_fac ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  /* Construction, pour les cellules,                             */
  /*  des vecteurs `ecs_vec_int_t' associs aux champs principaux */
  /*--------------------------------------------------------------*/

  vec_def_cel     = ecs_champ__initialise_vec_int(champ_def_cel) ;
  vec_typ_geo_cel = ecs_champ__initialise_vec_int(champ_typ_geo_cel) ;


  ecs_vec_def__decompose_cel(&vec_def_fac,
                             &vec_cel_def_fac,
                             vec_def_cel,
                             vec_typ_geo_cel) ;


  ecs_vec_int__detruit(vec_def_cel) ;


  ecs_champ__transfere_vec_int(champ_def_cel,
                               vec_cel_def_fac) ;

  ecs_champ__libere_vec_int(champ_typ_geo_cel,
                            vec_typ_geo_cel) ;


  nbr_fac = ecs_vec_int__ret_pos_nbr(vec_def_fac) - 1 ;

  vect_champ_fac[ECS_CHAMP_DEF]
    = ecs_champ__init_avec_vec_int(vec_def_fac,
                                   ECS_CHAMP_NOM_DEFINIT) ;

  vect_champ_fac[ECS_CHAMP_DEF]->statut_e  = ECS_CHAMP_STATUT_INDEFINI ;


  vect_champ_fac[ECS_CHAMP_TYP] = NULL ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui construit
 *   les dfinitions des artes par dcomposition des champs des faces
 *----------------------------------------------------------------------------*/

void ecs_champ_def__decompose_fac
(
 ecs_champ_t *vect_champ_are[],
 ecs_champ_t *champ_def_fac
)
{

  ecs_vec_int_t *vec_def_fac ;
  ecs_vec_int_t *vec_def_are ;
  ecs_vec_int_t *vec_fac_def_are ;

  size_t        nbr_are ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  /* Construction, pour les elements,                         */
  /*  des vecteurs `ecs_vec_int_t' associes aux champs principaux */
  /*----------------------------------------------------------*/

  vec_def_fac     = ecs_champ__initialise_vec_int(champ_def_fac) ;


  ecs_vec_def__decompose_fac(&vec_def_are,
                             &vec_fac_def_are,
                             vec_def_fac) ;


  ecs_vec_int__detruit(vec_def_fac) ;


  ecs_champ__transfere_vec_int(champ_def_fac,
                               vec_fac_def_are) ;


  nbr_are = ecs_vec_int__ret_pos_nbr(vec_def_are) - 1 ;

  vect_champ_are[ECS_CHAMP_DEF]
    = ecs_champ__init_avec_vec_int(vec_def_are,
                                   ECS_CHAMP_NOM_DEFINIT) ;

  vect_champ_are[ECS_CHAMP_DEF]->statut_e  = ECS_CHAMP_STATUT_INDEFINI ;


  vect_champ_are[ECS_CHAMP_TYP] = NULL ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui realise la fusion des definitions des elements
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_champ_def__int_fusionne
(
 ecs_champ_t   * this_champ_def, /* <-> Reference sur champ des definitions   */
 size_t        * nbr_elt_cpct,
 ecs_tab_int_t * signe_elt       /* <-- Signe des elts apres tri de leur def. */
)
{

  ecs_vec_int_t * vec_def ;      /* Vecteur des definitions des elements */
  ecs_tab_int_t   vect_transf ;  /* Vecteur de transformation */


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  vec_def = ecs_champ__initialise_vec_int(this_champ_def) ;



  /*--------------------------------------------------------------------------*/
  /* 1ere etape                                                               */
  /*--------------------------------------------------------------------------*/
  /*                                                                          */
  /* Fusion des definitions des elements sur la liste initiale                */
  /*                                                                          */
  /*--------------------------------------------------------------------------*/


  vect_transf = ecs_vec_def__fusionne(&vec_def,
                                      signe_elt) ;



  /*--------------------------------------------------------------------------*/
  /* 2eme etape                                                               */
  /*--------------------------------------------------------------------------*/
  /*                                                                          */
  /* Affectation de la nouvelle liste compactee                               */
  /*                                                                          */
  /*--------------------------------------------------------------------------*/


  *nbr_elt_cpct = ecs_vec_int__ret_pos_nbr(vec_def) - 1 ;


  ecs_champ__transfere_vec_int(this_champ_def,
                               vec_def) ;



  /*--------------------------------------------------------------------------*/
  /*                                                                          */
  /* Renvoi du vecteur de transformation                                      */
  /*                                                                          */
  /*--------------------------------------------------------------------------*/


  return vect_transf ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui realise la fusion des definitions des elements
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_champ_def__real_fusionne
(
 ecs_champ_t * this_champ_def, /* <-> Reference sur le champ des definitions */
 size_t      * nbr_elt_cpct
)
{

  ecs_vec_real_t * vec_def ;  /* Vecteur des definitions des elements     */

  ecs_tab_int_t    vect_transf ;  /* Vecteur de transformation                */


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/



  vec_def = ecs_champ__initialise_vec_real(this_champ_def) ;


  /*--------------------------------------------------------------------------*/
  /* 1ere etape                                                               */
  /*--------------------------------------------------------------------------*/
  /*                                                                          */
  /* Fusion des definitions des elements sur la liste initiale                */
  /*                                                                          */
  /*--------------------------------------------------------------------------*/


  vect_transf = ecs_vec_real_def__fusionne(&vec_def) ;



  /*--------------------------------------------------------------------------*/
  /* 2eme etape                                                               */
  /*--------------------------------------------------------------------------*/
  /*                                                                          */
  /* Affectation de la nouvelle liste compactee                               */
  /*                                                                          */
  /*--------------------------------------------------------------------------*/


  *nbr_elt_cpct = ecs_vec_real__ret_pos_nbr(vec_def) - 1 ;

  ecs_champ__transfere_vec_real(this_champ_def,
                                vec_def) ;



  /*--------------------------------------------------------------------------*/
  /*                                                                          */
  /* Renvoi du vecteur de transformation                                      */
  /*                                                                          */
  /*--------------------------------------------------------------------------*/


  return vect_transf ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui compte les faces priodiques.
 *----------------------------------------------------------------------------*/

void ecs_champ_def__compte_fac_perio
(
 ecs_champ_t    *const champ_att_fac,
 ecs_tab_int_t  *const nbr_fac_perio
)
{

  ecs_int_t       cpt_per ;

  ecs_int_t       nbr_fac_perio_loc ;

  ecs_champ_t   * ptr_champ_att ;

  ecs_vec_int_t * vec_fac_perio ;

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  cpt_per = 0 ;

  if (nbr_fac_perio != NULL) {
    nbr_fac_perio->nbr = 0 ;
    nbr_fac_perio->val = NULL ;
  }

  for (ptr_champ_att = champ_att_fac ;
       ptr_champ_att != NULL ;
       ptr_champ_att = ptr_champ_att->l_champ_sui)

    if (   strncmp(ptr_champ_att->nom, ECS_CHAMP_NOM_FAC_PERIO,
                   strlen(ECS_CHAMP_NOM_FAC_PERIO))== 0
        && ptr_champ_att->statut_e == ECS_CHAMP_STATUT_REF_ELT) {

      vec_fac_perio = ecs_champ__initialise_vec_int(ptr_champ_att) ;

      nbr_fac_perio_loc = ecs_vec_def__compte_elt_perio(vec_fac_perio) ;

      if (nbr_fac_perio != NULL) {
        nbr_fac_perio->nbr = cpt_per + 1 ;
        BFT_REALLOC(nbr_fac_perio->val, nbr_fac_perio->nbr, ecs_int_t) ;
        nbr_fac_perio->val[cpt_per] = nbr_fac_perio_loc ;
      }

      ecs_champ__libere_vec_int(ptr_champ_att,
                                vec_fac_perio) ;

      cpt_per++ ;

    }

}


/*----------------------------------------------------------------------------
 *  Fonction qui construit les listes de faces internes et de bord, ainsi que
 *  les listes de faces avec erreur de connectivit (i.e. qui appartiennent
 *   2 cellules ou plus vu d'un mme ct) et de faces isoles.
 *
 *  Les listes dont les pointeurs fournis en argument sont  NULL ne
 *  sont pas construites. Celles dont les pointeurs sont fournis
 *  sont initialises et alloues si elles ne sont pas vides.
 *----------------------------------------------------------------------------*/

void ecs_champ_def__cree_listes_fac
(
 ecs_champ_t      *const champ_def_fac_cel,
 size_t                  nbr_fac,
 ecs_tab_int_t    *const liste_fac_erreur,
 ecs_tab_int_t    *const liste_fac_interne,
 ecs_tab_int_t    *const liste_fac_de_bord,
 ecs_tab_int_t    *const liste_fac_isolee
)
{

  ecs_vec_int_t * vec_def_fac_cel ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(champ_def_fac_cel != NULL) ;


  vec_def_fac_cel = ecs_champ__initialise_vec_int(champ_def_fac_cel) ;


  ecs_vec_def__listes_fac(vec_def_fac_cel,
                          nbr_fac,
                          liste_fac_erreur,
                          liste_fac_interne,
                          liste_fac_de_bord,
                          liste_fac_isolee) ;


  ecs_champ__libere_vec_int(champ_def_fac_cel,
                            vec_def_fac_cel) ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui renvoie la liste des cellules partageant au moins un
 *  mme ct d'une de leurs faces avec une autre cellule.
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_champ_def__liste_cel_err_c
(
       size_t                nbr_cel_tot,
       ecs_champ_t    *const champ_def_fac_cel,
 const ecs_tab_int_t         liste_fac_erreur
)
{

  ecs_tab_int_t   liste_cel_err ;
  ecs_vec_int_t * vec_def_fac_cel ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(champ_def_fac_cel != NULL) ;


  vec_def_fac_cel = ecs_champ__initialise_vec_int(champ_def_fac_cel) ;


  liste_cel_err = ecs_vec_def__liste_cel_err_cnct(nbr_cel_tot,
                                                  vec_def_fac_cel,
                                                  liste_fac_erreur) ;


  ecs_champ__libere_vec_int(champ_def_fac_cel,
                            vec_def_fac_cel) ;

  return  liste_cel_err ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui construit la liste des cellules attaches  une liste
 *  de faces fournie en argument.
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_champ_def__liste_cel_fac
(
 const ecs_int_t            nbr_fac,
       ecs_champ_t   *const champ_def_cel,
 const ecs_tab_int_t        liste_fac
)
{
  ecs_tab_int_t   liste_cel ;
  ecs_vec_int_t * vec_def_cel ;

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/

  assert(champ_def_cel != NULL) ;


  vec_def_cel = ecs_champ__initialise_vec_int(champ_def_cel) ;


  liste_cel = ecs_vec_def__liste_cel_fac(nbr_fac,
                                         vec_def_cel,
                                         liste_fac) ;

  ecs_champ__libere_vec_int(champ_def_cel,
                            vec_def_cel) ;

  return liste_cel ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui construit la table de connectivite "faces -> sommets"
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_def__cree_fac_som
(
 ecs_champ_t       *const champ_def_fac,
 ecs_champ_t       *const champ_def_are
)
{

  ecs_champ_t   * champ_connect_fac_som ;

  ecs_vec_int_t * vec_connect_fac_som ;
  ecs_vec_int_t * vec_def_fac ;
  ecs_vec_int_t * vec_def_are ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(champ_def_fac != NULL) ;
  assert(champ_def_are != NULL) ;


  vec_def_fac = ecs_champ__initialise_vec_int(champ_def_fac) ;
  vec_def_are = ecs_champ__initialise_vec_int(champ_def_are) ;

  vec_connect_fac_som = ecs_vec_def__cree_fac_som(vec_def_fac,
                                                  vec_def_are) ;

  ecs_champ__libere_vec_int(champ_def_fac,
                            vec_def_fac) ;

  ecs_champ__libere_vec_int(champ_def_are,
                            vec_def_are) ;

  champ_connect_fac_som = ecs_champ__init_avec_vec_int(vec_connect_fac_som,
                                                       ECS_CHAMP_NOM_DEFINIT) ;

  champ_connect_fac_som->statut_e = ECS_CHAMP_STATUT_INDEFINI ;


  return champ_connect_fac_som ;


}


/*----------------------------------------------------------------------------
 *  Fonction realisant le decoupage des faces en triangles
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_def__dec_fac_tria
(
 ecs_champ_t        *      vect_champ_fac[],
 ecs_champ_t        *      vect_champ_are_new[],
 ecs_champ_t        *const champ_def_are,
 ecs_champ_t        *const champ_def_som,
 ecs_tab_int_t      *const liste_filtre,
 ecs_tab_int_t      *const liste_fac_dec
)
{

  size_t           nbr_are_new ;
  size_t           nbr_fac ;
  size_t           nbr_fac_new ;
  size_t           nbr_fac_val_new ;

  size_t           fac_nbr_are_max ;

  ecs_vec_int_t  * vec_def_fac_new ;
  ecs_vec_int_t  * vec_def_are_new ;
  ecs_vec_int_t  * vec_def_fac ;
  ecs_vec_int_t  * vec_def_are ;
  ecs_vec_real_t * vec_def_som ;
  ecs_vec_int_t  * vec_def_fac_dec ;

  ecs_champ_t    * champ_def_fac_dec ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(vect_champ_fac != NULL) ;
  assert(vect_champ_fac[ECS_CHAMP_DEF] != NULL) ;
  assert(vect_champ_fac[ECS_CHAMP_TYP] == NULL) ;

  vec_def_fac = ecs_champ__initialise_vec_int(vect_champ_fac[ECS_CHAMP_DEF]) ;
  vec_def_are = ecs_champ__initialise_vec_int(champ_def_are) ;
  vec_def_som = ecs_champ__initialise_vec_real(champ_def_som) ;

  nbr_fac = ecs_vec_int__ret_pos_nbr(vec_def_fac) - 1 ;


  /* Comptage pour allocations */

  ecs_vec_def__cpt_fac_tria(vec_def_fac,
                            liste_filtre,
                            &fac_nbr_are_max,
                            &nbr_fac_new,
                            &nbr_fac_val_new,
                            &nbr_are_new) ;


  /* Allocation et initialisation                                             */
  /*  pour les nouvelles (et anciennes) faces definies en fonction des aretes */
  /*  des vecteurs `ecs_vec_int_t' associes aux champs principaux             */

  vec_def_fac_new
    = ecs_vec_int__alloue(nbr_fac_new + 1,
                          nbr_fac_val_new);


  /* Allocation et initialisation                                 */
  /*  pour les nouvelles aretes definies en fonction des sommets  */
  /*  des vecteurs `ecs_vec_int_t' associes aux champs principaux */

  vec_def_are_new
    = ecs_vec_int__alloue (nbr_are_new + 1,
                           nbr_are_new * 2);


  /* Redefinition des faces en triangles (en fonctions des aretes) */
  /* Definition des nouvelles aretes issues du decoupage           */
  /* Renvoi de la definition                                       */
  /*  des anciennes faces en fonction des nouvelles (triangles)    */


  vec_def_fac_dec = ecs_vec_def__dec_fac_tria(vec_def_fac_new,
                                              vec_def_are_new,
                                              vec_def_fac,
                                              vec_def_are,
                                              vec_def_som,
                                              liste_filtre,
                                              fac_nbr_are_max,
                                              nbr_fac_new,
                                              nbr_are_new,
                                              liste_fac_dec) ;


  ecs_champ__libere_vec_int(champ_def_are,
                            vec_def_are) ;

  ecs_champ__libere_vec_real(champ_def_som,
                             vec_def_som) ;


  /*------------------------------------------------*/
  /* Construction du champ contenant la definition  */
  /*  des anciennes faces en fonction des nouvelles */
  /*------------------------------------------------*/


  champ_def_fac_dec = ecs_champ__init_avec_vec_int(vec_def_fac_dec,
                                                   ECS_CHAMP_NOM_DEFINIT) ;

  champ_def_fac_dec->statut_e = ECS_CHAMP_STATUT_INDEFINI ;



  /*--------------------------------------------------------*/
  /* Construction du vecteur des champs des nouvelles faces */
  /*--------------------------------------------------------*/


  /* Construction du champ contenant la definition */
  /*  des nouvelles faces en fonction des aretes   */
  /*-----------------------------------------------*/

  ecs_vec_int__detruit(vec_def_fac) ;

  nbr_fac_new = ecs_vec_int__ret_pos_nbr(vec_def_fac_new) - 1 ;

  ecs_champ__transfere_vec_int(vect_champ_fac[ECS_CHAMP_DEF],
                               vec_def_fac_new) ;


  /*---------------------------------------------------------*/
  /* Construction du vecteur des champs des nouvelles aretes */
  /*---------------------------------------------------------*/


  /* Construction du champ contenant la definition */
  /*  des nouvelles aretes en fonction des sommets */
  /*-----------------------------------------------*/

  nbr_are_new = ecs_vec_int__ret_pos_nbr(vec_def_are_new) - 1 ;


  if (nbr_are_new > 0) {

    vect_champ_are_new[ECS_CHAMP_DEF]
      = ecs_champ__init_avec_vec_int(vec_def_are_new,
                                     ECS_CHAMP_NOM_DEFINIT) ;

    vect_champ_are_new[ECS_CHAMP_DEF]->statut_e
      = ECS_CHAMP_STATUT_INDEFINI ;

  }
  else if (nbr_are_new == 0) {

    vect_champ_are_new[ECS_CHAMP_DEF] = NULL ;

    ecs_vec_int__detruit(vec_def_are_new) ;


    vect_champ_are_new[ECS_CHAMP_TYP] = NULL ;

  }


  return champ_def_fac_dec ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui construit un tableau de booleens conforme a une liste
 *   de sous-elements
 *  Un sous-element est a `ECS_TRUE'
 *   s'il intervient dans la definition des elements
 *----------------------------------------------------------------------------*/

void ecs_champ_def__cree_masque
(
 ecs_tab_bool_t        bool_sselt_select,
 ecs_champ_t    *const champ_def_elt
)
{

  ecs_vec_int_t * vec_def_elt ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  vec_def_elt = ecs_champ__initialise_vec_int(champ_def_elt) ;


  ecs_vec_def__cree_masque(bool_sselt_select,
                           vec_def_elt) ;


  ecs_champ__libere_vec_int(champ_def_elt,
                            vec_def_elt) ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui renvoie la liste des faces de type "polygone"
 *  (base sur la dfinition de faces par plus de 4 artes)
 *----------------------------------------------------------------------------*/

ecs_tab_int_t  ecs_champ_def__liste_fac_poly
(
 ecs_champ_t  *const champ_def_fac
)
{

  ecs_vec_int_t  * vec_def_fac ;

  ecs_tab_int_t  liste_fac_poly ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  vec_def_fac = ecs_champ__initialise_vec_int(champ_def_fac) ;


  liste_fac_poly = ecs_vec_def__lst_fac_poly(vec_def_fac) ;


  ecs_champ__libere_vec_int(champ_def_fac,
                            vec_def_fac) ;


  return liste_fac_poly ;


}


/*----------------------------------------------------------------------------
 *  Suppression des sommets ne participant pas  la connectivit
 *   (renvoie le champ de renumrotation des sommets, ou NULL si aucune
 *   renumrotation n'a t ncessaire).
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_def__nettoie_nodal
(
 ecs_champ_t       *const champ_def_som,
 ecs_champ_t       *const champ_def_are,
 ecs_champ_t       *const champ_def_fac,
 ecs_champ_t       *const champ_def_cel
)
{

  ecs_champ_t     * champ_som_old_new ;

  ecs_vec_int_t   * vec_som_old_new ;

  ecs_vec_real_t  * vec_def_som = NULL ;
  ecs_vec_int_t   * vec_def_fac = NULL ;
  ecs_vec_int_t   * vec_def_are = NULL ;
  ecs_vec_int_t   * vec_def_cel = NULL ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (champ_def_som == NULL)
    return NULL ;


  vec_def_som = ecs_champ__initialise_vec_real(champ_def_som) ;

  if (champ_def_are != NULL)
    vec_def_are = ecs_champ__initialise_vec_int(champ_def_are) ;

  if (champ_def_fac != NULL)
    vec_def_fac = ecs_champ__initialise_vec_int(champ_def_fac) ;

  if (champ_def_cel != NULL)
    vec_def_cel = ecs_champ__initialise_vec_int(champ_def_cel) ;


  vec_som_old_new = ecs_vec_def__nettoie_nodal(vec_def_som,
                                               vec_def_are,
                                               vec_def_fac,
                                               vec_def_cel) ;


  if (vec_def_are != NULL)
    ecs_champ__libere_vec_int(champ_def_are,
                              vec_def_are) ;

  if (vec_def_fac != NULL)
    ecs_champ__libere_vec_int(champ_def_fac,
                              vec_def_fac) ;

  if (vec_def_cel != NULL)
    ecs_champ__libere_vec_int(champ_def_cel,
                              vec_def_cel) ;


  /* Oprations ncessaires selon si l'on a effectu un compactage ou pas */

  if (vec_som_old_new != NULL) {

    ecs_champ__transfere_vec_real(champ_def_som,
                                  vec_def_som) ;


    champ_som_old_new = ecs_champ__init_avec_vec_int(vec_som_old_new,
                                                     NULL) ;

  }
  else {

    ecs_champ__libere_vec_real(champ_def_som,
                               vec_def_som) ;

    champ_som_old_new = NULL ;

  }


  return champ_som_old_new ;

}


/*----------------------------------------------------------------------------
 *  Correction si ncessaire de l'orientation des lments en
 *   connectivit nodale.
 *----------------------------------------------------------------------------*/

void ecs_champ_def__orient_nodal
(
       ecs_champ_t    *const champ_def_som,   /*  -> Dfinition des sommets   */
       ecs_champ_t    *const champ_def_fac,   /*  -> Dfinition des faces     */
       ecs_champ_t    *const champ_def_cel,   /*  -> Dfinition des cellules  */
       ecs_champ_t    *const champ_typ_cel,   /*  -> Types des cellules       */
       ecs_tab_int_t  *const liste_cel_err,   /* <-  Cels. erreur (option)    */
       ecs_tab_int_t  *const liste_cel_cor,   /* <-  Cels. correct. (option)  */
 const ecs_bool_t            correc_orient    /*  -> Correction ou non        */
)
{

  ecs_vec_real_t  * vec_def_som = NULL ;
  ecs_vec_int_t   * vec_def_fac = NULL ;
  ecs_vec_int_t   * vec_def_cel = NULL ;
  ecs_vec_int_t   * vec_typ_cel = NULL ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (champ_def_som == NULL)
    return ;


  vec_def_som = ecs_champ__initialise_vec_real(champ_def_som) ;

  if (champ_def_fac != NULL)
    vec_def_fac = ecs_champ__initialise_vec_int(champ_def_fac) ;

  if (champ_def_cel != NULL)
    vec_def_cel = ecs_champ__initialise_vec_int(champ_def_cel) ;

  if (champ_typ_cel != NULL)
    vec_typ_cel = ecs_champ__initialise_vec_int(champ_typ_cel) ;


  ecs_vec_def__orient_nodal(vec_def_som,
                            vec_def_fac,
                            vec_def_cel,
                            vec_typ_cel,
                            liste_cel_err,
                            liste_cel_cor,
                            correc_orient) ;


  ecs_champ__libere_vec_real(champ_def_som,
                             vec_def_som) ;

  if (vec_def_fac != NULL)
    ecs_champ__libere_vec_int(champ_def_fac,
                              vec_def_fac) ;

  if (vec_def_cel != NULL)
    ecs_champ__libere_vec_int(champ_def_cel,
                              vec_def_cel) ;

  if (vec_typ_cel != NULL)
    ecs_champ__libere_vec_int(champ_typ_cel,
                              vec_typ_cel) ;

}


/*----------------------------------------------------------------------------
 *  Fusion des sommets confondus d'aprs la longueur des artes.
 * La connectivit des artes est mise  jour.
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_def__nettoie_som_are
(
 ecs_champ_t       *const champ_def_som,
 ecs_champ_t       *const champ_def_are
)
{

  ecs_champ_t     * champ_som_old_new ;

  ecs_vec_int_t   * vec_som_old_new ;

  ecs_vec_real_t  * vec_def_som = NULL ;
  ecs_vec_int_t   * vec_def_are = NULL ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (champ_def_som == NULL || champ_def_are == NULL)
    return NULL ;


  vec_def_som = ecs_champ__initialise_vec_real(champ_def_som) ;
  vec_def_are = ecs_champ__initialise_vec_int(champ_def_are) ;


  vec_som_old_new = ecs_vec_def__nettoie_som_are(vec_def_som,
                                                 vec_def_are) ;


  if (vec_def_are != NULL)
    ecs_champ__libere_vec_int(champ_def_are,
                              vec_def_are) ;


  /* Oprations ncessaires selon si l'on a effectu un compactage ou pas */

  if (vec_som_old_new != NULL) {

    ecs_champ__transfere_vec_real(champ_def_som,
                                  vec_def_som) ;

    champ_som_old_new = ecs_champ__init_avec_vec_int(vec_som_old_new,
                                                     NULL) ;

  }
  else {

    ecs_champ__libere_vec_real(champ_def_som,
                               vec_def_som) ;

    champ_som_old_new = NULL ;

  }


  return champ_som_old_new ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui supprime les ventuelles artes dgnres
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_def__nettoie_are
(
 ecs_champ_t       *const champ_def_are
)
{

  ecs_champ_t     * champ_are_old_new ;

  ecs_vec_int_t   * vec_are_old_new ;

  ecs_vec_int_t   * vec_def_are = NULL ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (champ_def_are == NULL)
    return NULL ;


  vec_def_are = ecs_champ__initialise_vec_int(champ_def_are) ;


  vec_are_old_new = ecs_vec_def__nettoie_are(vec_def_are) ;


  /* Oprations ncessaires selon si l'on a effectu un compactage ou pas */

  if (vec_are_old_new != NULL) {

    ecs_champ__transfere_vec_int(champ_def_are,
                                 vec_def_are) ;

    champ_are_old_new = ecs_champ__init_avec_vec_int(vec_are_old_new,
                                                     NULL) ;

  }
  else {

    ecs_champ__libere_vec_int(champ_def_are,
                              vec_def_are) ;

    champ_are_old_new = NULL ;

  }


  return champ_are_old_new ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui supprime les ventuelles faces dgnres
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_def__nettoie_fac
(
 ecs_champ_t       *const champ_def_fac
)
{

  ecs_champ_t     * champ_fac_old_new ;

  ecs_vec_int_t   * vec_fac_old_new ;

  ecs_vec_int_t   * vec_def_fac = NULL ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (champ_def_fac == NULL)
    return NULL ;


  vec_def_fac = ecs_champ__initialise_vec_int(champ_def_fac) ;


  vec_fac_old_new = ecs_vec_def__nettoie_fac(vec_def_fac) ;


  /* Oprations ncessaires selon si l'on a effectu un compactage ou pas */

  if (vec_fac_old_new != NULL) {

    ecs_champ__transfere_vec_int(champ_def_fac,
                                 vec_def_fac) ;

    champ_fac_old_new = ecs_champ__init_avec_vec_int(vec_fac_old_new,
                                                     NULL) ;

  }
  else {

    ecs_champ__libere_vec_int(champ_def_fac,
                              vec_def_fac) ;

    champ_fac_old_new = NULL ;

  }


  return champ_fac_old_new ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui recolle les faces non conformes
 *
 *  Les listes des faces nouvelles ou modifies sont construites (et alloues)
 *  ici ; les structures liste_fac_new et liste_fac_mod correspondantes sont
 *  donc vides en entre ; idem pour liste_fac_err qui indiquera les indices
 *  des faces pour lesquelles le dcoupage en sous-faces a chou
 *
 *  On prend en entre soit une dfinition de type "visibilit" entre faces
 *   recoller (par exemple une filiation), ou alors une simple liste de
 *  faces slectionnes. L'un de vec_fac_vis et tab_fac_select doit donc
 *  tre  NULL, et l'autre non.
 *----------------------------------------------------------------------------*/

void ecs_champ_def__recolle
(
       ecs_champ_t       *const champ_def_fac,
       ecs_champ_t       *const champ_def_are,
       ecs_champ_t       *const champ_def_som,
       ecs_champ_t     * *const champ_fac_old_new,
       ecs_champ_t     * *const champ_are_old_new,
       ecs_champ_t     * *const champ_som_old_new,
       ecs_champ_t     * *const champ_fac_perio,
       ecs_champ_t       *const champ_fac_vis,
       ecs_tab_int_t     *const tab_fac_de_bord_select,
       ecs_tab_int_t     *const liste_fac_new,
       ecs_tab_int_t     *const liste_fac_mod,
       ecs_tab_int_t     *const liste_fac_err,
 const ecs_param_rc_t           param_rc
)
{

  ecs_vec_int_t  * vec_def_fac ;
  ecs_vec_int_t  * vec_def_are ;
  ecs_vec_real_t * vec_def_som ;

  ecs_vec_int_t  * vec_fac_old_new ;
  ecs_vec_int_t  * vec_are_old_new ;
  ecs_vec_int_t  * vec_som_old_new ;
  ecs_vec_int_t  * vec_fac_vis ;

  ecs_vec_int_t  * vec_fac_perio ;
  ecs_vec_int_t  * vec_are_perio ;
  ecs_vec_int_t  * vec_som_perio ;

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(champ_def_fac != NULL) ;
  assert(champ_def_are != NULL) ;
  assert(champ_def_som != NULL) ;


  vec_def_fac = ecs_champ__initialise_vec_int(champ_def_fac) ;
  vec_def_are = ecs_champ__initialise_vec_int(champ_def_are) ;
  vec_def_som = ecs_champ__initialise_vec_real(champ_def_som) ;

  if (champ_fac_vis != NULL)
    vec_fac_vis = ecs_champ__initialise_vec_int(champ_fac_vis) ;
  else
    vec_fac_vis = NULL ;

  vec_fac_old_new = NULL ;
  vec_are_old_new = NULL ;
  vec_som_old_new = NULL ;

  vec_fac_perio = NULL ;
  vec_are_perio = NULL ;
  vec_som_perio = NULL ;


  ecs_vec_def__recolle(&vec_def_fac,
                       &vec_def_are,
                       &vec_def_som,
                       &vec_fac_old_new,
                       &vec_are_old_new,
                       &vec_som_old_new,
                       vec_fac_vis,
                       &vec_fac_perio,
                       &vec_are_perio,
                       &vec_som_perio,
                       tab_fac_de_bord_select,
                       liste_fac_new,
                       liste_fac_mod,
                       liste_fac_err,
                       param_rc) ;


  if (champ_fac_vis != NULL)
    ecs_champ__libere_vec_int(champ_fac_vis,
                              vec_fac_vis) ;

  ecs_champ__transfere_vec_int(champ_def_fac,
                               vec_def_fac) ;

  ecs_champ__transfere_vec_int(champ_def_are,
                               vec_def_are) ;

  ecs_champ__transfere_vec_real(champ_def_som,
                                vec_def_som) ;


  *champ_fac_old_new = ecs_champ__init_avec_vec_int(vec_fac_old_new,
                                                    NULL) ;

  *champ_are_old_new = ecs_champ__init_avec_vec_int(vec_are_old_new,
                                                    NULL) ;

  *champ_som_old_new = ecs_champ__init_avec_vec_int(vec_som_old_new,
                                                    NULL) ;


  /* On reference le champ de correspondance des faces periodiques */

  if (param_rc.param_perio != NULL) {

    char *nom_perio ;


    BFT_MALLOC(nom_perio, strlen(ECS_CHAMP_NOM_FAC_PERIO) + 5 + 1, char) ;

    sprintf(nom_perio, "%s %4d", ECS_CHAMP_NOM_FAC_PERIO,
            (param_rc.param_perio)->num_perio) ;

    *champ_fac_perio = ecs_champ__init_avec_vec_int(vec_fac_perio, nom_perio) ;

    (*champ_fac_perio)->statut_e = ECS_CHAMP_STATUT_REF_ELT ;

    BFT_FREE(nom_perio) ;

  }


}


/*----------------------------------------------------------------------------
 *  Fonction qui retourne le vecteur des coordonnes des sommets
 *   pour le champ `ecs_champ_t' associ  des sommets.
 *
 *  La fonction retourne un pointeur sur le dbut du vecteur des coordonnes
 *   en cas de succs, NULL en cas d'chec (si le champ ne correspond
 *   pas  un champ rel de dimension 3).
 *----------------------------------------------------------------------------*/

ecs_real_t * ecs_champ_def__ret_coord_som
(
 const ecs_champ_t  *const champ_coord
)
{

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/

  assert(champ_coord != NULL) ;

  /* Si la table des valeurs n'a pas le bon type on retourne une erreur */

  if (champ_coord->typ_val !=  ECS_TYPE_ecs_real_t)
    return NULL ;

  /* Si la table des positions n'est pas une rgle on retourne une erreur */

  if (champ_coord->pos_tab != NULL)
    return NULL ;

  /* Si la dimension des coordonnes n'est pas 3,
     on retourne une erreur */

  if (champ_coord->pos_pas != ECS_DIM_3)
    return NULL ;

  /* On renvoie le pointeur sur les valeurs */

  return (ecs_real_t *)(champ_coord->val_tab) ;

}


/*============================================================================
 *                              Fonctions privees
 *============================================================================*/
