/*============================================================================
 *  Dfinitions des fonctions
 *   associes  la structure `ecs_champ_t' dcrivant un champ
 *   et propres aux calculs de type "grandeur"
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilit
 *============================================================================*/


/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <string.h> /* strlen() */


/*----------------------------------------------------------------------------
 *  Fichiers `include' systeme ou BFT
 *----------------------------------------------------------------------------*/

#include <bft_mem.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_chaine_glob.h"
#include "ecs_def.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles des paquetages visibles
 *----------------------------------------------------------------------------*/

#include "ecs_vec_int.h"
#include "ecs_vec_real.h"
#include "ecs_vec_grd.h"
#include "ecs_vec_grd.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_vec_int.h"
#include "ecs_champ_vec_real.h"
#include "ecs_champ.h"
#include "ecs_champ_chaine.h"


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associe au fichier courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_grd.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' prives   du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_priv.h"


/*============================================================================
 *                       Prototypes de fonctions prives
 *============================================================================*/


/*============================================================================
 *                             Fonctions publiques
 *============================================================================*/


/*----------------------------------------------------------------------------
 *  Fonction qui calcule les coordonnes minimales et maximales
 *----------------------------------------------------------------------------*/

void ecs_champ_grd__calc_coo_ext
(
 ecs_champ_t   *const champ_som_def
)
{

  ecs_vec_real_t  * vec_real_som_def     ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  vec_real_som_def = ecs_champ__initialise_vec_real(champ_som_def) ;

  ecs_vec_grd__calc_coo_ext(vec_real_som_def) ;

  ecs_champ__libere_vec_real(champ_som_def    ,
                             vec_real_som_def  ) ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui modifie les coordonnes du maillage
 *----------------------------------------------------------------------------*/

void ecs_champ_grd__transf_coo
(
 ecs_champ_t   *const champ_som_def,
 const double         matrice[3][4]
)
{

  ecs_vec_real_t  * vec_real_som_def ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  vec_real_som_def = ecs_champ__initialise_vec_real(champ_som_def) ;

  ecs_vec_grd__transf_coo(vec_real_som_def,
                          matrice) ;

  ecs_champ__transfere_vec_real(champ_som_def,
                                vec_real_som_def) ;

}



/*----------------------------------------------------------------------------
 *  Fonction qui calcule le maximum associ  une cellule d'un tableau de
 *   valeurs associes aux faces internes.
 *----------------------------------------------------------------------------*/

ecs_tab_real_t  ecs_champ_grd__max_fac_int
(
 const ecs_int_t               nbr_cel               ,
       ecs_champ_t      *const champ_connect_fac_cel ,
 const ecs_tab_int_t    *const liste_fac_interne     ,
 const ecs_tab_real_t   *const val_fac_interne       ,
 const ecs_real_t              val_min
)
{

  ecs_vec_int_t   * vec_int_connect_fac_cel  ;
  ecs_tab_real_t    val_real_cel  ;

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(champ_connect_fac_cel != NULL) ;

  vec_int_connect_fac_cel  =
    ecs_champ__initialise_vec_int(champ_connect_fac_cel) ;


  val_real_cel = ecs_vec_grd__max_fac_int(nbr_cel,
                                          vec_int_connect_fac_cel,
                                          liste_fac_interne,
                                          val_fac_interne,
                                          val_min) ;


  ecs_champ__libere_vec_int  (champ_connect_fac_cel, vec_int_connect_fac_cel) ;


  return val_real_cel ;

}


/*============================================================================
 *                              Fonctions prives
 *============================================================================*/
