/*============================================================================
 *  Dfinitions des fonctions
 *   associes  la structure `ecs_champ_t' dcrivant un champ
 *   et propres aux champs principaux de type "type gomtrique"
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilite
 *============================================================================*/


/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <string.h> /* strlen() */

#include <bft_mem.h>
#include <bft_printf.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"
#include "ecs_tab.h"

#include "ecs_chaine_glob.h"
#include "ecs_elt_typ_liste.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles des paquetages visibles
 *----------------------------------------------------------------------------*/

#include "ecs_vec_int.h"
#include "ecs_vec_def.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_vec_int.h"
#include "ecs_champ.h"


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associ au fichier courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_typ.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' privs   du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_priv.h"



/*============================================================================
 *                       Prototypes de fonctions prives
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui affiche le nombre d'lments de chaque type gomtrique
 *
 *  Cette fonction suppose que le tableau associ aux types gomtriques
 *  est ordonn suivant les types gomtriques
 *----------------------------------------------------------------------------*/

static void ecs_loc_champ_typ__affiche
(
 ecs_int_t               nbr_val,
 const ecs_int_t  *const tab_val
) ;


/*============================================================================
 *                             Fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui reconstruit les types gometriques des faces  partir de
 *  leur connectivit
 *----------------------------------------------------------------------------*/

ecs_champ_t  *  ecs_champ_typ__cree_typ_fac
(
 ecs_champ_t  * champ_def_fac
)
{

  ecs_champ_t    * champ_typ_geo ;
  ecs_vec_int_t  * vec_def_fac ;
  ecs_vec_int_t  * vec_typ_fac ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  vec_def_fac = ecs_champ__initialise_vec_int(champ_def_fac) ;


  vec_typ_fac = ecs_vec_def__cree_typ_fac(vec_def_fac) ;


  ecs_champ__libere_vec_int(champ_def_fac,
                            vec_def_fac) ;


  champ_typ_geo = ecs_champ__init_avec_vec_int(vec_typ_fac,
                                               ECS_CHAMP_NOM_TYP_GEO) ;

  champ_typ_geo->statut_e = ECS_CHAMP_STATUT_INDEFINI ;


  return champ_typ_geo ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui ralise le tri des types gomtriques
 *  La fonction affiche le nombre d'lments par type gomtrique
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_champ_typ__trie
(
 ecs_champ_t * this_champ_typ_geo
)
{

  ecs_tab_int_t   tab_typ_geo_ord ;
  ecs_tab_int_t   tab_typ_geo     ;

  ecs_tab_int_t   vect_renum      ;  /* Vecteur de renumrotation             */

  ecs_int_t     ielt            ;
  ecs_int_t     nbr_elt         ;  /* Nombre d'lments                       */


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(this_champ_typ_geo != NULL) ;

  vect_renum.nbr = 0    ;
  vect_renum.val = NULL ;

  nbr_elt = this_champ_typ_geo->nbr_elt ;

  if (nbr_elt == 0)
    return vect_renum ;


  assert(this_champ_typ_geo->pos_tab == NULL) ;
  assert(this_champ_typ_geo->pos_pas == 1) ;


  tab_typ_geo.val = this_champ_typ_geo->val_tab;
  tab_typ_geo.nbr = nbr_elt ;


  /* On regarde si les types gomtriques ne sont pas deja ordonns */

  ielt = 1 ;
  while (ielt < nbr_elt                                     &&
         tab_typ_geo.val[ielt] >= tab_typ_geo.val[ielt - 1]   )
    ielt++ ;

  if (ielt < nbr_elt) {

    /* Les types gomtriques ne sont pas ordonns */
    /* On ordonne les types gomtriques */


    vect_renum.nbr = nbr_elt ;
    BFT_MALLOC(vect_renum.val, nbr_elt, ecs_int_t) ;


    tab_typ_geo_ord = ecs_tab_int__trie_et_renvoie(tab_typ_geo,
                                                   vect_renum) ;

    /* `vect_renum' prend pour indice les indices nouveaux,           */
    /*  et ses valeurs contiennent les indices anciens correspondants */
    /* On inverse le contenu de `vect_renum' :                        */
    /*   chaque indice ancien, `vect_renum' donne la valeur          */
    /*  du nouvel indice                                              */

    ecs_tab_int__inverse(&vect_renum) ;

    BFT_FREE(this_champ_typ_geo->val_tab) ;

    this_champ_typ_geo->val_tab = tab_typ_geo_ord.val ;

  }
  /* else : rien a faire (les types gometriques sont deja ordonns */


  /* Message d'information sur la composition des lments du maillage */

  ecs_loc_champ_typ__affiche(nbr_elt,
                             this_champ_typ_geo->val_tab) ;


  /* Renvoi du vecteur de renumrotation */
  /*-------------------------------------*/


  return vect_renum ;

}


/*----------------------------------------------------------------------------
 *  Fonction ralisant l'hritage des valeurs du type gometrique d'un lment
 *   sur les sous-lments issus de sa dcomposition
 *----------------------------------------------------------------------------*/

ecs_champ_t * ecs_champ_typ__herite
(
       ecs_champ_t  *const champ_typ_elt       ,
       ecs_champ_t  *const champ_elt_def_sselt ,
 const ecs_int_t           nbr_sselt
)
{

  ecs_champ_t *this_champ_typ_sselt ;

  ecs_vec_int_t  *vec_typ_elt       ;
  ecs_vec_int_t  *vec_typ_sselt     ;
  ecs_vec_int_t  *vec_elt_def_sselt ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (champ_typ_elt != NULL) {

    vec_elt_def_sselt = ecs_champ__initialise_vec_int(champ_elt_def_sselt) ;


    /* Construction, pour les lments,                            */
    /*  du vecteur `ecs_vec_int_t' associ au champ "type gometrique" */
    /*-------------------------------------------------------------*/

    vec_typ_elt = ecs_champ__initialise_vec_int(champ_typ_elt) ;


    /* Allocation et initialisation pour les sous-lments,          */
    /*  des vecteurs `ecs_vec_int_t' associ au champ "type gometrique" */
    /*---------------------------------------------------------------*/


    vec_typ_sselt = ecs_vec_int__herite(vec_typ_elt,
                                        vec_elt_def_sselt,
                                        nbr_sselt) ;



    ecs_champ__libere_vec_int(champ_typ_elt,
                              vec_typ_elt) ;

    ecs_champ__libere_vec_int(champ_elt_def_sselt,
                              vec_elt_def_sselt) ;



    if (vec_typ_sselt != NULL) {

      /* Allocation et initialisation du champ "type geomtrique" */
      /*  de l'entit de maillage des sous-elmnts               */
      /*----------------------------------------------------------*/

      this_champ_typ_sselt = ecs_champ__init_avec_vec_int(vec_typ_sselt,
                                                          champ_typ_elt->nom) ;

      this_champ_typ_sselt->statut_e = ECS_CHAMP_STATUT_INDEFINI ;

    }
    else {

      this_champ_typ_sselt = NULL ;

    }

  }
  else {

    this_champ_typ_sselt = NULL ;

  }


  return this_champ_typ_sselt ;


}


/*============================================================================
 *                              Fonctions prives
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui affiche le nombre d'lments de chaque type gomtrique
 *
 *  Cette fonction suppose que le tableau associ aux types gomtriques
 *  est ordonn suivant les types gomtriques
 *----------------------------------------------------------------------------*/

static void ecs_loc_champ_typ__affiche
(
 ecs_int_t               nbr_val,
 const ecs_int_t  *const tab_val
)
{

  ecs_int_t    ival            ;
  ecs_int_t    cpt_val         ;
  ecs_int_t    lng_imp         ;
  ecs_int_t    nbr_val_typ_geo ;  /* Nbre d'lments de mme type gomtrique */
  ecs_int_t    val_typ_ref     ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  cpt_val = 0 ;

  while (cpt_val < nbr_val) {

    val_typ_ref = tab_val[cpt_val] ;

    /* On compte le nombre d'lments ayant le mme type gomtrique */

    nbr_val_typ_geo = 0 ;

    for (ival = cpt_val ;
         ival < nbr_val && tab_val[ival] == val_typ_ref ; ival++)
      nbr_val_typ_geo++ ;


    lng_imp = ECS_LNG_AFF_STR - ECS_ELT_TYP_LNG_MAX_NOM ;

    bft_printf("  %-*s%*.*s : %*" ECS_FORMAT_ecs_int_t "\n",
               lng_imp, _("Number of elements"),
               ECS_ELT_TYP_LNG_MAX_NOM, ECS_ELT_TYP_LNG_MAX_NOM,
               ecs_fic_elt_typ_liste_c[val_typ_ref].nom,
               ECS_LNG_AFF_ENT, nbr_val_typ_geo) ;

    cpt_val += nbr_val_typ_geo ;

  }

}

