/*============================================================================
 *  Dfinitions des fonctions de base
 *   associes a la structure `ecs_renum_t' dcrivant une renumrotation
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilit
 *============================================================================*/

#include "ecs_config.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>
#include <string.h>  /* memcmp() */

#include <bft_mem.h>
#include <bft_printf.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"
#include "ecs_tab.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles des paquetages visibles
 *----------------------------------------------------------------------------*/

#include "ecs_param_perio_glob.h"
#include "ecs_vec_int.h"
#include "ecs_vec_grd.h"

/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage courant
 *----------------------------------------------------------------------------*/

#if defined(HAVE_METIS)

#if defined(HAVE_METIS_H)

#include <metis.h>

#else

#ifdef __cplusplus
extern "C" {
#endif

/*
  Attention :
  -----------
  Les fichiers inclus par <metis.h> posent problme  minima sous Linux du 
  des redclarations de fonctions de <stdlib.h>. On recopie dans ce cas le
  minimum ncessaire issu de <metis.h> (correspondant  METIS 4.0)
*/

typedef int idxtype;
void METIS_PartGraphRecursive(int *, idxtype *, idxtype *, idxtype *, idxtype *,
                              int *, int *, int *, int *, int *, idxtype *);
void METIS_PartGraphKway(int *, idxtype *, idxtype *, idxtype *, idxtype *,
                         int *, int *, int *, int *, int *, idxtype *);

#ifdef __cplusplus
}
#endif

#endif /* defined(HAVE_METIS_H) */

#endif /* defined(HAVE_METIS) */


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associe au fichier courant
 *----------------------------------------------------------------------------*/

#include "ecs_renum.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' prives   du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_renum_priv.h"


/*============================================================================
 *                       Prototypes de fonctions prives
 *============================================================================*/

/*----------------------------------------------------------------------------
 *    Fonction de renumrotation des faces pour un seul domaine
 *----------------------------------------------------------------------------*/

void ecs_loc_renum__connect_fac_1
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
) ;


/*----------------------------------------------------------------------------
 *    Fonction de renumrotation locale des sommets.
 *    Les sommets sont utiliss pour des calculs gomtriques, mais ne
 *    portent pas de variables  ce jour. On ne cre donc pas de liste de
 *    correspondance entre sommets de domaines voisins, et on se contente
 *    de crer la connectivit locale  chaque sous-domaine.
 *----------------------------------------------------------------------------*/

void ecs_loc_renum__connect_som
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
) ;


/*----------------------------------------------------------------------------
 *    Fonction de renumrotation des sommets pour un seul domaine
 *----------------------------------------------------------------------------*/

static void ecs_loc_renum__connect_som_1
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
) ;



/*============================================================================
 *                             Fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui cre une structure de renumrotation
 *----------------------------------------------------------------------------*/

ecs_renum_t * ecs_renum__cree
(
 void
)
{

  ecs_renum_t   * renum ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  BFT_MALLOC(renum, 1, ecs_renum_t);


  renum->nbr_dom = 1 ;

  renum->nbr_per     = 0 ;
  renum->nbr_per_rot = 0 ;

  renum->nbr_cel = 0 ;
  renum->nbr_cel_loc = NULL ;
  renum->nbr_fac_par_loc = NULL ;

  renum->nbr_fac_interne = 0 ;
  renum->nbr_fac_de_bord = 0 ;
  renum->nbr_couples_perio   = 0 ;
  renum->nbr_fac_interne_loc = NULL ;
  renum->nbr_fac_de_bord_loc = NULL ;

  renum->nbr_som = 0 ;
  renum->nbr_som_loc = NULL ;

  renum->val_connect_fac_interne_cel = NULL ;
  renum->val_connect_fac_de_bord_cel = NULL ;
  renum->val_connect_fac_interne_cel_loc = NULL ;
  renum->val_connect_fac_de_bord_cel_loc = NULL ;

  renum->liste_fac_interne      = NULL ;
  renum->liste_fac_de_bord      = NULL ;
  renum->connect_fac_perio      = NULL ;

  renum->idx_fac_perio_loc   = NULL ;
  renum->liste_fac_perio_loc = NULL ;
  renum->num_dom_fac_perio   = NULL ;

  renum->pos_connect_fac_de_bord_som = NULL ;
  renum->pos_connect_fac_interne_som = NULL ;
  renum->val_connect_fac_de_bord_som = NULL ;
  renum->val_connect_fac_interne_som = NULL ;
  renum->pos_connect_fac_de_bord_som_loc = NULL ;
  renum->pos_connect_fac_interne_som_loc = NULL ;
  renum->val_connect_fac_de_bord_som_loc = NULL ;
  renum->val_connect_fac_interne_som_loc = NULL ;

  renum->num_glob_cel = NULL ;
  renum->num_glob_fac = NULL ;
  renum->num_glob_fbr = NULL ;
  renum->num_glob_som = NULL ;

  renum->num_dom_cel = NULL ;


  return renum ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui dtruit une structure de renumrotation
 *----------------------------------------------------------------------------*/

ecs_renum_t * ecs_renum__detruit
(
 ecs_renum_t *renum
)
{

  ecs_int_t   ind_dom ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;


  if (renum->nbr_dom > 1) {

    for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {

      if (renum->val_connect_fac_interne_cel_loc[ind_dom] !=
          renum->val_connect_fac_interne_cel)
        BFT_FREE(renum->val_connect_fac_interne_cel_loc[ind_dom]) ;

      if (renum->val_connect_fac_de_bord_cel_loc[ind_dom] !=
          renum->val_connect_fac_de_bord_cel)
        BFT_FREE(renum->val_connect_fac_de_bord_cel_loc[ind_dom]) ;

      if (renum->nbr_som > 0) {

        if ((renum->pos_connect_fac_interne_som_loc[ind_dom] != NULL) &&
            (renum->pos_connect_fac_interne_som_loc[ind_dom] !=
             renum->pos_connect_fac_interne_som)) {
          BFT_FREE(renum->val_connect_fac_interne_som_loc[ind_dom]) ;
          BFT_FREE(renum->pos_connect_fac_interne_som_loc[ind_dom]) ;
        }

        if ((renum->pos_connect_fac_de_bord_som_loc[ind_dom] != NULL) &&
            (renum->pos_connect_fac_de_bord_som_loc[ind_dom] !=
             renum->pos_connect_fac_de_bord_som)) {
          BFT_FREE(renum->val_connect_fac_de_bord_som_loc[ind_dom]) ;
          BFT_FREE(renum->pos_connect_fac_de_bord_som_loc[ind_dom]) ;
        }

      }

      if (renum->num_glob_cel != NULL)
        BFT_FREE(renum->num_glob_cel[ind_dom]) ;

      if (renum->num_glob_fac != NULL)
        BFT_FREE(renum->num_glob_fac[ind_dom]) ;

      if (renum->num_glob_fbr != NULL)
        BFT_FREE(renum->num_glob_fbr[ind_dom]) ;

      if (renum->num_glob_som != NULL)
        BFT_FREE(renum->num_glob_som[ind_dom]) ;

    }


    BFT_FREE(renum->num_dom_cel) ;

    BFT_FREE(renum->num_glob_cel) ;

    if (renum->num_glob_fac != NULL)
      BFT_FREE(renum->num_glob_fac) ;

    if (renum->num_glob_fbr != NULL)
      BFT_FREE(renum->num_glob_fbr) ;

    if (renum->num_glob_som != NULL)
      BFT_FREE(renum->num_glob_som);

    BFT_FREE(renum->nbr_fac_par_loc) ;

  }


  if (renum->nbr_per > 0) {

    /* On a besoin d'une structure identique au parallelisme pour la gestion
       des faces periodiques (meme dans le cas d'un calcul sequentiel) */

    /* Tableaux propres a la periodicite */

    for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {
      BFT_FREE(renum->idx_fac_perio_loc[ind_dom]) ;
      BFT_FREE(renum->liste_fac_perio_loc[ind_dom]);
    }

    if (renum->nbr_dom > 1) {
      for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {
        BFT_FREE(renum->num_dom_fac_perio[ind_dom]);
      }
    }
    else {
      BFT_FREE(renum->liste_fac_perio_loc[0]);
    }

    BFT_FREE(renum->connect_fac_perio) ;

    BFT_FREE(renum->idx_fac_perio_loc) ;
    BFT_FREE(renum->liste_fac_perio_loc) ;
    BFT_FREE(renum->num_dom_fac_perio) ;

  }


  if (renum->nbr_som > 0) {

    if (renum->pos_connect_fac_interne_som_loc != NULL) {
      BFT_FREE(renum->val_connect_fac_interne_som_loc) ;
      BFT_FREE(renum->pos_connect_fac_interne_som_loc) ;
    }

    if (renum->pos_connect_fac_de_bord_som_loc != NULL) {
      BFT_FREE(renum->val_connect_fac_de_bord_som_loc) ;
      BFT_FREE(renum->pos_connect_fac_de_bord_som_loc) ;
    }

  }

  BFT_FREE(renum->nbr_cel_loc);
  BFT_FREE(renum->nbr_fac_interne_loc) ;
  BFT_FREE(renum->nbr_fac_de_bord_loc) ;
  BFT_FREE(renum->nbr_som_loc);

  BFT_FREE(renum->val_connect_fac_interne_cel_loc) ;
  BFT_FREE(renum->val_connect_fac_de_bord_cel_loc) ;

  BFT_FREE(renum->liste_fac_interne);
  BFT_FREE(renum->liste_fac_de_bord);

  BFT_FREE(renum->val_connect_fac_interne_cel) ;
  BFT_FREE(renum->val_connect_fac_de_bord_cel) ;


  if (renum->pos_connect_fac_de_bord_som != NULL) {
    BFT_FREE(renum->val_connect_fac_de_bord_som) ;
    BFT_FREE(renum->pos_connect_fac_de_bord_som) ;
  }
  if (renum->pos_connect_fac_interne_som != NULL) {
    BFT_FREE(renum->val_connect_fac_interne_som) ;
    BFT_FREE(renum->pos_connect_fac_interne_som) ;
  }


  BFT_FREE(renum);


  return NULL ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui renvoie une structure ecs_tab_int_t correspondant aux
 *  numros de sous-domaine des cellules
 *----------------------------------------------------------------------------*/

ecs_tab_int_t ecs_renum__ret_num_domaine
(
 ecs_renum_t  * renum               /* --> Structure de renumerotation        */
)
{

  ecs_tab_int_t   num_domaine ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;


  num_domaine.nbr = renum->nbr_cel ;
  num_domaine.val = renum->num_dom_cel ;


  return num_domaine ;

}


/*----------------------------------------------------------------------------
 *    Fonction de dcoupage de maillage
 *----------------------------------------------------------------------------*/

void ecs_renum__decoup
(
 ecs_renum_t  * renum     ,         /* --> Structure de renumrotation        */
 ecs_int_t      nbr_dom             /* --> Nombre de blocs aprs dcoupage    */
)
{

  ecs_int_t * num_dom_cel ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;


  BFT_MALLOC(num_dom_cel, renum->nbr_cel, ecs_int_t);


#if defined(HAVE_METIS)

  {

    ecs_int_t   cpt_cel_ident ;

    int    nbr_cel      = (int)renum->nbr_cel ;
    int    nparts       = (int)nbr_dom ;

    int     wgtflag    = 0 ; /* Pas de pondration pour les faces ou cellules */
    int     numflag    = 1 ; /* Numrotation de 1  n (type Fortran) */
    int     options[5] = {0, 3, 1, 1, 0} ; /* Par dfaut si options[0] = 0 */
    int     edgecut    = 0 ; /* <-- nombre de faces sur la partition */
    size_t  ind ;
    int     ind_cel_1 ;
    int     ind_cel_2 ;

    idxtype  * nbr_connect ;
    idxtype  * pos_connect ;
    idxtype  * val_connect ;


    assert(sizeof(idxtype) == sizeof(ecs_int_t)) ; /* Sinon, rinstaller
                                                      METIS avec ixdtype
                                                      modifi dans
                                                      Lib/struct.h */

    /* On doit crer une connectivit cellules <-> cellules voisines */

    BFT_MALLOC(nbr_connect, renum->nbr_cel,             idxtype);
    BFT_MALLOC(pos_connect, renum->nbr_cel + 1,         idxtype);
    BFT_MALLOC(val_connect, renum->nbr_fac_interne * 2, idxtype);

    for (ind = 0 ; ind < renum->nbr_cel ; ind++)
      nbr_connect[ind] = 0 ;

    cpt_cel_ident = 0 ;


    for (ind = 0 ; ind < renum->nbr_fac_interne ; ind++) {

      ind_cel_1 = ECS_ABS(renum->val_connect_fac_interne_cel[ind * 2    ]) - 1 ;
      ind_cel_2 = ECS_ABS(renum->val_connect_fac_interne_cel[ind * 2 + 1]) - 1 ;

      if (ind_cel_1 != ind_cel_2) {
        nbr_connect[ind_cel_1] += 1 ;
        nbr_connect[ind_cel_2] += 1 ;
      }
      else
        cpt_cel_ident += 1 ;

    }


    BFT_REALLOC(val_connect,
                (renum->nbr_fac_interne - cpt_cel_ident)* 2,
                ecs_int_t) ;



    pos_connect[0] = 1 ;

    for (ind = 0 ; ind < renum->nbr_cel ; ind++)
      pos_connect[ind + 1] = pos_connect[ind] + nbr_connect[ind] ;


    for (ind = 0 ; ind < renum->nbr_cel ; ind++)
      nbr_connect[ind] = 0 ;


    for (ind = 0 ; ind < renum->nbr_fac_interne ; ind++) {

      ind_cel_1 = ECS_ABS(renum->val_connect_fac_interne_cel[ind * 2    ]) - 1 ;
      ind_cel_2 = ECS_ABS(renum->val_connect_fac_interne_cel[ind * 2 + 1]) - 1 ;

      /*
         Test de sortie si ind_cel_1 == ind_cel_2.
         Cas possible dans certaines configuration de periodicite :
            - rotation de 180 degres avec axe coupant une face
            - maillage trop mince (une cellule d'epaisseur)
            - rotation avec un coin 'non decoupe'
      */

      if (ind_cel_1 != ind_cel_2) {

        val_connect[pos_connect[ind_cel_1] + nbr_connect[ind_cel_1] - 1]
          = ind_cel_2 + 1 ;
        val_connect[pos_connect[ind_cel_2] + nbr_connect[ind_cel_2] - 1]
          = ind_cel_1 + 1;
        nbr_connect[ind_cel_1] += 1 ;
        nbr_connect[ind_cel_2] += 1 ;

      }

    }

    BFT_FREE(nbr_connect);


    if (nparts < 8)

      METIS_PartGraphRecursive(&nbr_cel,
                               pos_connect,
                               val_connect,
                               NULL,       /* vwgt   : poids des cellules */
                               NULL,       /* adjwgt : poids des faces    */
                               &wgtflag,
                               &numflag,
                               &nparts,
                               options,
                               &edgecut,
                               num_dom_cel) ;

    else

      METIS_PartGraphKway(&nbr_cel,
                          pos_connect,
                          val_connect,
                          NULL,       /* vwgt   : poids des cellules */
                          NULL,       /* adjwgt : poids des faces    */
                          &wgtflag,
                          &numflag,
                          &nparts,
                          options,
                          &edgecut,
                          num_dom_cel) ;

    BFT_FREE(pos_connect);
    BFT_FREE(val_connect);

  }

#else

  {
    ecs_int_t   ind_cel  ;
    ecs_int_t   diviseur ;

    diviseur = renum->nbr_cel / nbr_dom ;

    if (nbr_dom * diviseur < renum->nbr_cel)
      diviseur += 1;


    for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++)

      num_dom_cel[ind_cel] = (ind_cel / diviseur) + 1 ;

  }

#endif

  renum->nbr_dom     = nbr_dom   ;
  renum->num_dom_cel = num_dom_cel ;

}


/*----------------------------------------------------------------------------
 *    Fonction de renumrotation des faces
 *----------------------------------------------------------------------------*/

void ecs_renum__connect_fac
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
)
{

  int             signe ;

  ecs_bool_t      fac_est_perio ;
  ecs_int_t       ind_per ;
  ecs_int_t       ind_dom ;
  ecs_int_t       ind_dom_vis ;
  size_t          ind_cel ;
  size_t          ind_cel_vis ;
  size_t          ind_fac ;
  size_t          ipos ;
  size_t          ival ;

  ecs_int_t       num_per ;
  ecs_int_t       num_cel ;
  ecs_int_t       num_fac_per_1 ;
  ecs_int_t       num_fac_per_2 ;
  size_t          nbr_val ;

  ecs_int_t     * ind_fac_perio ;

  ecs_int_t     * num_loc_cel ;
  ecs_size_t    * nbr_cel_loc ;
  ecs_size_t    * nbr_fac_par_loc ;
  ecs_int_t   * * nbr_coupl_perio_loc ;
  ecs_size_t    * nbr_fac_interne_loc ;
  ecs_size_t    * nbr_fac_de_bord_loc ;

  ecs_int_t   * * num_glob_cel ;
  ecs_int_t   * * num_glob_fac ;
  ecs_int_t   * * num_glob_fbr ;

  ecs_int_t   * * val_connect_fac_interne_cel_loc ;
  ecs_int_t   * * val_connect_fac_de_bord_cel_loc ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;


  if (renum->nbr_dom == 1) {

    ecs_loc_renum__connect_fac_1(renum) ;
    return ;

  }

  /* Initialisations */

  ind_fac_perio = NULL ;

  num_per = 0 ;
  num_fac_per_1 = -1 ;
  num_fac_per_2 = -1 ;
  ind_per = -1 ;


  BFT_MALLOC(num_loc_cel, renum->nbr_cel, ecs_int_t) ;


  BFT_MALLOC(nbr_cel_loc, renum->nbr_dom, ecs_size_t) ;
  BFT_MALLOC(nbr_fac_par_loc, renum->nbr_dom, ecs_size_t) ;

  renum->nbr_cel_loc     = nbr_cel_loc ;
  renum->nbr_fac_par_loc = nbr_fac_par_loc ;


  /* Numrotation locale des cellules */
  /* -------------------------------- */

  for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {

    nbr_cel_loc[ind_dom] = 0 ;
    nbr_fac_par_loc[ind_dom] = 0 ;

  }


  /* Comptage */

  for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++) {

    ind_dom = renum->num_dom_cel[ind_cel] - 1 ;

    num_loc_cel[ind_cel] = ++(nbr_cel_loc[ind_dom]) ;

  }


  /* Marquage des faces priodiques */

  if (renum->nbr_per > 0) {

    BFT_MALLOC(ind_fac_perio, renum->nbr_fac_interne, ecs_int_t) ;
    for (ind_fac = 0 ; ind_fac < renum->nbr_fac_interne; ind_fac++)
      ind_fac_perio[ind_fac] = -1 ;

    for (ind_fac = 0 ; ind_fac < renum->nbr_couples_perio ; ind_fac++) {
      ind_fac_perio[renum->connect_fac_perio[ind_fac*3    ] - 1] = ind_fac ;
      ind_fac_perio[renum->connect_fac_perio[ind_fac*3 + 1] - 1] = ind_fac ;
    }

    BFT_MALLOC(nbr_coupl_perio_loc, renum->nbr_dom, ecs_int_t *) ;
    for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {
      BFT_MALLOC(nbr_coupl_perio_loc[ind_dom], renum->nbr_per, ecs_int_t) ;
      for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++)
        nbr_coupl_perio_loc[ind_dom][ind_per] = 0 ;
    }

  }


  /* Allocations */

  BFT_MALLOC(num_glob_cel, renum->nbr_dom, ecs_int_t *) ;

  for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++)
    BFT_MALLOC(num_glob_cel[ind_dom], nbr_cel_loc[ind_dom], ecs_int_t) ;

  renum->num_glob_cel    = num_glob_cel ;


  /* Remplissage du numro global de cellule (par domaine) */

  for (ind_cel = 0 ; ind_cel < renum->nbr_cel ; ind_cel++) {

    ind_dom = renum->num_dom_cel[ind_cel] - 1 ;

    renum->num_glob_cel[ind_dom][num_loc_cel[ind_cel] - 1] = ind_cel + 1 ;

  }


  /* Traitement des faces */
  /* -------------------- */

  /* Comptage des faces internes par domaine */

  BFT_MALLOC(nbr_fac_interne_loc, renum->nbr_dom, ecs_size_t) ;
  BFT_MALLOC(nbr_fac_de_bord_loc, renum->nbr_dom, ecs_size_t) ;

  for (ind_dom = 0 ;
       ind_dom < renum->nbr_dom ;
       ind_dom++) {

    nbr_fac_interne_loc[ind_dom] = 0 ;
    nbr_fac_de_bord_loc[ind_dom] = 0 ;

  }


  /* Comptage du nombre de faces internes et sur frontire parallle */

  for (ind_fac = 0 ; ind_fac < renum->nbr_fac_interne ; ind_fac++) {

    ind_cel
      = ECS_ABS(renum->val_connect_fac_interne_cel[ind_fac * 2])     - 1 ;
    ind_cel_vis
      = ECS_ABS(renum->val_connect_fac_interne_cel[ind_fac * 2 + 1]) - 1 ;

    ind_dom     = renum->num_dom_cel[ind_cel]     - 1 ;
    ind_dom_vis = renum->num_dom_cel[ind_cel_vis] - 1 ;

    nbr_fac_interne_loc[ind_dom] += 1 ;

    if (ind_dom != ind_dom_vis) {

      nbr_fac_interne_loc[ind_dom_vis] += 1 ;

      nbr_fac_par_loc[ind_dom] += 1 ;
      nbr_fac_par_loc[ind_dom_vis] += 1 ;

    }

    /* Comptage supplmentaire pour la priodicit */

    if (ind_fac_perio != NULL) {

      if (ind_fac_perio[ind_fac] != -1) {

        num_per = renum->connect_fac_perio[ind_fac_perio[ind_fac]*3 + 2] ;
        ind_per = ECS_ABS(num_per) - 1 ;

        num_fac_per_1 = renum->connect_fac_perio[ind_fac_perio[ind_fac]*3] ;

        /* On ne dfinit le couple de faces priodiques qu'un fois par domaine
           dans la structure joints associe */

        if (num_fac_per_1 == (ecs_int_t)(ind_fac + 1)) {

          nbr_coupl_perio_loc[ind_dom][ind_per] += 1 ;
          if (ind_dom != ind_dom_vis)
            nbr_coupl_perio_loc[ind_dom_vis][ind_per] += 1 ;

        }

      }

    }

  }


  /* Comptage des faces de bord par domaine */

  for (ind_fac = 0 ; ind_fac < renum->nbr_fac_de_bord ; ind_fac++) {

    ind_cel = renum->val_connect_fac_de_bord_cel[ind_fac] - 1 ;
    ind_dom = renum->num_dom_cel[ind_cel] - 1 ;
    nbr_fac_de_bord_loc[ind_dom] += 1 ;

  }


  renum->nbr_fac_interne_loc = nbr_fac_interne_loc ;
  renum->nbr_fac_de_bord_loc = nbr_fac_de_bord_loc ;


  /* Construction de l'index pour les joints priodiques et remise 
     zro du compteur de faces priodiques par domaine et priodicit */

  if (renum->nbr_per > 0) {

    BFT_MALLOC(renum->idx_fac_perio_loc, renum->nbr_dom, ecs_int_t *) ;
    BFT_MALLOC(renum->liste_fac_perio_loc, renum->nbr_dom, ecs_int_t *) ;
    BFT_MALLOC(renum->num_dom_fac_perio, renum->nbr_dom, ecs_int_t *);


    for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {

      BFT_MALLOC(renum->idx_fac_perio_loc[ind_dom],
                 renum->nbr_per + 1,
                 ecs_int_t) ;

      renum->idx_fac_perio_loc[ind_dom][0] = 0 ;
      for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++) {
        renum->idx_fac_perio_loc[ind_dom][ind_per + 1]
          =   renum->idx_fac_perio_loc[ind_dom][ind_per]
            + nbr_coupl_perio_loc[ind_dom][ind_per] ;
        nbr_coupl_perio_loc[ind_dom][ind_per] = 0 ;
      }

      BFT_MALLOC(renum->liste_fac_perio_loc[ind_dom],
                 renum->idx_fac_perio_loc[ind_dom][renum->nbr_per] * 2,
                 ecs_int_t);

      BFT_MALLOC(renum->num_dom_fac_perio[ind_dom],
                 renum->idx_fac_perio_loc[ind_dom][renum->nbr_per],
                 ecs_int_t);

    }

  }

  /* Allocations */

  BFT_MALLOC(num_glob_fac, renum->nbr_dom, ecs_int_t *) ;
  BFT_MALLOC(num_glob_fbr, renum->nbr_dom, ecs_int_t *) ;

  BFT_MALLOC(val_connect_fac_interne_cel_loc, renum->nbr_dom, ecs_int_t *) ;
  BFT_MALLOC(val_connect_fac_de_bord_cel_loc, renum->nbr_dom, ecs_int_t *) ;

  for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {

    BFT_MALLOC(num_glob_fac[ind_dom],
               nbr_fac_interne_loc[ind_dom], ecs_int_t) ;
    BFT_MALLOC(num_glob_fbr[ind_dom],
               nbr_fac_de_bord_loc[ind_dom], ecs_int_t) ;

    BFT_MALLOC(val_connect_fac_interne_cel_loc[ind_dom],
               nbr_fac_interne_loc[ind_dom] * 2, ecs_int_t) ;
    BFT_MALLOC(val_connect_fac_de_bord_cel_loc[ind_dom],
               nbr_fac_de_bord_loc[ind_dom], ecs_int_t) ;

  }

  renum->num_glob_fac = num_glob_fac ;
  renum->num_glob_fbr = num_glob_fbr ;

  renum->val_connect_fac_interne_cel_loc = val_connect_fac_interne_cel_loc ;
  renum->val_connect_fac_de_bord_cel_loc = val_connect_fac_de_bord_cel_loc ;


  /* Traitement des faces internes parallles */

  for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {
    nbr_fac_interne_loc[ind_dom] = 0 ;
    nbr_fac_par_loc[ind_dom] = 0 ;
  }


  /* Boucle principale sur les faces internes */

  for (ind_fac = 0 ; ind_fac < renum->nbr_fac_interne ; ind_fac++) {

    ind_cel
      = ECS_ABS(renum->val_connect_fac_interne_cel[ind_fac * 2])     - 1 ;
    ind_cel_vis
      = ECS_ABS(renum->val_connect_fac_interne_cel[ind_fac * 2 + 1]) - 1 ;

    ind_dom     = renum->num_dom_cel[ind_cel]     - 1 ;
    ind_dom_vis = renum->num_dom_cel[ind_cel_vis] - 1 ;

    /* On incrmente le compteur des faces sur frontire parallle */

    if (ind_dom != ind_dom_vis) {

      nbr_fac_par_loc[ind_dom] += 1 ;
      nbr_fac_par_loc[ind_dom_vis] += 1 ;

    }

    /* On effectuera un traitement diffrent si la face est priodique */

    fac_est_perio = ECS_FALSE ;
    if (ind_fac_perio != NULL) {
      if (ind_fac_perio[ind_fac] != -1)
        fac_est_perio = ECS_TRUE ;
    }

    /* Si l'on est sur une face interne non priodique */

    if (ind_dom == ind_dom_vis && fac_est_perio == ECS_FALSE) {

      val_connect_fac_interne_cel_loc[ind_dom]
                                     [nbr_fac_interne_loc[ind_dom] * 2]
        = num_loc_cel[ind_cel] ;
      val_connect_fac_interne_cel_loc[ind_dom]
                                     [nbr_fac_interne_loc[ind_dom] * 2 + 1]
        = num_loc_cel[ind_cel_vis] ;

      /* Correspondance numro local <-> numro global */

      num_glob_fac[ind_dom][nbr_fac_interne_loc[ind_dom]] = ind_fac + 1 ;
      nbr_fac_interne_loc[ind_dom] += 1 ;

    }

    /* Si l'on est sur une frontire parallle non priodique */

    else if (fac_est_perio == ECS_FALSE) { /* ind_dom != ind_dom_vis */

      /* La connectivit se fait par rapport aux cellules tendues,
         qui s'ajouteront en fin de liste des cellules locales ;
         ces cellules tant construites par le Noyau, on leur
         affecte temporairement le numro 0. */

      val_connect_fac_interne_cel_loc[ind_dom]
                                     [nbr_fac_interne_loc[ind_dom] * 2]
        = num_loc_cel[ind_cel] ;
      val_connect_fac_interne_cel_loc[ind_dom]
                                     [nbr_fac_interne_loc[ind_dom] * 2 + 1]
        = 0 ;

      val_connect_fac_interne_cel_loc[ind_dom_vis]
                                     [nbr_fac_interne_loc[ind_dom_vis] * 2]
        = 0 ;
      val_connect_fac_interne_cel_loc[ind_dom_vis]
                                     [nbr_fac_interne_loc[ind_dom_vis] * 2 + 1]
        = num_loc_cel[ind_cel_vis] ;

      /* Correspondance numro local <-> numro global */

      num_glob_fac[ind_dom][nbr_fac_interne_loc[ind_dom]] = ind_fac + 1 ;
      nbr_fac_interne_loc[ind_dom] += 1 ;

      num_glob_fac[ind_dom_vis][nbr_fac_interne_loc[ind_dom_vis]] = ind_fac + 1 ;
      nbr_fac_interne_loc[ind_dom_vis] += 1 ;

    }

    /* Si la face est priodique */

    else {

      assert(fac_est_perio == ECS_TRUE) ;

      num_per = renum->connect_fac_perio[ind_fac_perio[ind_fac]*3 + 2] ;
      ind_per = ECS_ABS(num_per) - 1 ;

      num_cel = renum->val_connect_fac_interne_cel[ind_fac * 2] ;

      num_fac_per_1 = renum->connect_fac_perio[ind_fac_perio[ind_fac]*3] ;
      num_fac_per_2 = renum->connect_fac_perio[ind_fac_perio[ind_fac]*3 + 1] ;

      /* On utilise un numro ngatif pour la connectivit de la cellule
         "correspondante" (par construction) */

      if (num_cel > 0) {

        val_connect_fac_interne_cel_loc[ind_dom]
                                       [nbr_fac_interne_loc[ind_dom] * 2]
          = num_loc_cel[ind_cel] ;
        val_connect_fac_interne_cel_loc[ind_dom]
                                       [nbr_fac_interne_loc[ind_dom] * 2 + 1]
          = 0 ;

        /* Correspondance numro local <-> numro global */

        num_glob_fac[ind_dom][nbr_fac_interne_loc[ind_dom]] = ind_fac + 1 ;
        ind_fac_perio[ind_fac] = nbr_fac_interne_loc[ind_dom] ;

        nbr_fac_interne_loc[ind_dom] += 1 ;

      }
      else {

        assert(renum->val_connect_fac_interne_cel[ind_fac * 2 + 1] > 0) ;

        val_connect_fac_interne_cel_loc[ind_dom_vis]
                                       [nbr_fac_interne_loc[ind_dom_vis] * 2]
          = 0 ;
        val_connect_fac_interne_cel_loc[ind_dom_vis]
                                       [nbr_fac_interne_loc[ind_dom_vis] * 2 + 1]
          = num_loc_cel[ind_cel_vis] ;

        /* Correspondance numro local <-> numro global */

        num_glob_fac[ind_dom_vis][nbr_fac_interne_loc[ind_dom_vis]]
          = ind_fac + 1 ;
        ind_fac_perio[ind_fac] = nbr_fac_interne_loc[ind_dom_vis] ;

        nbr_fac_interne_loc[ind_dom_vis] += 1 ;

      }

      /* On ne dfinit le couple de faces priodiques qu'un fois par domaine
         dans la structure joints associe ; On utilise une numrotation
         temporaire des faces des joints base sur la numrotation globale
         (car toutes les faces locales peuvent ne pas encore avoir t
         construites) */

      if (num_fac_per_1 == (ecs_int_t)(ind_fac + 1)) {

        /* renseignement de renum->liste_fac_perio_loc[ind_dom] */

        ipos = (  renum->idx_fac_perio_loc[ind_dom][ind_per]
                + nbr_coupl_perio_loc[ind_dom][ind_per]) * 2 ;

        if (num_cel > 0) {
          renum->liste_fac_perio_loc[ind_dom][ipos]     = num_fac_per_1 ;
          renum->liste_fac_perio_loc[ind_dom][ipos + 1] = num_fac_per_2 ;
        }
        else {
          renum->liste_fac_perio_loc[ind_dom][ipos]     = num_fac_per_2 ;
          renum->liste_fac_perio_loc[ind_dom][ipos + 1] = num_fac_per_1 ;
        }

        /* Orientation */

        signe = 1 ;
        if (num_per < 0)
          signe = -signe ;
        if (num_cel < 0)
          signe = -signe ;

        if (signe < 0)
          renum->liste_fac_perio_loc[ind_dom][ipos]
            = - renum->liste_fac_perio_loc[ind_dom][ipos] ;

        /* renseignement de renum->num_dom_fac_perio[ind_dom] */

        ipos = (  renum->idx_fac_perio_loc[ind_dom][ind_per]
                + nbr_coupl_perio_loc[ind_dom][ind_per]) ;

        renum->num_dom_fac_perio[ind_dom][ipos] = ind_dom_vis + 1 ;

        /* mise  jour du compteur */

        nbr_coupl_perio_loc[ind_dom][ind_per] += 1 ;

        /* Si les 2 faces priodiques n'appartiennent pas au mme
           domaine, on renseigne le couple pour chaque domaine */

        if (ind_dom != ind_dom_vis) {

          /* renseignement de renum->liste_fac_perio_loc[ind_dom_vis] */

          ipos = (  renum->idx_fac_perio_loc[ind_dom_vis][ind_per]
                  + nbr_coupl_perio_loc[ind_dom_vis][ind_per]) * 2 ;

          if (num_cel > 0) {
            renum->liste_fac_perio_loc[ind_dom_vis][ipos]     = num_fac_per_2 ;
            renum->liste_fac_perio_loc[ind_dom_vis][ipos + 1] = num_fac_per_1 ;
          }
          else {
            renum->liste_fac_perio_loc[ind_dom_vis][ipos]     = num_fac_per_1 ;
            renum->liste_fac_perio_loc[ind_dom_vis][ipos + 1] = num_fac_per_2 ;
          }

          if (signe > 0)
            renum->liste_fac_perio_loc[ind_dom_vis][ipos]
              = - renum->liste_fac_perio_loc[ind_dom_vis][ipos] ;

          /* renseignement de renum->num_dom_fac_perio[ind_dom_vis] */

          ipos = (  renum->idx_fac_perio_loc[ind_dom_vis][ind_per]
                    + nbr_coupl_perio_loc[ind_dom_vis][ind_per]) ;

          renum->num_dom_fac_perio[ind_dom_vis][ipos] = ind_dom + 1 ;

          /* mise  jour du compteur */

          nbr_coupl_perio_loc[ind_dom_vis][ind_per] += 1 ;

        }

      }

    } /* Fin du test sur le type de face */

  } /* Fin de la boucle principale sur les faces */


  /* Boucle de numrotation locale des faces priodiques */

  if (ind_fac_perio != NULL) {

    for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {

      nbr_val = (renum->idx_fac_perio_loc[ind_dom][renum->nbr_per]) * 2 ;

      for (ival = 0 ; ival < nbr_val ; ival++) {
        ecs_int_t num_fac = renum->liste_fac_perio_loc[ind_dom][ival] ;
        if (num_fac > 0)
          renum->liste_fac_perio_loc[ind_dom][ival]
            = ind_fac_perio[num_fac - 1] + 1 ;
        else
          renum->liste_fac_perio_loc[ind_dom][ival]
            = - (ind_fac_perio[-num_fac - 1] + 1) ;
      }

    }

  } /* Fin de la boucle de numrotation locale des faces priodiques */


  if(renum->nbr_per > 0) {
    BFT_FREE(ind_fac_perio) ;
    for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++)
      BFT_FREE(nbr_coupl_perio_loc[ind_dom]);
    BFT_FREE(nbr_coupl_perio_loc);
  }


  /* Traitement des faces de bord parallles */

  for (ind_dom = 0 ;
       ind_dom < renum->nbr_dom ;
       nbr_fac_de_bord_loc[ind_dom++] = 0) ;


  for (ind_fac = 0 ; ind_fac < renum->nbr_fac_de_bord ; ind_fac++) {

    ind_cel = renum->val_connect_fac_de_bord_cel[ind_fac] - 1 ;
    ind_dom = renum->num_dom_cel[ind_cel] - 1 ;

    val_connect_fac_de_bord_cel_loc[ind_dom][nbr_fac_de_bord_loc[ind_dom]]
      = num_loc_cel[ind_cel] ;

    num_glob_fbr[ind_dom][nbr_fac_de_bord_loc[ind_dom]] = ind_fac + 1 ;
    nbr_fac_de_bord_loc[ind_dom] += 1 ;

  }

  BFT_FREE(num_loc_cel) ;


  /* Traitement de la connectivit faces <-> sommets */

  ecs_loc_renum__connect_som(renum) ;

}


/*----------------------------------------------------------------------------
 *    Fonction d'affichage des infos sur le domaine principal
 *----------------------------------------------------------------------------*/

void ecs_renum__print_info
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
)
{


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;

  bft_printf(_("\n\nMain mesh properties\n"
               "--------------------\n\n")) ;

  bft_printf(_("  Number of cells:                            %10d\n"
               "  Number of internal faces:                   %10d\n"),
             renum->nbr_cel,
             renum->nbr_fac_interne) ;


  if (renum->nbr_per > 0)
    bft_printf(_("    including periodic faces:                 %10d\n"),
               renum->nbr_couples_perio * 2) ;


  bft_printf(_("  Number of boundary faces:                   %10d\n"),
             renum->nbr_fac_de_bord) ;


  if (renum->nbr_som != 0)
    bft_printf(_("  Number of vertices:                         %10d\n"),
               renum->nbr_som) ;

}


/*----------------------------------------------------------------------------
 *    Fonction d'affichage des infos sur les sous-domaines
 *----------------------------------------------------------------------------*/

void ecs_renum__print_info_loc
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
)
{

  ecs_int_t       ind_per ;
  ecs_int_t       ind_dom ;
  ecs_int_t       ind_fac ;

  ecs_int_t       nbr_fac_crt ;
  ecs_int_t       nbr_fac_par ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;

  if (renum->nbr_dom == 1)

    return ;


  for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {


    bft_printf(_("  Sub-domain %d\n\n"
                 "    Number of cells :                         %10d\n"
                 "    Number of internal faces:                 %10d\n"
                 "      including faces on parallel boundaries: %10d\n"
                 "    Number of boundary faces:                 %10d\n\n"),
               ind_dom + 1,
               renum->nbr_cel_loc[ind_dom],
               renum->nbr_fac_interne_loc[ind_dom],
               renum->nbr_fac_par_loc[ind_dom],
               renum->nbr_fac_de_bord_loc[ind_dom]) ;


    /* Impressions relatives  la priodicit */

    for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++) {

      nbr_fac_crt = 0 ;
      nbr_fac_par = 0 ;

      for (ind_fac = renum->idx_fac_perio_loc[ind_dom][ind_per] ;
           ind_fac < renum->idx_fac_perio_loc[ind_dom][ind_per + 1] ;
           ind_fac++) {
        if (renum->num_dom_fac_perio[ind_dom][ind_fac] == ind_dom + 1)
          nbr_fac_crt += 1;
        else
          nbr_fac_par += 1;
      }

      bft_printf(_("    Periodicity %1d:\n"), ind_per + 1) ;
      bft_printf(_("      Number of periodic faces in same sub-domain: "
                   " %10d\n"),
                   nbr_fac_crt) ;
      bft_printf(_("      Number of periodic faces in other sub-domain:"
                   " %10d\n\n"),
                 nbr_fac_par) ;

    }

  }


}


/*============================================================================
 *                              Fonctions prives
 *============================================================================*/


/*----------------------------------------------------------------------------
 *    Fonction de renumrotation des faces pour un seul domaine
 *----------------------------------------------------------------------------*/

void ecs_loc_renum__connect_fac_1
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
)
{

  ecs_int_t       ind_per ;
  ecs_int_t       num_cel ;
  ecs_int_t       num_cel_vis ;
  size_t          ind_fac ;
  size_t          ipos_1 ;

  ecs_int_t       num_per ;
  ecs_int_t       num_fac_per_1 ;
  ecs_int_t       num_fac_per_2 ;

  ecs_int_t     * ind_fac_perio ;

  ecs_size_t    * nbr_cel_loc ;
  ecs_int_t   * * nbr_coupl_perio_loc ;
  ecs_size_t    * nbr_fac_interne_loc ;
  ecs_size_t    * nbr_fac_de_bord_loc ;

  ecs_int_t   * * val_connect_fac_interne_cel_loc ;
  ecs_int_t   * * val_connect_fac_de_bord_cel_loc ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;


  /* Initialisations */

  ind_fac_perio = NULL ;

  num_per = 0 ;
  num_fac_per_1 = -1 ;
  num_fac_per_2 = -1 ;
  ind_per = -1 ;


  BFT_MALLOC(nbr_cel_loc, 1, ecs_size_t) ;

  nbr_cel_loc[0]  = renum->nbr_cel ;

  renum->nbr_cel_loc = nbr_cel_loc ;


  BFT_MALLOC(nbr_fac_interne_loc, 1, ecs_size_t) ;
  BFT_MALLOC(nbr_fac_de_bord_loc, 1, ecs_size_t) ;

  nbr_fac_interne_loc[0] = renum->nbr_fac_interne ;
  nbr_fac_de_bord_loc[0] = renum->nbr_fac_de_bord ;

  renum->nbr_fac_interne_loc = nbr_fac_interne_loc ;
  renum->nbr_fac_de_bord_loc = nbr_fac_de_bord_loc ;


  /* Tableaux */

  BFT_MALLOC(val_connect_fac_interne_cel_loc, 1, ecs_int_t *) ;
  BFT_MALLOC(val_connect_fac_de_bord_cel_loc, 1, ecs_int_t *) ;

  val_connect_fac_interne_cel_loc[0] = renum->val_connect_fac_interne_cel ;
  val_connect_fac_de_bord_cel_loc[0] = renum->val_connect_fac_de_bord_cel ;

  nbr_fac_interne_loc[0] = renum->nbr_fac_interne ;


  /* Marquage des faces priodiques */

  if (renum->nbr_per > 0) {

    BFT_MALLOC(ind_fac_perio, renum->nbr_fac_interne, ecs_int_t) ;
    for (ind_fac = 0 ; ind_fac < renum->nbr_fac_interne ; ind_fac++)
      ind_fac_perio[ind_fac] = -1 ;

    for (ind_fac = 0 ; ind_fac < renum->nbr_couples_perio ; ind_fac++) {
      ind_fac_perio[renum->connect_fac_perio[ind_fac*3    ] - 1] = ind_fac ;
      ind_fac_perio[renum->connect_fac_perio[ind_fac*3 + 1] - 1] = ind_fac ;
    }

    BFT_MALLOC(nbr_coupl_perio_loc, 1, ecs_int_t *) ;
    BFT_MALLOC(nbr_coupl_perio_loc[0], renum->nbr_per, ecs_int_t) ;
    for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++)
      nbr_coupl_perio_loc[0][ind_per] = 0 ;

  }


  /* Allocations dans le cas priodique */

  /* Comptage pour la priodicit */

  if (ind_fac_perio != NULL) {

    for (ind_fac = 0 ; ind_fac < renum->nbr_fac_interne ; ind_fac++) {

      if (ind_fac_perio[ind_fac] != -1) {

        num_per = renum->connect_fac_perio[ind_fac_perio[ind_fac]*3 + 2] ;
        ind_per = ECS_ABS(num_per) - 1 ;

        nbr_coupl_perio_loc[0][ind_per] += 1 ;

      }

    }

    for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++)
      nbr_coupl_perio_loc[0][ind_per] /= 2 ;


    /* Construction de l'index pour les joints priodiques et remise 
       zro du compteur de faces priodiques par domaine et priodicit */

    BFT_MALLOC(renum->idx_fac_perio_loc, 1, ecs_int_t *) ;
    BFT_MALLOC(renum->idx_fac_perio_loc[0], renum->nbr_per + 1, ecs_int_t) ;
    renum->idx_fac_perio_loc[0][0] = 0 ;
    for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++) {
      renum->idx_fac_perio_loc[0][ind_per + 1]
        =   renum->idx_fac_perio_loc[0][ind_per]
          + nbr_coupl_perio_loc[0][ind_per] ;
      nbr_coupl_perio_loc[0][ind_per] = 0 ;
    }
    BFT_MALLOC(renum->liste_fac_perio_loc, 1, ecs_int_t *);
    BFT_MALLOC(renum->liste_fac_perio_loc[0],
               renum->idx_fac_perio_loc[0][renum->nbr_per] * 2,
               ecs_int_t);

  }


  /* Prparation en cas de priodicit */

  if (ind_fac_perio != NULL) {

    /* Traitement des faces internes */

    for (ind_fac = 0 ; ind_fac < renum->nbr_fac_interne ; ind_fac++) {

      if (ind_fac_perio[ind_fac] != -1) {

        num_cel     = renum->val_connect_fac_interne_cel[ind_fac * 2] ;
        num_cel_vis = renum->val_connect_fac_interne_cel[ind_fac * 2 + 1] ;

        num_fac_per_1 = renum->connect_fac_perio[ind_fac_perio[ind_fac]*3] ;
        num_fac_per_2 = renum->connect_fac_perio[ind_fac_perio[ind_fac]*3 + 1] ;

        /* On utilise un numro ngatif pour la connectivit de la cellule
           "correspondante" (par construction) */

        if (num_cel > 0) {
          val_connect_fac_interne_cel_loc[0][ind_fac * 2]     = num_cel ;
          val_connect_fac_interne_cel_loc[0][ind_fac * 2 + 1] = 0 ;
        }
        else {
          val_connect_fac_interne_cel_loc[0][ind_fac * 2]     = 0 ;
          val_connect_fac_interne_cel_loc[0][ind_fac * 2 + 1] = num_cel_vis ;
        }

        /* On ne dfinit qu'une fois le couple de faces priodiques
           dans la structure joints associe */

        if (num_fac_per_1 == (ecs_int_t)(ind_fac + 1)) {

          num_per = renum->connect_fac_perio[ind_fac_perio[ind_fac]*3 + 2] ;
          ind_per = ECS_ABS(num_per) - 1 ;

          ipos_1 = (  renum->idx_fac_perio_loc[0][ind_per]
                    + nbr_coupl_perio_loc[0][ind_per]) * 2 ;

          renum->liste_fac_perio_loc[0][ipos_1]     = num_fac_per_1 ;
          renum->liste_fac_perio_loc[0][ipos_1 + 1] = num_fac_per_2 ;

          if (num_per < 0)
            renum->liste_fac_perio_loc[0][ipos_1]
              = - renum->liste_fac_perio_loc[0][ipos_1] ;

          nbr_coupl_perio_loc[0][ind_per] += 1 ;

        }

      }

    }

  }


  if(renum->nbr_per > 0) {
    BFT_FREE(ind_fac_perio) ;
    BFT_FREE(nbr_coupl_perio_loc[0]);
    BFT_FREE(nbr_coupl_perio_loc);
  }


  renum->val_connect_fac_interne_cel_loc = val_connect_fac_interne_cel_loc ;
  renum->val_connect_fac_de_bord_cel_loc = val_connect_fac_de_bord_cel_loc ;


  /* Traitement de la connectivit faces <-> sommets */

  ecs_loc_renum__connect_som_1(renum) ;

}


/*----------------------------------------------------------------------------
 *    Fonction de renumrotation locale des sommets.
 *    Les sommets sont utiliss pour des calculs gomtriques, mais ne
 *    portent pas de variables  ce jour. On ne cre donc pas de liste de
 *    correspondance entre sommets de domaines voisins, et on se contente
 *    de crer la connectivit locale  chaque sous-domaine.
 *----------------------------------------------------------------------------*/

void ecs_loc_renum__connect_som
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
)
{

  ecs_int_t       ind_dom ;
  size_t          ind_som ;
  size_t          ind_fac ;
  size_t          ind_fac_loc ;

  ecs_int_t       iloc ;
  size_t          ipos ;
  size_t          ipos_loc ;

  ecs_int_t       nbr_som_fac ;
  size_t          nbr_val_connect_fac_interne_loc ;
  size_t          nbr_val_connect_fac_de_bord_loc ;

  ecs_int_t     * num_loc_som ;
  ecs_int_t     * cur_dom_som ;

  ecs_size_t    * nbr_som_loc ;

  ecs_int_t   * * num_glob_som ;

  ecs_size_t  * * pos_connect_fac_interne_som_loc ;
  ecs_size_t  * * pos_connect_fac_de_bord_som_loc ;
  ecs_int_t   * * val_connect_fac_interne_som_loc ;
  ecs_int_t   * * val_connect_fac_de_bord_som_loc ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;

  BFT_MALLOC(nbr_som_loc, renum->nbr_dom, ecs_size_t) ;

  renum->nbr_som_loc = nbr_som_loc ;

  for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; nbr_som_loc[ind_dom++] = 0) ;


  if (renum->nbr_som == 0)
    return ;


  /* Tableaux locaux */

  BFT_MALLOC(num_loc_som, renum->nbr_som, ecs_int_t) ;
  BFT_MALLOC(cur_dom_som, renum->nbr_som, ecs_int_t) ;

  for (ind_som = 0 ; ind_som < renum->nbr_som ; num_loc_som[ind_som++] = -1) ;
  for (ind_som = 0 ; ind_som < renum->nbr_som ; cur_dom_som[ind_som++] = -1) ;


  /* Tableaux pour la structure renum */

  BFT_MALLOC(pos_connect_fac_de_bord_som_loc, renum->nbr_dom, ecs_size_t *) ;
  BFT_MALLOC(val_connect_fac_de_bord_som_loc, renum->nbr_dom, ecs_int_t *) ;
  BFT_MALLOC(pos_connect_fac_interne_som_loc, renum->nbr_dom, ecs_size_t *) ;
  BFT_MALLOC(val_connect_fac_interne_som_loc, renum->nbr_dom, ecs_int_t *) ;


  renum->pos_connect_fac_de_bord_som_loc = pos_connect_fac_de_bord_som_loc ;
  renum->pos_connect_fac_interne_som_loc = pos_connect_fac_interne_som_loc ;
  renum->val_connect_fac_de_bord_som_loc = val_connect_fac_de_bord_som_loc ;
  renum->val_connect_fac_interne_som_loc = val_connect_fac_interne_som_loc ;


  BFT_MALLOC(num_glob_som, renum->nbr_dom, ecs_int_t *) ;

  renum->num_glob_som = num_glob_som ;


  /* Boucle sur les sous-domaines */

  for (ind_dom = 0 ; ind_dom < renum->nbr_dom ; ind_dom++) {


    /* Prparation pour la numrotation locale des sommets */


    nbr_val_connect_fac_interne_loc = 0 ;
    nbr_val_connect_fac_de_bord_loc = 0 ;


    for (ind_fac_loc = 0 ;
         ind_fac_loc < renum->nbr_fac_interne_loc[ind_dom] ;
         ind_fac_loc++) {

      ind_fac = renum->num_glob_fac[ind_dom][ind_fac_loc] - 1 ;

      for (ipos = renum->pos_connect_fac_interne_som[ind_fac    ] - 1 ;
           ipos < renum->pos_connect_fac_interne_som[ind_fac + 1] - 1 ;
           ipos++) {

        ind_som = renum->val_connect_fac_interne_som[ipos] - 1;

        nbr_val_connect_fac_interne_loc += 1 ;

        if (cur_dom_som[ind_som] < ind_dom) {

          nbr_som_loc[ind_dom] += 1 ;
          num_loc_som[ind_som] = nbr_som_loc[ind_dom] ;
          cur_dom_som[ind_som] = ind_dom ;

        }

      }

    }


    for (ind_fac_loc = 0 ;
         ind_fac_loc < renum->nbr_fac_de_bord_loc[ind_dom] ;
         ind_fac_loc++) {

      ind_fac = renum->num_glob_fbr[ind_dom][ind_fac_loc] - 1 ;

      for (ipos = renum->pos_connect_fac_de_bord_som[ind_fac    ] - 1 ;
           ipos < renum->pos_connect_fac_de_bord_som[ind_fac + 1] - 1 ;
           ipos++) {

        ind_som = renum->val_connect_fac_de_bord_som[ipos] - 1 ;

        nbr_val_connect_fac_de_bord_loc += 1 ;

        if (cur_dom_som[ind_som] < ind_dom) {

          nbr_som_loc[ind_dom] += 1 ;
          num_loc_som[ind_som] = nbr_som_loc[ind_dom] ;
          cur_dom_som[ind_som] = ind_dom ;

        }

      }

    }


    BFT_MALLOC(num_glob_som[ind_dom], nbr_som_loc[ind_dom], ecs_int_t) ;


    /* Connectivit des faces internes locales avec les sommets locaux */


    BFT_MALLOC(pos_connect_fac_interne_som_loc[ind_dom],
              renum->nbr_fac_interne_loc[ind_dom] + 1,
              ecs_size_t) ;
    BFT_MALLOC(val_connect_fac_interne_som_loc[ind_dom],
              nbr_val_connect_fac_interne_loc,
              ecs_int_t) ;


    pos_connect_fac_interne_som_loc[ind_dom][0] = 1 ;


    for (ind_fac_loc = 0 ;
         ind_fac_loc < renum->nbr_fac_interne_loc[ind_dom] ;
         ind_fac_loc++) {

      ind_fac = renum->num_glob_fac[ind_dom][ind_fac_loc] - 1 ;


      nbr_som_fac =   renum->pos_connect_fac_interne_som[ind_fac + 1]
                    - renum->pos_connect_fac_interne_som[ind_fac    ] ;

      pos_connect_fac_interne_som_loc[ind_dom][ind_fac_loc + 1]
        = pos_connect_fac_interne_som_loc[ind_dom][ind_fac_loc]
          + nbr_som_fac ;


      for (iloc = 0 ; iloc < nbr_som_fac ; iloc++) {

        ipos     =   renum->pos_connect_fac_interne_som[ind_fac] - 1 + iloc ;
        ipos_loc =   pos_connect_fac_interne_som_loc[ind_dom][ind_fac_loc]
                   - 1 + iloc ;

        ind_som = renum->val_connect_fac_interne_som[ipos] - 1 ;

        val_connect_fac_interne_som_loc[ind_dom][ipos_loc]
          = num_loc_som[ind_som] ;

        num_glob_som[ind_dom][num_loc_som[ind_som] - 1] = ind_som + 1 ;

      }

    }


    /* Connectivit des faces de bord locales avec les sommets locaux */


    BFT_MALLOC(pos_connect_fac_de_bord_som_loc[ind_dom],
              renum->nbr_fac_de_bord_loc[ind_dom] + 1,
              ecs_size_t) ;
    BFT_MALLOC(val_connect_fac_de_bord_som_loc[ind_dom],
              nbr_val_connect_fac_de_bord_loc,
              ecs_int_t) ;


    pos_connect_fac_de_bord_som_loc[ind_dom][0] = 1 ;


    for (ind_fac_loc = 0 ;
         ind_fac_loc < renum->nbr_fac_de_bord_loc[ind_dom] ;
         ind_fac_loc++) {

      ind_fac = renum->num_glob_fbr[ind_dom][ind_fac_loc] - 1 ;


      nbr_som_fac =   renum->pos_connect_fac_de_bord_som[ind_fac + 1]
                    - renum->pos_connect_fac_de_bord_som[ind_fac    ] ;

      pos_connect_fac_de_bord_som_loc[ind_dom][ind_fac_loc + 1]
        = pos_connect_fac_de_bord_som_loc[ind_dom][ind_fac_loc]
          + nbr_som_fac ;


      for (iloc = 0 ; iloc < nbr_som_fac ; iloc++) {

        ipos     =   renum->pos_connect_fac_de_bord_som[ind_fac] - 1 + iloc ;
        ipos_loc =   pos_connect_fac_de_bord_som_loc[ind_dom][ind_fac_loc]
                   - 1 + iloc ;

        ind_som = renum->val_connect_fac_de_bord_som[ipos] - 1 ;

        val_connect_fac_de_bord_som_loc[ind_dom][ipos_loc]
          = num_loc_som[ind_som] ;

        num_glob_som[ind_dom][num_loc_som[ind_som] - 1] = ind_som + 1 ;

      }

    }

  }


  /* Libration des tableaux locaux */

  BFT_FREE(num_loc_som) ;
  BFT_FREE(cur_dom_som) ;


}


/*----------------------------------------------------------------------------
 *    Fonction de renumrotation des sommets pour un seul domaine
 *----------------------------------------------------------------------------*/

static void ecs_loc_renum__connect_som_1
(
 ecs_renum_t  * renum               /* --> Structure de renumrotation        */
)
{

  ecs_size_t    * nbr_som_loc ;

  ecs_size_t  * * pos_connect_fac_interne_som_loc ;
  ecs_size_t  * * pos_connect_fac_de_bord_som_loc ;
  ecs_int_t   * * val_connect_fac_interne_som_loc ;
  ecs_int_t   * * val_connect_fac_de_bord_som_loc ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(renum != NULL) ;


  BFT_MALLOC(nbr_som_loc, 1, ecs_size_t) ;

  nbr_som_loc[0] = renum->nbr_som ;

  renum->nbr_som_loc = nbr_som_loc ;


  if (renum->nbr_som == 0)
    return ;


  /* Tableaux ventuels */


  BFT_MALLOC(pos_connect_fac_interne_som_loc, 1, ecs_size_t *) ;
  BFT_MALLOC(val_connect_fac_interne_som_loc, 1, ecs_int_t *) ;
  BFT_MALLOC(pos_connect_fac_de_bord_som_loc, 1, ecs_size_t *) ;
  BFT_MALLOC(val_connect_fac_de_bord_som_loc, 1, ecs_int_t *) ;

  pos_connect_fac_interne_som_loc[0] = renum->pos_connect_fac_interne_som ;
  val_connect_fac_interne_som_loc[0] = renum->val_connect_fac_interne_som ;
  pos_connect_fac_de_bord_som_loc[0] = renum->pos_connect_fac_de_bord_som ;
  val_connect_fac_de_bord_som_loc[0] = renum->val_connect_fac_de_bord_som ;

  renum->pos_connect_fac_interne_som_loc = pos_connect_fac_interne_som_loc ;
  renum->val_connect_fac_interne_som_loc = val_connect_fac_interne_som_loc ;
  renum->pos_connect_fac_de_bord_som_loc = pos_connect_fac_de_bord_som_loc ;
  renum->val_connect_fac_de_bord_som_loc = val_connect_fac_de_bord_som_loc ;

}

