/*============================================================================
 *  Dfinitions des fonctions
 *   associes aux structures `ecs_vec_int_t' et `ecs_vec_real_t' decrivant
 *   les vecteurs indexes entier et reel
 *   et propres aux vecteurs indexs
 *   lis aux champs auxiliaires de type "grandeur"
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilite
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <limits.h>
#include <math.h>

#include <bft_mem.h>
#include <bft_printf.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' systeme
 *----------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"
#include "ecs_tab.h"

#include "ecs_elt_typ_glob.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles des paquetages visibles
 *----------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_vec_int.h"
#include "ecs_vec_real.h"


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associe au fichier courant
 *----------------------------------------------------------------------------*/

#include "ecs_vec_grd.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' prives   du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_vec_int_priv.h"
#include "ecs_vec_real_priv.h"



/*============================================================================
 *                       Macros globales au fichier
 *============================================================================*/

#if !defined(FLT_MAX)
#define FLT_MAX HUGE_VAL
#endif

enum {
  X ,
  Y ,
  Z
} ;


#define ECS_LOC_PRODUIT_VECTORIEL(prod_vect, vect1, vect2) ( \
prod_vect[X] = vect1[Y] * vect2[Z] - vect2[Y] * vect1[Z] ,   \
prod_vect[Y] = vect2[X] * vect1[Z] - vect1[X] * vect2[Z] ,   \
prod_vect[Z] = vect1[X] * vect2[Y] - vect2[X] * vect1[Y]   )


#define ECS_LOC_PRODUIT_SCALAIRE(vect1, vect2)                        ( \
  vect1[X] * vect2[X] + vect1[Y] * vect2[Y] + vect1[Z] * vect2[Z] )


#define ECS_LOC_MODULE(vect) \
     sqrt(vect[X] * vect[X] + vect[Y] * vect[Y] + vect[Z] * vect[Z])



/*============================================================================
 *                       Prototypes de fonctions privees
 *============================================================================*/


/*============================================================================
 *                             Fonctions publiques
 *============================================================================*/


/*----------------------------------------------------------------------------
  *  Fonction qui calcule les coordonnes minimales et maximales
 *----------------------------------------------------------------------------*/

void ecs_vec_grd__calc_coo_ext
(
 ecs_vec_real_t * vec_som
)
{

  size_t      icoo ;
  size_t      ipos ;
  size_t      isom ;

  ecs_real_t  coo_min[3] ;
  ecs_real_t  coo_max[3] ;

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (vec_som->pos_nbr < 2)
    return ;

  ipos = 0 ;

  for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++) {
    coo_min[icoo] = vec_som->val_tab[ipos + icoo] ;
    coo_max[icoo] = vec_som->val_tab[ipos + icoo] ;
  }

  for (isom = 1 ; isom < vec_som->pos_nbr - 1 ; isom++) {

    ipos = vec_som->pos_pas * isom ;

    for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++) {
      if (vec_som->val_tab[ipos + icoo] < coo_min[icoo])
        coo_min[icoo] = vec_som->val_tab[ipos + icoo] ;
      else if (vec_som->val_tab[ipos + icoo] > coo_max[icoo])
        coo_max[icoo] = vec_som->val_tab[ipos + icoo] ;
    }

  }

  bft_printf(_("\n  Domain coordinate extents:\n\n")) ;

  bft_printf("  [% 10.5e, % 10.5e, % 10.5e]\n",
             coo_min[0], coo_min[1], coo_min[2]) ;
  bft_printf("  [% 10.5e, % 10.5e, % 10.5e]\n",
             coo_max[0], coo_max[1], coo_max[2]) ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui modifie les coordonnes du maillage
 *----------------------------------------------------------------------------*/

void ecs_vec_grd__transf_coo
(
 ecs_vec_real_t  * vec_som,
 const double      matrice[3][4]
)
{

  size_t      icoo ;
  size_t      ipos ;
  size_t      isom ;

  ecs_real_t  coo_tmp[3] ;

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (vec_som->pos_nbr < 2)
    return ;

  for (isom = 0; isom < vec_som->pos_nbr - 1 ; isom++) {

    ipos = vec_som->pos_pas * isom ;

    for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
      coo_tmp[icoo] = vec_som->val_tab[ipos + icoo] ;

    for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
      vec_som->val_tab[ipos + icoo]
        = (  matrice[icoo][0] * coo_tmp[0]
           + matrice[icoo][1] * coo_tmp[1]
           + matrice[icoo][2] * coo_tmp[2]
           + matrice[icoo][3] * 1.0) ;

  }

}


/*----------------------------------------------------------------------------
 *  Fonction qui calcule le maximum associ  une cellule d'un tableau de
 *   valeurs associes aux faces internes.
 *----------------------------------------------------------------------------*/

ecs_tab_real_t ecs_vec_grd__max_fac_int
(
       size_t                   nbr_cel           ,
 const ecs_vec_int_t     *const vec_def_fac_cel   ,
 const ecs_tab_int_t     *const liste_fac_interne ,
 const ecs_tab_real_t    *const val_fac_interne   ,
       ecs_real_t               val_min
)
{

  size_t     icel ;
  size_t     ifac ;
  size_t     ifac_int ;
  size_t     icel_inf ;
  size_t     icel_sup ;

  size_t     ind_cel_inf ;
  size_t     ind_cel_sup ;

  ecs_tab_real_t tab_val_cel ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/

  assert(vec_def_fac_cel   != NULL) ;
  assert(liste_fac_interne != NULL) ;
  assert(val_fac_interne   != NULL) ;


  tab_val_cel.nbr = nbr_cel ;
  BFT_MALLOC(tab_val_cel.val, tab_val_cel.nbr, ecs_real_t) ;

  for (icel = 0 ; icel < tab_val_cel.nbr ; icel++)
    tab_val_cel.val[icel] = val_min ;


  /* Boucle gnrale sur les faces */
  /*===============================*/


  for (ifac_int = 0 ; ifac_int < liste_fac_interne->nbr  ; ifac_int++) {

    ifac = liste_fac_interne->val[ifac_int] ;

    assert((  vec_def_fac_cel->pos_tab[ifac + 1]
            - vec_def_fac_cel->pos_tab[ifac    ]) == 2) ;


    /* Numros des cellules qui encadrent la face interne */
    /*----------------------------------------------------*/

    icel_inf = vec_def_fac_cel->pos_tab[ifac] - 1 ;
    icel_sup = vec_def_fac_cel->pos_tab[ifac] ;

    ind_cel_inf = ECS_ABS(vec_def_fac_cel->val_tab[icel_inf]) - 1 ;
    ind_cel_sup = ECS_ABS(vec_def_fac_cel->val_tab[icel_sup]) - 1 ;

    /* Si on est en priodicit, on a cr des cellules "fantomes"
       supplmentaires pour pouvoir les considrer en tant que faces
       internes. On fait donc un test sur le numro de cellule.      */

    if (ind_cel_inf < nbr_cel &&
        tab_val_cel.val[ind_cel_inf] < val_fac_interne->val[ifac_int])
      tab_val_cel.val[ind_cel_inf] = val_fac_interne->val[ifac_int] ;

    if (ind_cel_sup < nbr_cel &&
        tab_val_cel.val[ind_cel_sup] < val_fac_interne->val[ifac_int])
      tab_val_cel.val[ind_cel_sup] = val_fac_interne->val[ifac_int] ;

  } /* Fin : boucle sur les faces */


  return tab_val_cel ;


}


/*----------------------------------------------------------------------------*
 *  Calcul de la matrice d'une rotation dfinie par un angle theta (en
 *   radians) et un vecteur unitaire direction.
 *   La matrice est calcule via la formule suivante :
 *
 *  R = (1-cos(theta))direction.transp(direction) + cos(theta)I + sin(theta)V
 *
 *           [ 0            -direction(3)  direction(2)]
 *  avec V = [ direction(3)       0       -direction(1)]
 *           [-direction(2)  direction(1)       0      ]
 *
 *  Remarque : le calcul de la matrice necessite 10 additions et 24 produits.
 *             on pourrait rduire le nombre de multiplications (tout en
 *             augmentant le nombre d'additions) en passant par les
 *             quaternions. On pourrait aussi rduire le nombre de produits
 *              18 en stockant 6 produits dans des variables intermdiares.
 *----------------------------------------------------------------------------*/

void ecs_vec_grd__mat_rot_3d
(
 ecs_real_t  angle,              /* --> Angle de rotation en degres           */
 ecs_real_t  direction[3],       /* --> Vecteur directeur (unitaire) de l'axe */
 ecs_real_t  matrice[3][3]       /* <-- Matrice definissant la rotation       */
)
{

  /* initialisations */

  ecs_int_t        icoo ;
  ecs_real_t       norme_dir ;

  const ecs_real_t pi = 4 * atan(1) ;
  const ecs_real_t theta = pi * angle/180 ;
  const ecs_real_t cost = cos(theta) ;
  const ecs_real_t sint = sin(theta) ;
  const ecs_real_t unmcost = (1.0 - cost) ;


  if (ECS_ABS(angle) < 1.e-12)
    return ;


  norme_dir = ECS_LOC_MODULE(direction) ;
  assert(norme_dir > 1.e-12) ;

  for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
    direction[icoo] /= norme_dir ;


  /* premire ligne de la matrice de rotation */

  matrice[0][0] = (unmcost) * direction[0]*direction[0] + cost                ;
  matrice[0][1] = (unmcost) * direction[0]*direction[1] - sint * direction[2] ;
  matrice[0][2] = (unmcost) * direction[0]*direction[2] + sint * direction[1] ;

  /* deuxime ligne de la matrice de rotation */

  matrice[1][0] = (unmcost) * direction[1]*direction[0] + sint * direction[2] ;
  matrice[1][1] = (unmcost) * direction[1]*direction[1] + cost                ;
  matrice[1][2] = (unmcost) * direction[1]*direction[2] - sint * direction[0] ;

  /* troisime ligne de la matrice de rotation */

  matrice[2][0] = (unmcost) * direction[2]*direction[0] - sint * direction[1] ;
  matrice[2][1] = (unmcost) * direction[2]*direction[1] + sint * direction[0] ;
  matrice[2][2] = (unmcost) * direction[2]*direction[2] + cost                ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui calcule les nouvelles coordonnees d'un vecteur ou d'un point
 *  dans le cas d'une periodicite.
 *----------------------------------------------------------------------------*/

ecs_vec_real_t * ecs_vec_grd__vec_new_coor_perio
(
 const ecs_vec_real_t    * vec_def             ,
 const ecs_real_t          translation[3]      ,
 const ecs_real_t          point_inv[3]        ,
 const ecs_real_t          matrice[3][3]       ,
 const ecs_bool_t          periodicite_inverse
)
{

  size_t            ielt ;
  size_t            icoo ;
  size_t            irot ;
  size_t            ipos ;

  ecs_real_t        vect[3] ;

  ecs_vec_real_t  * vec_def_new ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(vec_def->pos_nbr != 0) ;

  vec_def_new = ecs_vec_real__alloue(vec_def->pos_nbr,
                                     (vec_def->pos_nbr - 1) * ECS_DIM_3) ;


  if (periodicite_inverse == ECS_FALSE)


    for (ielt = 0 ; ielt < vec_def->pos_nbr - 1 ; ielt++) {

      ipos = vec_def->pos_pas * ielt ;

      for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
        vect[icoo] = 0. ;

      for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
        for (irot = 0 ; irot < ECS_DIM_3 ; irot++)
          vect[icoo] = vect[icoo] + matrice[icoo][irot] *
            (vec_def->val_tab[ipos+irot]-point_inv[irot] + translation[irot]) ;

      for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
        vec_def_new->val_tab[ipos+icoo] =  vect[icoo] + point_inv[icoo] ;

    }


  else


    for (ielt = 0 ; ielt < vec_def->pos_nbr - 1 ; ielt++) {

      ipos = vec_def->pos_pas * ielt ;

      for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
        vect[icoo] = 0. ;

      for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
        for (irot = 0 ; irot < ECS_DIM_3 ; irot++)
          vect[icoo] = vect[icoo] + matrice[irot][icoo] *
            (vec_def->val_tab[ipos+irot]-point_inv[irot]) ;

      for (icoo = 0 ; icoo < ECS_DIM_3 ; icoo++)
        vec_def_new->val_tab[ipos+icoo] =  vect[icoo] + point_inv[icoo]
                                           - translation[icoo] ;

    }



  return vec_def_new ;


}


/*============================================================================
 *                              Fonctions prives
 *============================================================================*/

