/*============================================================================
 *  Definitions des fonctions
 *   associees a la structure `ecs_champ_t' decrivant un champ
 *   et realisant les sorties pour la communication
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilite
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <string.h> /* strlen() */

#include <bft_error.h>
#include <bft_mem.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_chaine_glob.h"
#include "ecs_def.h"
#include "ecs_tab.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Post-Traitement"
 *----------------------------------------------------------------------------*/

#include "ecs_comm.h"


/*----------------------------------------------------------------------------
  *  Fichiers `include' visibles des paquetages visibles
 *----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_vec_int.h"
#include "ecs_champ_vec_real.h"

#include "ecs_champ.h"


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associe au fichier courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_comm.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' prives   du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_champ_priv.h"



/*============================================================================
 *                       Prototypes de fonctions privees
 *============================================================================*/


/*============================================================================
 *                             Fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui ecrit le contenu d'un champ
 *   dans le fichier d'interface pour le Noyau
 *----------------------------------------------------------------------------*/

void ecs_champ_comm__ecr
(
       ecs_champ_t  *const this_champ        ,
 const char         *const comm_nom_rubrique ,
       ecs_comm_t   *const comm
)
{

  size_t      nbr_val ;

  void      * val_tab ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(this_champ != NULL) ;


  nbr_val = ecs_champ__ret_val_nbr(this_champ) ;

  assert(nbr_val > 0) ;

  val_tab = (ecs_int_t *)(this_champ->val_tab) ;


  ecs_comm__ecrit_rubrique(comm_nom_rubrique,
                           nbr_val,
                           val_tab,
                           this_champ->typ_val,
                           comm) ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui ecrit les valeurs des elements d'un champ vectoriel
 *   contenus dans la liste fournie en argument
 *   dans le fichier d'interface pour le Noyau.
 *----------------------------------------------------------------------------*/

void ecs_champ_comm__liste_vec
(
       ecs_champ_t  *const this_champ_vectoriel ,
 const ecs_int_t    *const liste_elt            ,
 const ecs_int_t           nbr_elt_liste        ,
 const ecs_int_t           deb_elt_liste        ,
 const char         *const comm_nom_rubrique    ,
       ecs_comm_t   *const comm
)
{

  ecs_int_t     cpt_val ;
  ecs_int_t     nbr_cp  ;
  ecs_int_t     icoo ;
  ecs_int_t     ielt ;
  ecs_int_t     ind_elt ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(this_champ_vectoriel != NULL) ;
  assert(   this_champ_vectoriel->pos_tab == NULL
         && this_champ_vectoriel->pos_pas > 0) ;

  /* traitement selon le type de valeurs */

  switch (this_champ_vectoriel->typ_val) {

  case ECS_TYPE_ecs_int_t:

    {

      ecs_int_t  * composante    ;
      ecs_int_t  * val_tab ;


      nbr_cp  = this_champ_vectoriel->pos_pas ; /* nombre de composantes */

      val_tab = (ecs_int_t *)(this_champ_vectoriel->val_tab) ;

      BFT_MALLOC(composante, nbr_elt_liste * nbr_cp, ecs_int_t) ;

      cpt_val = 0 ;

      for (ielt = 0 ; ielt < nbr_elt_liste ; ielt++) {

        ind_elt = liste_elt[ielt] - deb_elt_liste ;

        for (icoo = 0 ; icoo < nbr_cp ; icoo++)

          composante[cpt_val++] = val_tab[ind_elt * nbr_cp + icoo] ;

      }


      ecs_comm__ecrit_rubrique(comm_nom_rubrique,
                               cpt_val,
                               (void *)composante,
                               ECS_TYPE_ecs_int_t,
                               comm) ;


      BFT_FREE(composante) ;

    }

    break ;

  case ECS_TYPE_ecs_real_t:

    {

      ecs_real_t  * composante    ;
      ecs_real_t  * val_tab ;


      nbr_cp  = this_champ_vectoriel->pos_pas ; /* nombre de composantes */

      val_tab = (ecs_real_t *)this_champ_vectoriel->val_tab ;

      assert(val_tab != NULL) ;

      BFT_MALLOC(composante, nbr_elt_liste * nbr_cp, ecs_real_t) ;

      cpt_val = 0 ;

      for (ielt = 0 ; ielt < nbr_elt_liste ; ielt++) {

        ind_elt = liste_elt[ielt] - deb_elt_liste ;

        for (icoo = 0 ; icoo < nbr_cp ; icoo++)

          composante[cpt_val++] = val_tab[ind_elt * nbr_cp + icoo] ;

      }


      ecs_comm__ecrit_rubrique(comm_nom_rubrique,
                               cpt_val,
                               (void *)composante,
                               ECS_TYPE_ecs_real_t,
                               comm) ;


      BFT_FREE(composante) ;

    }

    break ;

  default:

    assert(   this_champ_vectoriel->typ_val == ECS_TYPE_ecs_int_t
           || this_champ_vectoriel->typ_val == ECS_TYPE_ecs_real_t) ;

  }

}


/*============================================================================
 *                              Fonctions privees
 *============================================================================*/

