/*============================================================================
 *  Dfinitions des fonctions
 *   associes  la structure `ecs_entmail_t' dcrivant une entit de maillage
 *   et ralisant les sorties pour l'interfacage avec le Noyau
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/

/*============================================================================
 *                                 Visibilit
 *============================================================================*/


/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include <bft_error.h>
#include <bft_mem.h>
#include <bft_printf.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_chaine_glob.h"
#include "ecs_def.h"
#include "ecs_comm.h"
#include "ecs_fic.h"
#include "ecs_tab.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Post-Traitement"
 *----------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles des paquetages visibles
 *----------------------------------------------------------------------------*/

#include "ecs_champ.h"
#include "ecs_champ_chaine.h"
#include "ecs_champ_att.h"
#include "ecs_champ_def.h"
#include "ecs_champ_grd.h"
#include "ecs_champ_vec_int.h"
#include "ecs_champ_comm.h"
#include "ecs_famille_chaine.h"
#include "ecs_param_perio_glob.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_entmail.h"


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associ au fichier courant
 *----------------------------------------------------------------------------*/

#include "ecs_entmail_ncs.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' privs   du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_entmail_priv.h"
#include "ecs_renum_priv.h"

/*============================================================================
 *                       Dfinition de macros
 *============================================================================*/

/*  Numro de la famille par dfaut (sans couleur ni groupe) */

#define ECS_NCS_FAMILLE_VAL_PAR_DEFAUT  0


/*============================================================================
 *                       Prototypes de fonctions prives
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui crit les composantes d'un champ index par une liste
 *  globale (indice de 0  n-1)  laquelle on applique une liste locale
 *  (issue du dcoupage de domaines, et indice de 1  m)
 *----------------------------------------------------------------------------*/

static void ecs_loc_entmail_ncs__lis_champ
(
       ecs_champ_t     *const        this_champ,
       ecs_int_t       *const        liste,
       ecs_int_t       *const *const liste_loc,
 const ecs_int_t                     nbr_liste_loc,
 const char            *const        nom_rub,
       ecs_comm_t      *const        comm,
 const ecs_int_t                     ind_noeud
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui crit les valeurs d'un tableau d'entiers global auquel on
 *  applique une liste locale (issue du dcoupage de domaines, et indice
 *  de 1  m)
 *----------------------------------------------------------------------------*/

static void ecs_loc_entmail_ncs__tab_int
(
       ecs_int_t       *const        val_int,
       ecs_int_t       *const *const liste_loc,
 const ecs_int_t                     nbr_liste_loc,
 const char            *const        nom_rub,
       ecs_comm_t      *const        comm,
 const ecs_int_t                     ind_noeud
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui renvoie la liste des champs "famille"
 *   pour toutes les entits de maillage
 *  et qui dtermine :
 *  - le nombre   de familles  transfrer au Noyau
 *  - les numros de famille  des cellules
 *  - les numros de famille  des faces de bord
 *  - le nombre maximal de  proprits des familles
 *  - les proprits des familles
 *----------------------------------------------------------------------------*/

static ecs_champ_t * * ecs_loc_entmail_ncs__cree_fam
(
       size_t                          nbr_cel,
 const ecs_int_t         *const        liste_fac_de_bord,
       size_t                          nbr_fac_de_bord,
       ecs_entmail_t     *const *const vect_entmail,
       ecs_famille_t     *const *const vect_fam_tete,
       int               *const        nbr_fam,
       int               *const        nbr_max_propr,
       ecs_tab_int_t     *const        tab_fam_cel,
       ecs_tab_int_t     *const        tab_fam_fac_de_bord,
       ecs_tab_int_t     *const        tab_propr_fam,
       ecs_tab_char_t    *const        noms_groupes
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui affiche le nombre de cellules, faces internes, et faces
 *  de bord fournies en argument.
 *----------------------------------------------------------------------------*/

static void ecs_loc_entmail_ncs__aff_nbr_ent
(
 const ecs_int_t         nbr_elt_cel,
 const ecs_int_t         nbr_elt_fac_interne,
 const ecs_int_t         nbr_elt_fac_de_bord
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui (re)numrote les groupes et construit un tableau contenant
 *   les noms ordonns de ces groupes. Le tableau de chanes de caractres
 *   noms_groupes doit initialement tre vide.
 *----------------------------------------------------------------------------*/

static void ecs_loc_entmail_ncs__renum_groupes
(
       ecs_famille_t   *const *const vect_fam_tete,
 const ecs_int_t                     nbr_fam_ent[],
       ecs_tab_char_t         *const tab_propr_nom_fam_ent[],
       ecs_tab_char_t         *const noms_groupes
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui rordonne la connectivit faces internces -> cellules
 *   pour le Noyau ;
 *
 *  La liste des faces priodiques est code de la manire suivante
 *    (avec numros i, j en numrotation 1  n) :
 *    [i,j,n; ...] avec i->j ou j->i (i>j) et n le numro de la
 *    priodicit prcd d'un signe correspondant au sens  appliquer
 *----------------------------------------------------------------------------*/

static void ecs_loc_entmail_ncs__ord_fac
(
       size_t              nbr_fac,
 const ecs_tab_int_t       liste_fac_perio,
       ecs_int_t  *const   val_connect_fac_cel
) ;


/*----------------------------------------------------------------------------
 *  Fonction qui rordonne la connectivit des faces de bord pour le noyau
 *----------------------------------------------------------------------------*/

static void ecs_loc_entmail_ncs__ord_fbr
(
       size_t              nbr_fbr,
       ecs_int_t  *const   val_connect_fbr_cel,
 const ecs_size_t *const   pos_connect_fbr_som,
       ecs_int_t  *const   val_connect_fbr_som
) ;


/*============================================================================
 *                             Fonctions publiques
 *============================================================================*/


/*----------------------------------------------------------------------------
 *  Fonction qui cre la connectivit pour le Noyau
 *----------------------------------------------------------------------------*/

void ecs_entmail_ncs__renum_init
(
 ecs_entmail_t     *const *const vect_entmail,
 ecs_renum_t       *const        renum
)
{

  size_t            dim_connect_fac_interne_cel ;
  size_t            dim_connect_fac_de_bord_cel ;
  size_t            dim_connect_fac_interne_som ;
  size_t            dim_connect_fac_de_bord_som ;

  size_t            nbr_per = 0 ;
  size_t            nbr_cel = 0 ;
  size_t            nbr_fac = 0 ;
  size_t            nbr_som = 0 ;

  ecs_size_t    *   pos_connect_fac_de_bord_cel   = NULL ;
  ecs_size_t    *   pos_connect_fac_interne_cel   = NULL ;
  ecs_int_t     *   val_connect_fac_de_bord_cel   = NULL ;
  ecs_int_t     *   val_connect_fac_interne_cel   = NULL ;
  ecs_size_t    *   pos_connect_fac_de_bord_som   = NULL ;
  ecs_size_t    *   pos_connect_fac_interne_som   = NULL ;
  ecs_int_t     *   val_connect_fac_de_bord_som   = NULL ;
  ecs_int_t     *   val_connect_fac_interne_som   = NULL ;

  ecs_tab_int_t     nbr_fac_perio ;

  ecs_tab_int_t     liste_fac_perio ;

  ecs_tab_int_t     liste_fac_interne ;
  ecs_tab_int_t     liste_fac_de_bord ;

  ecs_champ_t   *   champ_connect_fac_som         = NULL ;
  ecs_champ_t   *   champ_connect_inverse_fac_cel = NULL ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(vect_entmail[ECS_ENTMAIL_CEL] != NULL) ;
  assert(vect_entmail[ECS_ENTMAIL_FAC] != NULL) ;
  assert(vect_entmail[ECS_ENTMAIL_ARE] == NULL) ;
  assert(vect_entmail[ECS_ENTMAIL_SOM] != NULL) ;


  /* Dtermination du nombre de cellules */
  /*-------------------------------------*/

  nbr_cel
    = ecs_champ__ret_pos_nbr
        (vect_entmail[ECS_ENTMAIL_CEL]->champ[ECS_CHAMP_DEF]) - 1 ;


  /* Dtermination du nombre de faces    */
  /*-------------------------------------*/

  nbr_fac
    = ecs_champ__ret_pos_nbr
        (vect_entmail[ECS_ENTMAIL_FAC]->champ[ECS_CHAMP_DEF]) - 1 ;


  /*----------------------------------------------------------------------*/
  /* Construction de la table de connectivit inverse "faces -> cellules" */
  /*----------------------------------------------------------------------*/

  champ_connect_inverse_fac_cel
    = ecs_champ__inverse(vect_entmail[ECS_ENTMAIL_CEL]->champ[ECS_CHAMP_DEF],
                         nbr_fac) ;


  /*------------------------------------------------------------------*/
  /* Modification de la connectivit "face -> cellules" en vue de la  */
  /* distinction face priodique / face de bord.                      */
  /* Les faces priodiques deviennent faces internes.                 */
  /* On renvoie un entier correspondant au nombre de priodicits.    */
  /*------------------------------------------------------------------*/

  nbr_per = ecs_champ_chaine__connect_perio
    (vect_entmail[ECS_ENTMAIL_FAC]->champ[ECS_CHAMP_ATT],
     champ_connect_inverse_fac_cel) ;


  nbr_fac_perio.nbr = 0 ;
  nbr_fac_perio.val = NULL ;


  /* Dtermination du nombre de faces de bord et de faces internes, */
  /* et du nombre de faces priodiques s'il en existe               */
  /*----------------------------------------------------------------*/

  ecs_champ_def__cree_listes_fac(champ_connect_inverse_fac_cel,
                                 nbr_fac,
                                 NULL,
                                 &liste_fac_interne,
                                 &liste_fac_de_bord,
                                 NULL) ;


  /*
     Construction de la liste des faces priodiques et de celle du sens
     de la priodicit.
     On prpare aussi la dfinition de la liste des informations utiles
     pour la cration des joints priodiques depuis le noyau. Par
     priodicit, on stockera le numro local de la face de dpart et
     d'arrive ainsi que leur domaine. Le sens de la priodicit
     sera contenu dans le signe du numro local de la face de dpart.
  */

  if (nbr_per > 0) {

    /*
      La liste des faces priodiques est code de la manire suivante
      (avec numros i, j en numrotation 1  n) :
      [i,j,n; ...] avec i->j ou j->i (i>j) et n le numro de la
      priodicit prcd d'un signe correspondant au sens  appliquer
    */

    liste_fac_perio = ecs_champ_chaine__liste_fac_int_perio
                       (vect_entmail[ECS_ENTMAIL_FAC]->champ[ECS_CHAMP_ATT],
                        liste_fac_interne.nbr,
                        liste_fac_interne.val) ;

  }
  else {

    liste_fac_perio.nbr = 0 ;
    liste_fac_perio.val = NULL ;

  }


  /* Construction de la connectivit faces internes -> cellules voisines */

  BFT_MALLOC(pos_connect_fac_interne_cel, liste_fac_interne.nbr + 1,
             ecs_size_t) ;

  val_connect_fac_interne_cel
      = ecs_champ__liste(pos_connect_fac_interne_cel,
                         &dim_connect_fac_interne_cel,
                         champ_connect_inverse_fac_cel,
                         liste_fac_interne.val,
                         liste_fac_interne.nbr) ;

  BFT_FREE(pos_connect_fac_interne_cel);


  /* Construction de la connectivit faces de bord  -> cellules voisines */

  BFT_MALLOC(pos_connect_fac_de_bord_cel, liste_fac_de_bord.nbr + 1,
             ecs_size_t) ;

  val_connect_fac_de_bord_cel
    = ecs_champ__liste(pos_connect_fac_de_bord_cel,
                       &dim_connect_fac_de_bord_cel,
                       champ_connect_inverse_fac_cel,
                       liste_fac_de_bord.val,
                       liste_fac_de_bord.nbr) ;

  BFT_FREE(pos_connect_fac_de_bord_cel);


  champ_connect_inverse_fac_cel
      = ecs_champ__detruit(champ_connect_inverse_fac_cel) ;


  /* Dtermination du nombre de sommets */
  /*------------------------------------*/

  nbr_som
    = ecs_champ__ret_pos_nbr
    (vect_entmail[ECS_ENTMAIL_SOM]->champ[ECS_CHAMP_DEF]) - 1;


  /*-------------------------------------------------------------*/
  /* Construction des positions des sommets des faces internes   */
  /* Construction de la connectivit "faces internes -> sommets" */
  /* Construction des positions des sommets des faces de bord    */
  /* Construction de la connectivit "faces de bord  -> sommets" */
  /*-------------------------------------------------------------*/

  /* Rcupration de la table de connectivit "faces -> sommets" */

  champ_connect_fac_som
    = vect_entmail[ECS_ENTMAIL_FAC]->champ[ECS_CHAMP_DEF] ;


  /* Construction des positions des sommets des faces internes   */
  /* Construction de la connectivit "faces internes -> sommets" */

  BFT_MALLOC(pos_connect_fac_interne_som, liste_fac_interne.nbr + 1,
             ecs_size_t);

  dim_connect_fac_interne_som = 0 ;

  val_connect_fac_interne_som
    = ecs_champ__liste(pos_connect_fac_interne_som,
                       &dim_connect_fac_interne_som,
                       champ_connect_fac_som,
                       liste_fac_interne.val,
                       liste_fac_interne.nbr) ;


  /* Construction des positions des sommets des faces de bord    */
  /* Construction de la connectivit "faces de bord  -> sommets" */

  BFT_MALLOC(pos_connect_fac_de_bord_som, liste_fac_de_bord.nbr + 1,
             ecs_size_t) ;

  dim_connect_fac_de_bord_som = 0 ;

  val_connect_fac_de_bord_som
    = ecs_champ__liste(pos_connect_fac_de_bord_som,
                       &dim_connect_fac_de_bord_som,
                       champ_connect_fac_som,
                       liste_fac_de_bord.val,
                       liste_fac_de_bord.nbr) ;


  /* On corrige les orientations des faces */

  ecs_loc_entmail_ncs__ord_fac(liste_fac_interne.nbr,
                               liste_fac_perio,
                               val_connect_fac_interne_cel) ;

  ecs_loc_entmail_ncs__ord_fbr(liste_fac_de_bord.nbr,
                               val_connect_fac_de_bord_cel,
                               pos_connect_fac_de_bord_som,
                               val_connect_fac_de_bord_som) ;


  renum->nbr_per                     = nbr_per ;

  renum->nbr_cel                     = nbr_cel ;
  renum->nbr_som                     = nbr_som ;
  renum->nbr_couples_perio           = liste_fac_perio.nbr / 3 ;
  renum->nbr_fac_interne             = liste_fac_interne.nbr ;
  renum->nbr_fac_de_bord             = liste_fac_de_bord.nbr ;

  renum->liste_fac_interne           = liste_fac_interne.val ;
  renum->liste_fac_de_bord           = liste_fac_de_bord.val ;
  renum->connect_fac_perio           = liste_fac_perio.val ;


  renum->val_connect_fac_interne_cel = val_connect_fac_interne_cel ;
  renum->val_connect_fac_de_bord_cel = val_connect_fac_de_bord_cel ;

  renum->pos_connect_fac_interne_som = pos_connect_fac_interne_som ;
  renum->pos_connect_fac_de_bord_som = pos_connect_fac_de_bord_som ;
  renum->val_connect_fac_interne_som = val_connect_fac_interne_som ;
  renum->val_connect_fac_de_bord_som = val_connect_fac_de_bord_som ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui crit le fichier d'interface pour le Noyau de Code_Saturne.
 *----------------------------------------------------------------------------*/

void ecs_entmail_ncs__ecr
(
 ecs_comm_typ_t                  comm_typ_ncs_e,
 ecs_int_t                       echo_comm,
 ecs_entmail_t     *const *const vect_entmail,
 ecs_famille_t     *const *const vect_fam_tete,
 ecs_int_t                       dim_spatiale,
 ecs_param_perio_t        *const liste_param_perio,
 ecs_renum_t              *const renum
)
{

  ecs_int_t       nbr_fam ;
  ecs_int_t       nbr_max_propr ;
  size_t          dim_connect_fac_interne_som ;
  size_t          dim_connect_fac_de_bord_som ;

  int             ind_noeud ;
  ecs_int_t       ind_per ;

  size_t          ind_grp ;
  ecs_int_t     * pos_nom_grp ;
  char          * val_nom_grp ;

  ecs_tab_int_t   tab_fam_cel ;
  ecs_tab_int_t   tab_fam_fac_de_bord ;
  ecs_tab_int_t   tab_propr_fam ;

  ecs_tab_char_t  tab_noms_groupes ;

  ecs_champ_t  * * vect_champ_famille ;

  ecs_comm_t    * comm ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(vect_entmail[ECS_ENTMAIL_CEL] != NULL) ;
  assert(vect_entmail[ECS_ENTMAIL_FAC] != NULL) ;
  assert(vect_entmail[ECS_ENTMAIL_ARE] == NULL) ;
  assert(vect_entmail[ECS_ENTMAIL_SOM] != NULL) ;


  /*--------------------------------------------------------------------------*/
  /* Dtermination :                                                          */
  /* - du nombre   de familles  transfrer au Noyau                          */
  /* - des numros de famille  des faces de bord                              */
  /* - du nombre maximal de  proprits des familles                          */
  /* - des proprits des familles                                            */
  /*--------------------------------------------------------------------------*/


  vect_champ_famille
    = ecs_loc_entmail_ncs__cree_fam(renum->nbr_cel,
                                    renum->liste_fac_de_bord,
                                    renum->nbr_fac_de_bord,
                                    vect_entmail,
                                    vect_fam_tete,
                                    &nbr_fam,
                                    &nbr_max_propr,
                                    &tab_fam_cel,
                                    &tab_fam_fac_de_bord,
                                    &tab_propr_fam,
                                    &tab_noms_groupes) ;

  BFT_FREE(vect_champ_famille) ;

  if (tab_noms_groupes.nbr > 0) {

    BFT_MALLOC(pos_nom_grp, tab_noms_groupes.nbr + 1, ecs_int_t) ;

    pos_nom_grp[0] = 1 ;
    for (ind_grp = 0 ; ind_grp < tab_noms_groupes.nbr ; ind_grp++)
      pos_nom_grp[ind_grp + 1]
        = pos_nom_grp[ind_grp] + strlen(tab_noms_groupes.val[ind_grp]) + 1 ;

    BFT_MALLOC(val_nom_grp, pos_nom_grp[tab_noms_groupes.nbr] - 1, char) ;

    for (ind_grp = 0 ; ind_grp < tab_noms_groupes.nbr ; ind_grp++)
      strcpy(val_nom_grp + (pos_nom_grp[ind_grp] - 1),
             tab_noms_groupes.val[ind_grp]) ;

  }


  /* Boucle sur les fichiers de sortie */

  for (ind_noeud = 0 ; ind_noeud < renum->nbr_dom ; ind_noeud++) {

    /* Initialisation de la communication vers le Noyau */
    /*--------------------------------------------------*/

    comm = ecs_comm__initialise(ind_noeud + 1,
                                comm_typ_ncs_e,
                                echo_comm) ;


    /* criture de la rubrique du dbut de bloc sur les dimensions */
    /*-------------------------------------------------------------*/

    ecs_comm__ecrit_rubrique("start_block:dimensions",
                             0,
                             NULL,
                             ECS_TYPE_void,
                             comm) ;


    /*------------------------------------------------------------------------*/
    /* criture des dimensions :                                              */
    /*------------------------------------------------------------------------*/
    /* - dimension spatiale                                                   */
    /* - nombre de cellules                                                   */
    /* - nombre de faces internes                                             */
    /* - nombre de faces de bord                                              */
    /* - nombre de familles                                                   */
    /* - nombre de proprits max. des familles de faces de bord              */
    /* S'il faut crire les sommets :                                         */
    /* - nombre de sommets                                                    */
    /* - dimensionnement table de connectivit "faces internes -> sommets"    */
    /* - dimensionnement table de connectivit "faces de bord  -> sommets"    */
    /*------------------------------------------------------------------------*/


    /* criture de la dimension spatiale */
    /*-----------------------------------*/

    ecs_comm__ecrit_rubrique("ndim",
                             1,
                             &dim_spatiale,
                             ECS_TYPE_ecs_int_t,
                             comm) ;


    /* criture du nombre de cellules */

    ecs_comm__ecrit_rubrique("ncel",
                             1,
                             &(renum->nbr_cel_loc[ind_noeud]),
                             ECS_TYPE_ecs_size_t,
                             comm) ;


    /* criture du nombre de faces internes */

    ecs_comm__ecrit_rubrique("nfac",
                             1,
                             &(renum->nbr_fac_interne_loc[ind_noeud]),
                             ECS_TYPE_ecs_size_t,
                             comm) ;


    /* criture du nombre de faces de bord */

    ecs_comm__ecrit_rubrique("nfabor",
                             1,
                             &(renum->nbr_fac_de_bord_loc[ind_noeud]),
                             ECS_TYPE_ecs_size_t,
                             comm) ;


    /* criture du nombre de familles (on ne compte pas la famille `0') */

    ecs_comm__ecrit_rubrique("maillage:dim:familles:nbr",
                             1,
                             &nbr_fam,
                             ECS_TYPE_ecs_int_t,
                             comm) ;


    /* criture du nombre de proprits max. des familles */

    ecs_comm__ecrit_rubrique("maillage:dim:familles:prop:nbr",
                             1,
                             &nbr_max_propr,
                             ECS_TYPE_ecs_int_t,
                             comm) ;


    /* criture du nombre de groupes si lieu */

    if (tab_noms_groupes.nbr > 0)

      ecs_comm__ecrit_rubrique("maillage:dim:groupes:nbr",
                               1,
                               &(tab_noms_groupes.nbr),
                               ECS_TYPE_size_t,
                               comm) ;


    /* criture du nombre de sommets */

    ecs_comm__ecrit_rubrique("maillage:dim:nsom",
                             1,
                             &(renum->nbr_som_loc[ind_noeud]),
                             ECS_TYPE_ecs_size_t,
                             comm) ;


    /* criture du dimensionnement de la table de connectivit */
    /* "faces internes -> sommets"                             */

    dim_connect_fac_interne_som
      = renum->pos_connect_fac_interne_som_loc[ind_noeud]
                                              [renum->nbr_fac_interne_loc
                                                      [ind_noeud]] - 1 ;

    ecs_comm__ecrit_rubrique("lndfac",
                             1,
                             &dim_connect_fac_interne_som,
                             ECS_TYPE_size_t,
                             comm) ;


    /* criture du dimensionnement de la table de connectivit */
    /* "faces de bord  -> sommets"                             */

    dim_connect_fac_de_bord_som
      = renum->pos_connect_fac_de_bord_som_loc[ind_noeud]
                                              [renum->nbr_fac_de_bord_loc
                                                      [ind_noeud]] - 1 ;

    ecs_comm__ecrit_rubrique("lndfbr",
                             1,
                             &dim_connect_fac_de_bord_som,
                             ECS_TYPE_size_t,
                             comm) ;


    /* critures supplmentaires pour le paralllisme (et la priodicit) */

    if (renum->nbr_dom > 1) {

      ecs_int_t _nbr_dom = renum->nbr_dom ;
      ecs_int_t _ind_noeud = ind_noeud ;

      /* nombre de domaines parallles */

      ecs_comm__ecrit_rubrique("maillage:dim:nbrdom",
                               1,
                               &_nbr_dom,
                               ECS_TYPE_ecs_int_t,
                               comm) ;

      /* numro du domaine courant */

      ecs_comm__ecrit_rubrique("maillage:dim:inddom",
                               1,
                               &_ind_noeud,
                               ECS_TYPE_ecs_int_t,
                               comm) ;
    }


    /* critures supplmentaires pour la priodicit */

    if (renum->nbr_per > 0) {

      ecs_int_t _nbr_per = renum->nbr_per ;
      ecs_int_t _nbr_per_rot = renum->nbr_per_rot ;

      /* nombre total de priodicits */

      ecs_comm__ecrit_rubrique("maillage:dim:per:nbrtot",
                               1,
                               &_nbr_per,
                               ECS_TYPE_ecs_int_t,
                               comm) ;

      /* nombre de priodicits de rotation */

      ecs_comm__ecrit_rubrique("maillage:dim:per:nbrrot",
                               1,
                               &_nbr_per_rot,
                               ECS_TYPE_ecs_int_t,
                               comm) ;

      /* Nombre de faces priodiques par priodicit */

      ecs_comm__ecrit_rubrique("maillage:dim:per:idxfac",
                               renum->nbr_per + 1,
                               renum->idx_fac_perio_loc[ind_noeud],
                               ECS_TYPE_ecs_int_t,
                               comm) ;

    }


    /* Passage du bloc des dimensions au bloc des donnes */
    /*----------------------------------------------------*/

    ecs_comm__ecrit_rubrique("end_block:dimensions",
                             0,
                             NULL,
                             ECS_TYPE_void,
                             comm) ;

    ecs_comm__ecrit_rubrique("start_block:data",
                             0,
                             NULL,
                             ECS_TYPE_void,
                             comm) ;


    /*---------------------------------------------------------------*/
    /* criture des donnes    :                                     */
    /*---------------------------------------------------------------*/
    /* - connectivit faces internes -> cellules voisines            */
    /* - connectivit faces de bord  -> cellules voisines            */
    /* - numeros     de  famille                  des faces de bord  */
    /* - proprits  des familles                 des faces de bord  */
    /* - coordonnes des sommets                                     */
    /* - positions   des sommets des faces internes                  */
    /* - connectivit faces internes -> sommets                      */
    /* - positions   des sommets des faces de bord                   */
    /* - connectivit faces de bord  -> sommets                      */
    /*---------------------------------------------------------------*/


    /* criture de la connectivit faces internes -> cellules voisines */

    if (renum->liste_fac_interne != NULL) {

      ecs_comm__ecrit_rubrique("ifacel",
                               renum->nbr_fac_interne_loc[ind_noeud] * 2,
                               renum->val_connect_fac_interne_cel_loc[ind_noeud],
                               ECS_TYPE_ecs_int_t,
                               comm) ;

    }


    /* criture de la connectivit faces de bord  -> cellules voisines */

    if (renum->liste_fac_de_bord != NULL) {

      ecs_comm__ecrit_rubrique("ifabor",
                               renum->nbr_fac_de_bord_loc[ind_noeud],
                               renum->val_connect_fac_de_bord_cel_loc[ind_noeud],
                               ECS_TYPE_ecs_int_t,
                               comm) ;

    }


    /* criture des numros de famille des cellules */

    ecs_loc_entmail_ncs__tab_int(tab_fam_cel.val,
                                 renum->num_glob_cel,
                                 renum->nbr_cel_loc[ind_noeud],
                                 "maillage:data:cel:famille",
                                 comm,
                                 ind_noeud) ;

    /* criture des numros de famille des faces de bord  */

    if (renum->liste_fac_de_bord != NULL)

      ecs_loc_entmail_ncs__tab_int(tab_fam_fac_de_bord.val,
                                   renum->num_glob_fbr,
                                   renum->nbr_fac_de_bord_loc[ind_noeud],
                                   "ifafbr",
                                   comm,
                                   ind_noeud) ;


    /* criture des proprits des familles        */
    /* - proprits des familles des cellules      */
    /* - proprits des familles des faces de bord */
    /* Pas de proprit pour la famille `0'        */

    ecs_comm__ecrit_rubrique("iprfml",
                             nbr_fam * nbr_max_propr,
                             tab_propr_fam.val,
                             ECS_TYPE_ecs_int_t,
                             comm) ;


    /* criture des noms groupes si lieu */

    if (tab_noms_groupes.nbr > 0) {

      ecs_comm__ecrit_rubrique("maillage:data:groupes:pos",
                               tab_noms_groupes.nbr + 1,
                               pos_nom_grp,
                               ECS_TYPE_ecs_int_t,
                               comm) ;

      ecs_comm__ecrit_rubrique("maillage:data:groupes:noms",
                               pos_nom_grp[tab_noms_groupes.nbr] - 1,
                               val_nom_grp,
                               ECS_TYPE_char,
                               comm) ;

    }


    /* criture des coordonnes des sommets */

    ecs_loc_entmail_ncs__lis_champ(vect_entmail[ECS_ENTMAIL_SOM]
                                           ->champ[ECS_CHAMP_DEF],
                                   NULL,
                                   renum->num_glob_som,
                                   renum->nbr_som_loc[ind_noeud],
                                   "xyznod",
                                   comm,
                                   ind_noeud) ;


    /* criture des positions des sommets des faces internes   */

    ecs_comm__ecrit_rubrique("ipnfac",
                             renum->nbr_fac_interne_loc[ind_noeud] + 1,
                             renum->pos_connect_fac_interne_som_loc[ind_noeud],
                             ECS_TYPE_ecs_size_t,
                             comm) ;

    /* criture de la connectivit "faces internes -> sommets" */

    dim_connect_fac_interne_som
      = renum->pos_connect_fac_interne_som_loc[ind_noeud]
                                              [renum->nbr_fac_interne_loc
                                                      [ind_noeud]] - 1 ;

    ecs_comm__ecrit_rubrique("nodfac",
                             dim_connect_fac_interne_som,
                             renum->val_connect_fac_interne_som_loc[ind_noeud],
                             ECS_TYPE_ecs_int_t,
                             comm) ;


    /* criture des positions des sommets des faces de bord    */

    ecs_comm__ecrit_rubrique("ipnfbr",
                             renum->nbr_fac_de_bord_loc[ind_noeud] + 1,
                             renum->pos_connect_fac_de_bord_som_loc[ind_noeud],
                             ECS_TYPE_ecs_size_t,
                             comm) ;

    /* criture de la connectivit "faces de bord  -> sommets" */

    dim_connect_fac_de_bord_som
      = renum->pos_connect_fac_de_bord_som_loc[ind_noeud]
                                              [renum->nbr_fac_de_bord_loc
                                                      [ind_noeud]] - 1 ;

    ecs_comm__ecrit_rubrique("nodfbr",
                             dim_connect_fac_de_bord_som,
                             renum->val_connect_fac_de_bord_som_loc[ind_noeud],
                             ECS_TYPE_ecs_int_t,
                             comm) ;


    /* critures supplmentaires pour le paralllisme */
    /*------------------------------------------------*/

    if (renum->nbr_dom > 1) {

      /* numros globaux des cellules */

      ecs_comm__ecrit_rubrique("maillage:data:cel:num",
                               renum->nbr_cel_loc[ind_noeud],
                               renum->num_glob_cel[ind_noeud],
                               ECS_TYPE_ecs_int_t,
                               comm) ;


      /* numros globaux des faces internes */

      ecs_comm__ecrit_rubrique("maillage:data:fac:num",
                               renum->nbr_fac_interne_loc[ind_noeud],
                               renum->num_glob_fac[ind_noeud],
                               ECS_TYPE_ecs_int_t,
                               comm) ;

      /* numros globaux des faces de bord */

      ecs_comm__ecrit_rubrique("maillage:data:fbr:num",
                               renum->nbr_fac_de_bord_loc[ind_noeud],
                               renum->num_glob_fbr[ind_noeud],
                               ECS_TYPE_ecs_int_t,
                               comm) ;

      /* numros globaux des sommets */

      ecs_comm__ecrit_rubrique("maillage:data:som:num",
                               renum->nbr_som_loc[ind_noeud],
                               renum->num_glob_som[ind_noeud],
                               ECS_TYPE_ecs_int_t,
                               comm) ;

    } /* Fin des critures supplmentaires paralllisme */


    /* critures supplmentaires pour la priodicit */
    /*-----------------------------------------------*/

    if (renum->nbr_per > 0) {

      ecs_int_t   type_perio ;
      ecs_int_t   nbr_couples ;
      ecs_int_t  *liste_joints ;
      char        nom_param[ECS_COMM_LNG_NOM_RUB + 1] ;


      /* Liste des paramtres de priodicit pour chacune d'entre elles */


      for (ind_per = 0 ; ind_per < renum->nbr_per ; ind_per++) {

        sprintf(nom_param, "%s%02d", "maillage:data:per:type:",
                liste_param_perio[ind_per].num_perio) ;

        type_perio = (ecs_int_t)(liste_param_perio[ind_per].type_perio) ;
        ecs_comm__ecrit_rubrique(nom_param,
                                 1,
                                 &type_perio,
                                 ECS_TYPE_ecs_int_t,
                                 comm) ;

        sprintf(nom_param, "%s%02d", "maillage:data:per:trans:",
                liste_param_perio[ind_per].num_perio) ;

        ecs_comm__ecrit_rubrique(nom_param,
                                 ECS_DIM_3,
                                 liste_param_perio[ind_per].translation,
                                 ECS_TYPE_ecs_real_t,
                                 comm) ;

        sprintf(nom_param, "%s%02d", "maillage:data:per:ptinv:",
                liste_param_perio[ind_per].num_perio) ;

        ecs_comm__ecrit_rubrique(nom_param,
                                 ECS_DIM_3,
                                 liste_param_perio[ind_per].point_inv,
                                 ECS_TYPE_ecs_real_t,
                                 comm) ;

        sprintf(nom_param, "%s%02d", "maillage:data:per:matrice:",
                liste_param_perio[ind_per].num_perio) ;

        ecs_comm__ecrit_rubrique(nom_param,
                                 ECS_DIM_3*ECS_DIM_3,
                                 liste_param_perio[ind_per].matrice,
                                 ECS_TYPE_ecs_real_t,
                                 comm) ;

        /* Informations pour la cration des joints priodiques */

        nbr_couples =   renum->idx_fac_perio_loc[ind_noeud][ind_per + 1]
                      - renum->idx_fac_perio_loc[ind_noeud][ind_per];

        liste_joints =   renum->liste_fac_perio_loc[ind_noeud]
                       + (renum->idx_fac_perio_loc[ind_noeud][ind_per] * 2);

        sprintf(nom_param, "%s%02d", "maillage:data:per:fac:",
                ind_per + 1) ;

        ecs_comm__ecrit_rubrique(nom_param,
                                 nbr_couples * 2,
                                 liste_joints,
                                 ECS_TYPE_ecs_int_t,
                                 comm) ;

        if (renum->nbr_dom > 1) {

          ecs_int_t  *dom_fac_perio
            =   (renum->num_dom_fac_perio[ind_noeud]
              + (renum->idx_fac_perio_loc[ind_noeud][ind_per]));

          sprintf(nom_param, "%s%02d", "maillage:data:per:dom:",
                  ind_per + 1) ;

          ecs_comm__ecrit_rubrique(nom_param,
                                   nbr_couples,
                                   dom_fac_perio,
                                   ECS_TYPE_ecs_int_t,
                                   comm) ;

        }

      }

    } /* Fin des critures supplmentaires pour la priodicit */


    /* Ecriture de la rubrique de fin  du bloc sur les donnes */
    /*---------------------------------------------------------*/

    ecs_comm__ecrit_rubrique("end_block:data",
                             0,
                             NULL,
                             ECS_TYPE_void,
                             comm) ;

    /* Ecriture de la rubrique de fin de fichier et fermeture */

    ecs_comm__ecrit_rubrique("EOF",
                             0,
                             NULL,
                             ECS_TYPE_void,
                             comm) ;


    /* Fermeture du fichier de communication */
    /*---------------------------------------*/

    comm = ecs_comm__termine(comm) ;

  }


  /* Libration de la mmoire */
  /*--------------------------*/

  if (tab_noms_groupes.nbr > 0) {

    BFT_FREE(pos_nom_grp) ;
    BFT_FREE(val_nom_grp) ;

    for (ind_grp = 0 ; ind_grp < tab_noms_groupes.nbr ; ind_grp++)
      BFT_FREE(tab_noms_groupes.val[ind_grp]) ;
    BFT_FREE(tab_noms_groupes.val) ;
    tab_noms_groupes.nbr = 0 ;

  }

  if (tab_fam_cel.nbr != 0)
    BFT_FREE(tab_fam_cel.val);

  if (tab_fam_fac_de_bord.nbr != 0)
    BFT_FREE(tab_fam_fac_de_bord.val);

  if (tab_propr_fam.nbr != 0)
    BFT_FREE(tab_propr_fam.val);

}


/*============================================================================
 *                              Fonctions privees
 *============================================================================*/


/*----------------------------------------------------------------------------
 *  Fonction qui crit les composantes d'un champ index par une liste
 *  globale (indice de 0  n-1)  laquelle on applique une liste locale
 *  (issue du dcoupage de domaines, et indice de 1  m)
 *----------------------------------------------------------------------------*/

static void ecs_loc_entmail_ncs__lis_champ
(
       ecs_champ_t     *const        this_champ,
       ecs_int_t       *const        liste,
       ecs_int_t       *const *const liste_loc,
 const ecs_int_t                     nbr_liste_loc,
 const char            *const        nom_rub,
       ecs_comm_t      *const        comm,
 const ecs_int_t                     ind_noeud
)
{

  ecs_int_t         iloc ;

  ecs_int_t     *   liste_prod ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/

  /* Cas sans liste globale */

  if (liste == NULL) {

    if (liste_loc == NULL)

      ecs_champ_comm__ecr(this_champ,
                          nom_rub,
                          comm) ;

    else

      ecs_champ_comm__liste_vec(this_champ,
                                liste_loc[ind_noeud],
                                nbr_liste_loc,
                                1,
                                nom_rub,
                                comm) ;

  }

  /* Cas avec liste globale */

  else {

    if (liste_loc == NULL)

      liste_prod = liste ;

    else {

      BFT_MALLOC(liste_prod, nbr_liste_loc, ecs_int_t) ;

      for (iloc = 0 ; iloc < nbr_liste_loc ; iloc++)
        liste_prod[iloc] = liste[liste_loc[ind_noeud][iloc] - 1] ;

    }

    ecs_champ_comm__liste_vec(this_champ,
                              liste_prod,
                              nbr_liste_loc,
                              0,
                              nom_rub,
                              comm) ;

    if (liste_loc != NULL)

      BFT_FREE(liste_prod) ;

  }

}


/*----------------------------------------------------------------------------
 *  Fonction qui crit les valeurs d'un tableau d'entiers global auquel on
 *  applique une liste locale (issue du dcoupage de domaines, et indice
 *  de 1  m)
 *----------------------------------------------------------------------------*/

static void ecs_loc_entmail_ncs__tab_int
(
       ecs_int_t       *const        val_int,
       ecs_int_t       *const *const liste_loc,
 const ecs_int_t                     nbr_liste_loc,
 const char            *const        nom_rub,
       ecs_comm_t      *const        comm,
 const ecs_int_t                     ind_noeud
)
{

  ecs_int_t         iloc ;

  ecs_int_t     *   val_loc ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/

  if (liste_loc == NULL)

    val_loc = val_int ;

  else {

    BFT_MALLOC(val_loc, nbr_liste_loc, ecs_int_t) ;

    for (iloc = 0 ; iloc < nbr_liste_loc ; iloc++)
      val_loc[iloc] = val_int[liste_loc[ind_noeud][iloc] - 1] ;

  }

  ecs_comm__ecrit_rubrique(nom_rub,
                           nbr_liste_loc,
                           val_loc,
                           ECS_TYPE_ecs_int_t,
                           comm) ;

  if (liste_loc != NULL)

    BFT_FREE(val_loc) ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui renvoie la liste des champs "famille"
 *   pour toutes les entits de maillage
 *  et qui dtermine :
 *  - le nombre   de familles  transfrer au Noyau
 *  - le nombre maximal de  proprits des familles
 *  - les numros de famille  des cellules
 *  - les numros de famille  des faces de bord
 *  - les numros  affecter aux groupes (et inversement, les noms par numro)
 *  - les proprits des familles
 *----------------------------------------------------------------------------*/

static ecs_champ_t * * ecs_loc_entmail_ncs__cree_fam
(
       size_t                          nbr_cel,
 const ecs_int_t         *const        liste_fac_de_bord,
       size_t                          nbr_fac_de_bord,
       ecs_entmail_t     *const *const vect_entmail,
       ecs_famille_t     *const *const vect_fam_tete,
       int               *const        nbr_fam,
       int               *const        nbr_max_propr,
       ecs_tab_int_t     *const        tab_fam_cel,
       ecs_tab_int_t     *const        tab_fam_fac_de_bord,
       ecs_tab_int_t     *const        tab_propr_fam,
       ecs_tab_char_t    *const        noms_groupes
)
{

  ecs_bool_t      bool_cree_fam_par_defaut ;

  size_t          cpt_fam ;
  size_t          cpt_prop ;
  size_t          decal_fam_ent ;
  size_t          icel ;
  size_t          ient ;
  size_t          ifac ;
  ecs_int_t       ifam ;
  size_t          igrp ;
  size_t          ipropr ;
  size_t          nbr_cel_avec_fam_defaut ;
  size_t          nbr_fac_de_bord_avec_fam_defaut ;
  ecs_int_t       nbr_fam_tot ;
  ecs_int_t       nbr_loc_propr ;
  ecs_int_t       num_fam_defaut ;

  ecs_int_t     * cpt_elt_ent_fam ;
  ecs_int_t   * * cpt_elt_fam_ent ;
  ecs_int_t       nbr_fam_ent[ECS_ENTMAIL_FIN] ;

  ecs_int_t       left ;
  ecs_int_t       right ;
  ecs_int_t       mid ;
  ecs_int_t       num_grp_loc ;
  char          * nom_grp_loc ;

  ecs_champ_t   * * vect_champ_famille ;

  ecs_famille_t   * fam_tete_ent ;
  ecs_famille_t   * fam_tete_globale ;

  ecs_tab_int_t     tab_nbr_cel_fam ;
  ecs_tab_int_t     tab_nbr_fac_de_bord_fam ;
  ecs_tab_int_t   * tab_propr_ide_fam_ent[ECS_ENTMAIL_FIN] ;
  ecs_tab_char_t  * tab_propr_nom_fam_ent[ECS_ENTMAIL_FIN] ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  noms_groupes->nbr  = 0 ;
  noms_groupes->val = NULL ;


  BFT_MALLOC(vect_champ_famille, ECS_ENTMAIL_FIN, ecs_champ_t *) ;


  nbr_fam_tot = 0 ;

  for (ient =  (ecs_int_t)ECS_ENTMAIL_CEL ;
       ient >= (ecs_int_t)ECS_ENTMAIL_FAC ; ient--) {

    assert(vect_entmail[ient] != NULL) ;

    if (vect_entmail[ient]->champ[ECS_CHAMP_ATT] != NULL) {

      vect_champ_famille[ient] =
        ecs_champ_chaine__trouve_nom(vect_entmail[ient]->champ[ECS_CHAMP_ATT],
                                     ECS_CHAMP_NOM_FAMILLE) ;

    }
    else {

      vect_champ_famille[ient] = NULL ;

    }


    if (vect_champ_famille[ient] != NULL) {

      nbr_fam_ent[ient] = ecs_famille_chaine__ret_nbr(vect_fam_tete[ient]) ;
      nbr_fam_tot += nbr_fam_ent[ient] ;

    }
    else {

      assert(vect_fam_tete[ient] == NULL) ;

      nbr_fam_ent[ient] = 0 ;

    }

  } /* Fin : boucle sur les entits de maillage */


  if (nbr_fam_tot != 0) {
    tab_nbr_cel_fam.nbr = nbr_fam_tot ;
    BFT_MALLOC(tab_nbr_cel_fam.val,
               tab_nbr_cel_fam.nbr, ecs_int_t) ;
    tab_nbr_fac_de_bord_fam.nbr = nbr_fam_tot ;
    BFT_MALLOC(tab_nbr_fac_de_bord_fam.val,
               tab_nbr_fac_de_bord_fam.nbr, ecs_int_t) ;
  }

  num_fam_defaut = nbr_fam_tot + 1 ;


  /* Numros des familles des cellules */
  /*-----------------------------------*/

  if (vect_champ_famille[ECS_ENTMAIL_CEL] != NULL && nbr_cel != 0) {

    *tab_fam_cel
      = ecs_champ_att__fam_liste_elt(vect_champ_famille[ECS_ENTMAIL_CEL],
                                     NULL,
                                     nbr_cel,
                                     ECS_TRUE,
                                     num_fam_defaut,
                                     &tab_nbr_cel_fam,
                                     &nbr_cel_avec_fam_defaut) ;

  }
  else {

    BFT_MALLOC(tab_fam_cel->val, nbr_cel, ecs_int_t) ;
    tab_fam_cel->nbr = nbr_cel ;

    for (icel = 0 ; icel < nbr_cel ; icel++)
      tab_fam_cel->val[icel] = num_fam_defaut ;

    for (ifam = 0 ; ifam < nbr_fam_tot ; ifam++)
      tab_nbr_cel_fam.val[ifam] = 0 ;

    nbr_cel_avec_fam_defaut = nbr_cel ;

  }


  /* Numros des familles des faces de bord */
  /*----------------------------------------*/

  if (vect_champ_famille[ECS_ENTMAIL_FAC] != NULL && nbr_fac_de_bord != 0) {

    nbr_fac_de_bord_avec_fam_defaut = 0 ;

    *tab_fam_fac_de_bord
      = ecs_champ_att__fam_liste_elt(vect_champ_famille[ECS_ENTMAIL_FAC],
                                     liste_fac_de_bord,
                                     nbr_fac_de_bord,
                                     ECS_TRUE,
                                     num_fam_defaut,
                                     &tab_nbr_fac_de_bord_fam,
                                     &nbr_fac_de_bord_avec_fam_defaut) ;

  }
  else {

    BFT_MALLOC(tab_fam_fac_de_bord->val, nbr_fac_de_bord, ecs_int_t) ;
    tab_fam_fac_de_bord->nbr = nbr_fac_de_bord ;

    for (ifac = 0 ; ifac < nbr_fac_de_bord ; ifac++)
      tab_fam_fac_de_bord->val[ifac] = num_fam_defaut ;

    for (ifam = 0 ; ifam < nbr_fam_tot ; ifam++)
      tab_nbr_fac_de_bord_fam.val[ifam] = 0 ;

    nbr_fac_de_bord_avec_fam_defaut = nbr_fac_de_bord ;

  }


  /*-------------------------------------------------------*/
  /* Dtermination du  nombre  de  proprits des familles */
  /* Dtermination             des proprits des familles */
  /*-------------------------------------------------------*/


  *nbr_max_propr = 1 ;

  for (ient =  (ecs_int_t)ECS_ENTMAIL_CEL ;
       ient >= (ecs_int_t)ECS_ENTMAIL_FAC ; ient--) {

    if (vect_fam_tete[ient] != NULL) {

      tab_propr_ide_fam_ent[ient]
        = ecs_famille_chaine__ret_ide(vect_fam_tete[ient]) ;

      tab_propr_nom_fam_ent[ient]
        = ecs_famille_chaine__ret_nom(vect_fam_tete[ient]) ;

      /* Calcul du nombre maximal de proprits */
      for (ifam = 0 ; ifam < nbr_fam_ent[ient] ; ifam++) {
        nbr_loc_propr =   tab_propr_ide_fam_ent[ient][ifam].nbr
                        + tab_propr_nom_fam_ent[ient][ifam].nbr ;
        *nbr_max_propr = ECS_MAX(*nbr_max_propr, nbr_loc_propr) ;
      }

    }

  }

  /* Mise  jour du tableau des numros locaux des groupes */

  ecs_loc_entmail_ncs__renum_groupes(vect_fam_tete,
                                     nbr_fam_ent,
                                     tab_propr_nom_fam_ent,
                                     noms_groupes) ;

  if (nbr_fam_tot != 0) {

    BFT_MALLOC(cpt_elt_fam_ent, nbr_fam_tot, ecs_int_t *) ;
    for (ifam = 0 ; ifam < nbr_fam_tot ; ifam++)
      BFT_MALLOC(cpt_elt_fam_ent[ifam], ECS_ENTMAIL_FIN, ecs_int_t) ;

    for (ient =  (ecs_int_t)ECS_ENTMAIL_CEL ;
         ient >= (ecs_int_t)ECS_ENTMAIL_FAC ; ient--) {

      cpt_elt_ent_fam = NULL ;

      if (vect_champ_famille[ient] != NULL) {

        /* Nombre d'lments par entit et par famille */
        cpt_elt_ent_fam
          = ecs_champ_att__ret_nbr_elt_val(vect_champ_famille[ient],
                                           nbr_fam_tot) ;

      }
      else {

        BFT_MALLOC(cpt_elt_ent_fam, nbr_fam_tot, ecs_int_t) ;
        for (ifam = 0 ; ifam < nbr_fam_tot ; ifam++)
          cpt_elt_ent_fam[ifam] = 0 ;

      }

      for (ifam = 0 ; ifam < nbr_fam_tot ; ifam++)
        cpt_elt_fam_ent[ifam][ient] = cpt_elt_ent_fam[ifam] ;

      if (cpt_elt_ent_fam != NULL)
        BFT_FREE(cpt_elt_ent_fam) ;

    }

    for (ient =  (ecs_int_t)ECS_ENTMAIL_SOM ;
         ient <= (ecs_int_t)ECS_ENTMAIL_ARE ; ient++) {

      for (ifam = 0 ; ifam < nbr_fam_tot ; ifam++)
        cpt_elt_fam_ent[ifam][ient] = 0 ;

    }

  }


  /* Affichage des dfinitions des familles */
  /*----------------------------------------*/

  bft_printf(_("\n\n"
               "Definition of face and cell families\n"
               "------------------------------------\n\n")) ;

  if (nbr_fac_de_bord_avec_fam_defaut > 0) {

    /* On a du attribuer une famille par dfaut  certaines faces de bord */

    ecs_warn() ;
    bft_printf
      (_("%d boundary faces from a total of %d have no color\n"
         "and belong to no group...\n"
         "A default family is assigned to those faces.\n\n"),
       (int)nbr_fac_de_bord_avec_fam_defaut, (int)nbr_fac_de_bord) ;

  }

  if (nbr_cel_avec_fam_defaut > 0) {

    /* On a du attribuer une famille par dfaut  certaines cellules */

    ecs_warn() ;
    bft_printf
      (_("%d cells from a total of %d have no color\n"
         "and belong to no group...\n"
         "A default family is assigned to those cells.\n\n"),
       (int)nbr_cel_avec_fam_defaut, (int)nbr_cel) ;

  }


  cpt_fam = 0 ;

  if (   nbr_fac_de_bord_avec_fam_defaut > 0
      || nbr_cel_avec_fam_defaut         > 0)
    bool_cree_fam_par_defaut = ECS_TRUE ;
  else
    bool_cree_fam_par_defaut = ECS_FALSE ;


  for (ient =  (ecs_int_t)ECS_ENTMAIL_CEL ;
       ient >= (ecs_int_t)ECS_ENTMAIL_FAC ; ient--) {

    for (ifam = 0 ; ifam < nbr_fam_ent[ient] ; ifam++) {

      ecs_famille_chaine__affiche(ifam + 1,
                                  vect_fam_tete[ient]) ;

      ecs_loc_entmail_ncs__aff_nbr_ent
        (cpt_elt_fam_ent[cpt_fam][(ecs_int_t) ECS_ENTMAIL_CEL],
         (cpt_elt_fam_ent[cpt_fam][(ecs_int_t) ECS_ENTMAIL_FAC]
          - tab_nbr_fac_de_bord_fam.val[cpt_fam]),
         tab_nbr_fac_de_bord_fam.val[cpt_fam]) ;

      cpt_fam++ ;

    }

  }

  if (nbr_fam_tot != 0) {

    for (ifam = 0 ; ifam < nbr_fam_tot ; ifam++)
      BFT_FREE(cpt_elt_fam_ent[ifam]) ;
    BFT_FREE(cpt_elt_fam_ent) ;

    if (tab_nbr_cel_fam.nbr != 0)
      BFT_FREE(tab_nbr_cel_fam.val) ;

    if (tab_nbr_fac_de_bord_fam.nbr != 0)
      BFT_FREE(tab_nbr_fac_de_bord_fam.val) ;

  }

  /* Affichage famille par dfaut si ncessaire */

  if (bool_cree_fam_par_defaut == ECS_TRUE) {

    bft_printf("  %s %" ECS_FORMAT_ecs_int_t "\n", _("Family"),
               cpt_fam + 1) ;

    bft_printf("  %*s%s\n", strlen(_("Family")) + 1, "",
               _("Default family")) ;
    bft_printf("  %*s%s\n", strlen(_("Family")) + 1, "",
               _("(with no color nor group)")) ;

    /* Comme on ne traite pas les faces internes, inutile d'affecter la */
    /* famille par dfaut  celles n'appartenant pas  une famille > 0  */

    ecs_loc_entmail_ncs__aff_nbr_ent
      (nbr_cel_avec_fam_defaut,
       0,
       nbr_fac_de_bord_avec_fam_defaut) ;


  } /* Fin : si affichage de la famille par dfaut */


  /* Affichage des couleurs et des groupes en fonction des familles */
  /*----------------------------------------------------------------*/

  if (nbr_fam_tot != 0) {

    /* Affichage du titre */

    bft_printf(_("\n\nDefinition of colors and groups"
                 " based on families\n"
                 "-------------------------------"
                 "------------------\n\n")) ;


    fam_tete_globale = NULL ;

    for (ient =  (ecs_int_t)ECS_ENTMAIL_CEL ;
         ient >= (ecs_int_t)ECS_ENTMAIL_FAC ; ient--) {

      fam_tete_ent = ecs_famille_chaine__copie(vect_fam_tete[ient]) ;

      ecs_famille_chaine__ajoute(&fam_tete_globale,
                                 fam_tete_ent) ;

    } /* Fin : boucle sur les entits de maillages */


    ecs_famille_chaine__aff_fam_att(fam_tete_globale) ;


    ecs_famille_chaine__detruit(&fam_tete_globale) ;

    /* Affichage des numros et noms de groupes si lieu */

    if (noms_groupes->nbr > 0) {

      bft_printf(_("\n\n"
                   "Groups numbering\n"
                   "----------------\n\n")) ;

      for (igrp   = 0 ; igrp < noms_groupes->nbr ; igrp++)
        bft_printf(_("  Group %3d : \"%s\"\n"),
                   igrp + 1, noms_groupes->val[igrp]);

    }

  } /* Fin : s'il y a au moins une famille */


  *nbr_fam = cpt_fam ;
  if (bool_cree_fam_par_defaut == ECS_TRUE)
    *nbr_fam += 1 ;


  /* Concatnation des proprits des familles */
  /*-------------------------------------------*/

  /* Pas de proprits pour la famille 0 */

  tab_propr_fam->nbr = (*nbr_fam) * (*nbr_max_propr) ;
  BFT_MALLOC(tab_propr_fam->val, tab_propr_fam->nbr, ecs_int_t);

  /* Liste des proprits des familles  */

  /* Attention : pour le noyau on n'crit pas les proprits par famille */
  /*             mais on crit les i-emes proprits de toutes           */

  decal_fam_ent = 0 ;

  for (ient =  (ecs_int_t)ECS_ENTMAIL_CEL ;
       ient >= (ecs_int_t)ECS_ENTMAIL_FAC ; ient--) {

    for (ifam = 0 ; ifam < nbr_fam_ent[ient] ; ifam++) {

      cpt_prop = 0 ;

      /* Couleurs */

      for (ipropr = 0 ;
           ipropr < tab_propr_ide_fam_ent[ient][ifam].nbr ; ipropr++) {

        if (tab_propr_ide_fam_ent[ient][ifam].val[ipropr] >= 0) {
          tab_propr_fam->val[decal_fam_ent + cpt_prop * (*nbr_fam) + ifam]
            = tab_propr_ide_fam_ent[ient][ifam].val[ipropr] ;
          cpt_prop++ ;
        }

      }

      /* Groupes */

      for (ipropr = 0 ;
           ipropr < tab_propr_nom_fam_ent[ient][ifam].nbr ; ipropr++) {

        /* Recherche par dichotomie dans le tableau des groupes */

        nom_grp_loc = tab_propr_nom_fam_ent[ient][ifam].val[ipropr] ;

        left  = 0 ;
        right = noms_groupes->nbr - 1 ;

        while ((right - left) > 1) {

          mid = (right + left) / 2 ;  /* Division entire */
          if (strcmp(nom_grp_loc, noms_groupes->val[mid]) <= 0)
            right = mid ;
          else
            left  = mid ;

        }

        if (strcmp(nom_grp_loc, noms_groupes->val[right]) == 0)
          num_grp_loc = -(right + 1) ;
        else if (strcmp(nom_grp_loc, noms_groupes->val[left]) == 0)
          num_grp_loc = -(left + 1) ;
        else {
          assert(0) ;
          num_grp_loc = 0 ;
        }

        tab_propr_fam->val[decal_fam_ent + cpt_prop * (*nbr_fam) + ifam]
          = num_grp_loc ;

        cpt_prop++ ;

      }

      /* On complte par des '0' */

      for (ipropr = cpt_prop ;
           ipropr < (size_t)(*nbr_max_propr) ; ipropr++)
        tab_propr_fam->val[decal_fam_ent + ipropr * (*nbr_fam) + ifam] = 0 ;

    }

    decal_fam_ent += nbr_fam_ent[ient] ;

  }

  if (bool_cree_fam_par_defaut == ECS_TRUE) {

    /* On remplit la famille par dfaut avec des '0' */

    for (ipropr = 0 ; ipropr < (size_t)(*nbr_max_propr) ; ipropr++)
      tab_propr_fam->val[decal_fam_ent + ipropr * (*nbr_fam)] = 0 ;

  }

  for (ient =  (ecs_int_t)ECS_ENTMAIL_CEL ;
       ient >= (ecs_int_t)ECS_ENTMAIL_FAC ; ient--) {
    for (ifam = 0 ; ifam < nbr_fam_ent[ient] ; ifam++) {
      BFT_FREE(tab_propr_ide_fam_ent[ient][ifam].val) ;
      BFT_FREE(tab_propr_nom_fam_ent[ient][ifam].val) ;
    }
    if (nbr_fam_ent[ient] != 0) {
      BFT_FREE(tab_propr_ide_fam_ent[ient]) ;
      BFT_FREE(tab_propr_nom_fam_ent[ient]) ;
    }
  }


  return vect_champ_famille ;


}


/*----------------------------------------------------------------------------
 *  Fonction qui affiche le nombre de cellules, faces internes, et faces
 *  de bord fournies en argument.
 *----------------------------------------------------------------------------*/

static void ecs_loc_entmail_ncs__aff_nbr_ent
(
 const ecs_int_t         nbr_elt_cel,
 const ecs_int_t         nbr_elt_fac_interne,
 const ecs_int_t         nbr_elt_fac_de_bord
)
{

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/

  size_t lng_var_nbr ;

  lng_var_nbr = strlen(_("Number of cells")) ;
  lng_var_nbr = ECS_MAX(lng_var_nbr, strlen(_("Number of internal faces"))) ;
  lng_var_nbr = ECS_MAX(lng_var_nbr, strlen(_("Number of boundary faces"))) ;

  if (nbr_elt_cel > 0) {
    bft_printf("  ") ;
    ecs_print_padded_str(_("Number of cells"), lng_var_nbr) ;
    bft_printf(" : %*ld\n", ECS_LNG_AFF_ENT, (long)nbr_elt_cel) ;
  }

  if (nbr_elt_fac_interne > 0) {
    bft_printf("  ") ;
    ecs_print_padded_str(_("Number of internal faces"), lng_var_nbr) ;
    bft_printf(" : %*ld\n", ECS_LNG_AFF_ENT, (long)nbr_elt_fac_interne) ;
  }

  if (nbr_elt_fac_de_bord > 0) {
    bft_printf("  ") ;
    ecs_print_padded_str(_("Number of boundary faces"), lng_var_nbr) ;
    bft_printf(" : %*ld\n", ECS_LNG_AFF_ENT, (long)nbr_elt_fac_de_bord) ;
  }

}


/*----------------------------------------------------------------------------
 *  Fonction qui (re)numrote les groupes et construit un tableau contenant
 *   les noms ordonns de ces groupes. Le tableau de chanes de caractres
 *   noms_groupes doit initialement tre vide.
 *----------------------------------------------------------------------------*/

static void ecs_loc_entmail_ncs__renum_groupes
(
       ecs_famille_t   *const *const vect_fam_tete,
 const ecs_int_t                     nbr_fam_ent[],
       ecs_tab_char_t         *const tab_propr_nom_fam_ent[],
       ecs_tab_char_t         *const noms_groupes
)
{

  size_t          ient ;
  ecs_int_t       ifam ;
  size_t          iloc ;
  size_t          ind ;

  size_t          cpt_nom_tot ;

  ecs_tab_int_t   tab_renum ;

  ecs_tab_char_t  tab_nom_cat ;
  ecs_tab_char_t  tab_nom_trie ;
  ecs_tab_char_t  tab_nom_cpct ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  noms_groupes->nbr = 0 ;
  noms_groupes->val = NULL ;


  /* Comptage */

  cpt_nom_tot = 0 ;

  for (ient =  (ecs_int_t)ECS_ENTMAIL_CEL ;
       ient >= (ecs_int_t)ECS_ENTMAIL_FAC ; ient--) {

    if (vect_fam_tete[ient] != NULL) {

      for (ifam = 0 ; ifam < nbr_fam_ent[ient] ; ifam++)
        cpt_nom_tot += tab_propr_nom_fam_ent[ient][ifam].nbr ;

    }

  }

  tab_nom_cat.nbr = cpt_nom_tot ;
  BFT_MALLOC(tab_nom_cat.val, tab_nom_cat.nbr, char *) ;


  /* Prparation */

  cpt_nom_tot = 0 ;

  for (ient =  (ecs_int_t)ECS_ENTMAIL_CEL ;
       ient >= (ecs_int_t)ECS_ENTMAIL_FAC ; ient--) {

    if (vect_fam_tete[ient] != NULL) {

      for (ifam = 0 ; ifam < nbr_fam_ent[ient] ; ifam++) {

        for (iloc = 0 ; iloc < tab_propr_nom_fam_ent[ient][ifam].nbr ; iloc++) {

          tab_nom_cat.val[cpt_nom_tot]
            = tab_propr_nom_fam_ent[ient][ifam].val[iloc] ;

          cpt_nom_tot++ ;

        }

      }

    }

  }


  /* Tri et renumrotation */

  tab_renum.nbr = tab_nom_cat.nbr ;
  BFT_MALLOC(tab_renum.val, tab_renum.nbr,  ecs_int_t) ;

  tab_nom_trie = ecs_tab_char__trie_et_renvoie(tab_nom_cat,
                                               tab_renum) ;

  BFT_FREE(tab_nom_cat.val) ;
  BFT_FREE(tab_renum.val) ;

  tab_nom_cpct = ecs_tab_char__compacte(tab_nom_trie) ;

  BFT_FREE(tab_nom_trie.val) ;


  /* Recopie dans le tableau des noms de groupes ; */

  noms_groupes->nbr = tab_nom_cpct.nbr ;
  BFT_MALLOC(noms_groupes->val, noms_groupes->nbr, char *) ;

  for (ind = 0 ; ind < tab_nom_cpct.nbr ; ind++) {

    BFT_MALLOC(noms_groupes->val[ind], strlen(tab_nom_cpct.val[ind]) + 1, char) ;

    strcpy(noms_groupes->val[ind], tab_nom_cpct.val[ind]) ;

  }

  BFT_FREE(tab_nom_cpct.val) ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui rordonne la connectivit faces internces -> cellules
 *   pour le Noyau ;
 *
 *  La liste des faces priodiques est code de la manire suivante
 *    (avec numros i, j en numrotation 1  n) :
 *    [i,j,n; ...] avec i->j ou j->i (i>j) et n le numro de la
 *    priodicit prcd d'un signe correspondant au sens  appliquer
 *----------------------------------------------------------------------------*/

static void ecs_loc_entmail_ncs__ord_fac
(
       size_t              nbr_fac,
 const ecs_tab_int_t       liste_fac_perio,
       ecs_int_t  *const   val_connect_fac_cel
)
{

  size_t     ind_fac ;
  size_t     iloc ;
  ecs_int_t  icel ;
  ecs_int_t  ifac ;
  ecs_int_t  icel_1 ;
  ecs_int_t  icel_2 ;
  ecs_int_t  ifac_1 ;
  ecs_int_t  ifac_2 ;

  size_t      nbr_fac_perio = 0 ;
  ecs_int_t  *cell_dist = NULL ;

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  assert(val_connect_fac_cel != NULL) ;

  /* Marquage pour les faces priodiques des cellules distantes associes ;
      ce stade, val_connect_fac_cel est sign, et la cellule "distante"
     via la priodicit est en 2me position, par construction */

  if (liste_fac_perio.nbr > 0) {

    nbr_fac_perio = liste_fac_perio.nbr / 3 ;

    BFT_MALLOC(cell_dist, nbr_fac_perio * 2, ecs_int_t) ;

    for (ind_fac = 0 ; ind_fac < nbr_fac_perio ; ind_fac++) {

      ifac_1 = liste_fac_perio.val[ind_fac*3]     - 1 ;
      ifac_2 = liste_fac_perio.val[ind_fac*3 + 1] - 1 ;

      cell_dist[ind_fac*2]     = val_connect_fac_cel[ifac_1*2 + 1] ;
      cell_dist[ind_fac*2 + 1] = val_connect_fac_cel[ifac_2*2 + 1] ;

    }

  }


  /* Pour les faces internes, il suffit de permuter les cellules */
  /*-------------------------------------------------------------*/

  for (ind_fac = 0 ; ind_fac < nbr_fac ; ind_fac++) {

    /* Une face interne apparat avec un signe diffrent
       pour chaque cellule  laquelle elle appartient ;
       on ordonnne val_connect_fac_cel de manire  ce que la normale
       de la face aille de la premire  la seconde cellule, et on met
       cette connectivit  la valeur absolue */

    if (val_connect_fac_cel[ind_fac*2] < 0) {

      assert(val_connect_fac_cel[ind_fac*2 + 1] > 0) ;

      icel_1 = - val_connect_fac_cel[ind_fac*2] ;
      icel_2 =   val_connect_fac_cel[ind_fac*2 + 1] ;

      val_connect_fac_cel[ind_fac*2]     = icel_2 ;
      val_connect_fac_cel[ind_fac*2 + 1] = icel_1 ;

    }

    else {

      assert(val_connect_fac_cel[ind_fac*2 + 1] < 0) ;

      val_connect_fac_cel[ind_fac*2 + 1] = - val_connect_fac_cel[ind_fac*2 + 1] ;

    }

  } /* Fin de la boucle sur les faces internes */


  /* Marquage des cellules distantes associes en cas de priodicit ;
     on affecte un signe ngatif aux cellules distantes */

  if (nbr_fac_perio > 0) {

    for (ind_fac = 0 ; ind_fac < nbr_fac_perio ; ind_fac++) {

      for (iloc = 0 ; iloc < 2 ; iloc++) {

        ifac = liste_fac_perio.val[ind_fac*3 + iloc] - 1 ;
        icel = cell_dist[ind_fac*2 + iloc] ;

        if (icel > 0) { /* On a du permuter les cellules autour de ifac */
          assert(val_connect_fac_cel[ifac*2] == icel);
          val_connect_fac_cel[ifac*2] = - icel;
        }
        else {
          assert(val_connect_fac_cel[ifac*2+1] == - icel);
          val_connect_fac_cel[ifac*2+1] = icel;
        }

      }

    }

    BFT_FREE(cell_dist) ;

  }

}


/*----------------------------------------------------------------------------
 *  Fonction qui rordonne la connectivit des faces de bord pour le noyau
 *----------------------------------------------------------------------------*/

static void ecs_loc_entmail_ncs__ord_fbr
(
       size_t              nbr_fbr,
       ecs_int_t  *const   val_connect_fbr_cel,
 const ecs_size_t *const   pos_connect_fbr_som,
       ecs_int_t  *const   val_connect_fbr_som
)
{

  size_t     ind_fac ;
  size_t     ipos_1 ;
  size_t     ipos_2 ;
  ecs_int_t  isom_1 ;
  ecs_int_t  isom_2 ;


  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  if (val_connect_fbr_cel != NULL && val_connect_fbr_som != NULL) {

    /* Permutation du sens de parcours des polygones */
    /*-----------------------------------------------*/

    assert(pos_connect_fbr_som != NULL) ;

    for (ind_fac = 0 ; ind_fac < nbr_fbr ; ind_fac++) {

      if (val_connect_fbr_cel[ind_fac] < 0) {

        ipos_1 = pos_connect_fbr_som[ind_fac] - 1 ;
        ipos_2 = pos_connect_fbr_som[ind_fac + 1] - 2 ;

        while (ipos_2 > ipos_1) {

          isom_1 = val_connect_fbr_som[ipos_1] ;
          isom_2 = val_connect_fbr_som[ipos_2] ;

          val_connect_fbr_som[ipos_1] = isom_2 ;
          val_connect_fbr_som[ipos_2] = isom_1 ;

          assert(ipos_2 > 0);

          ipos_1 += 1 ;
          ipos_2 -= 1 ;

        }

        /* Mise  la valeur absolue de la connectivit */

        val_connect_fbr_cel[ind_fac] = - val_connect_fbr_cel[ind_fac] ;

      }

    } /* Fin de la boucle sur les faces */

  }

}
