/*============================================================================
 *  Dfinitions des fonctions
 *   associes  la structure `ecs_maillage_t' dcrivant un maillage
 *   et ralisant les sorties pour l'interfacage avec le Noyau
 *============================================================================*/

/*
                      Code_Saturne version 1.3
                      ------------------------

  This file is part of the Code_Saturne Preprocessor, element of the
  Code_Saturne CFD tool.

  Copyright (C) 1999-2007 EDF S.A., France

  contact: saturne-support@edf.fr

  The Code_Saturne Preprocessor is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  The Code_Saturne Preprocessor is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the Code_Saturne Preprocessor; if not, write to the
  Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor,
  Boston, MA  02110-1301  USA
*/


/*============================================================================
 *                                 Visibilit
 *============================================================================*/

#include "ecs_config.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' librairie standard C ou BFT
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <string.h>

#include <bft_error.h>
#include <bft_mem.h>
#include <bft_printf.h>


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Utilitaire"
 *----------------------------------------------------------------------------*/

#include "ecs_def.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage global "Post-Traitement"
 *----------------------------------------------------------------------------*/

#include "ecs_post.h"
#include "ecs_post_ens.h"

#if defined(HAVE_MED)
#include "ecs_post_med.h"
#endif /* HAVE_MED */

#include "ecs_comm.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles des paquetages visibles
 *----------------------------------------------------------------------------*/

#include "ecs_entmail.h"
#include "ecs_entmail_ncs.h"
#include "ecs_entmail_post.h"
#include "ecs_maillage.h"
#include "ecs_param_perio_glob.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' visibles du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_renum.h"
#include "ecs_renum_priv.h"


/*----------------------------------------------------------------------------
 *  Fichier  `include' du  paquetage courant associe au fichier courant
 *----------------------------------------------------------------------------*/

#include "ecs_maillage_ncs.h"


/*----------------------------------------------------------------------------
 *  Fichiers `include' prives   du  paquetage courant
 *----------------------------------------------------------------------------*/

#include "ecs_maillage_priv.h"


/*============================================================================
 *                       Prototypes de fonctions privees
 *============================================================================*/


/*============================================================================
 *                             Fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 *  Fonction qui cre la connectivit pour le Noyau
 *----------------------------------------------------------------------------*/

void ecs_maillage_ncs__renum_init
(
 ecs_maillage_t  * const  maillage ,
 ecs_renum_t     * const  renum
)
{

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  /* Si on est pas en dimension 3, on sort */

  if (maillage->dim_e != ECS_DIM_3) return ;

  ecs_entmail_ncs__renum_init(maillage->entmail,
                              renum) ;

}


/*----------------------------------------------------------------------------
 *  Fonction qui crit les donnes dans le fichier d'interface pour le noyau
 *----------------------------------------------------------------------------*/

void ecs_maillage_ncs__ecr
(
 ecs_comm_typ_t            comm_typ_ncs_e     ,
 ecs_int_t                 echo_comm          ,
 ecs_maillage_t     *const maillage           ,
 ecs_renum_t        *const renum              ,
 ecs_param_perio_t  *const liste_param_perio
)
{

  /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/


  /* Si on est pas en dimension 3 ,                      */
  /* on n'crit pas le fichier d'interface pour le Noyau */

  if (maillage->dim_e != ECS_DIM_3) return ;


  ecs_entmail_ncs__ecr(comm_typ_ncs_e,
                       echo_comm,
                       maillage->entmail,
                       maillage->famille,
                       (ecs_int_t)maillage->dim_e,
                       liste_param_perio,
                       renum) ;

}


/*============================================================================
 *                              Fonctions privees
 *============================================================================*/
